/**--------------------------------------------------------------------------**\
					==========================
					 y_colours - X11 colours!
					==========================
Description:
	This holds the colour information that used to be part of the text system
	but which is far more useful than just for text.  This now supports the full
	set of X11 colours, both by name and by definition.  You can also define
	your own if you so choose (up to 32 - should be enough given that this
	includes the X11 colours).
Legal:
	Version: MPL 1.1
	
	The contents of this file are subject to the Mozilla Public License Version 
	1.1 (the "License"); you may not use this file except in compliance with 
	the License. You may obtain a copy of the License at 
	http://www.mozilla.org/MPL/
	
	Software distributed under the License is distributed on an "AS IS" basis,
	WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
	for the specific language governing rights and limitations under the
	License.
	
	The Original Code is the YSI colours include.
	
	The Initial Developer of the Original Code is Alex "Y_Less" Cole.
	Portions created by the Initial Developer are Copyright (C) 2011
	the Initial Developer. All Rights Reserved.
	
	Contributors:
		ZeeX, koolk, JoeBullet/Google63, g_aSlice/Slice
	
	Thanks:
		JoeBullet/Google63 - Handy arbitrary ASM jump code using SCTRL.
		ZeeX - Very productive conversations.
		koolk - IsPlayerinAreaEx code.
		TheAlpha - Danish translation.
		breadfish - German translation.
		Fireburn - Dutch translation.
		yom - French translation.
		50p - Polish translation.
		Zamaroht - Spanish translation.
		Dracoblue, sintax, mabako, Xtreme, other coders - Producing other modes
			for me to strive to better.
		Pixels^ - Running XScripters where the idea was born.
		Matite - Pestering me to release it and using it.
	
	Very special thanks to:
		Thiadmer - PAWN, whose limits continue to amaze me!
		Kye/Kalcor - SA:MP.
		SA:MP Team past, present and future - SA:MP.
	
Version:
	1.0
Changelog:
	06/01/14:
		Added embeddable versions of all colours.
		Cached all macros for faster compilation.
		Added "GT" macro for "~r~" use in SA:MP text.
		Split gametext colours in to a separate file.
	29/11/10:
		Added the ability to use "X11" prefixes in strings.
		Added colourspace resolution for converting {FF0000} to ~r~.
	25/11/10:
		First version.
\**--------------------------------------------------------------------------**/

// Standard colours
#define X11_SNOW                    0xFFFAFAAA
#define X11_GHOSTWHITE              0xF8F8FFAA
#define X11_GHOST_WHITE             0xF8F8FFAA
#define X11_WHITESMOKE              0xF5F5F5AA
#define X11_WHITE_SMOKE             0xF5F5F5AA
#define X11_GAINSBORO               0xDCDCDCAA
#define X11_FLORALWHITE             0xFFFAF0AA
#define X11_FLORAL_WHITE            0xFFFAF0AA
#define X11_OLDLACE                 0xFDF5E6AA
#define X11_OLD_LACE                0xFDF5E6AA
#define X11_LINEN                   0xFAF0E6AA
#define X11_ANTIQUEWHITE            0xFAEBD7AA
#define X11_ANTIQUE_WHITE           0xFAEBD7AA
#define X11_PAPAYAWHIP              0xFFEFD5AA
#define X11_PAPAYA_WHIP             0xFFEFD5AA
#define X11_BLANCHEDALMOND          0xFFEBCDAA
#define X11_BLANCHED_ALMOND         0xFFEBCDAA
#define X11_BISQUE                  0xFFE4C4AA
#define X11_PEACHPUFF               0xFFDAB9AA
#define X11_PEACH_PUFF              0xFFDAB9AA
#define X11_NAVAJOWHITE             0xFFDEADAA
#define X11_NAVAJO_WHITE            0xFFDEADAA
#define X11_MOCCASIN                0xFFE4B5AA
#define X11_CORNSILK                0xFFF8DCAA
#define X11_IVORY                   0xFFFFF0AA
#define X11_LEMONCHIFFON            0xFFFACDAA
#define X11_LEMON_CHIFFON           0xFFFACDAA
#define X11_SEASHELL                0xFFF5EEAA
#define X11_HONEYDEW                0xF0FFF0AA
#define X11_MINTCREAM               0xF5FFFAAA
#define X11_MINT_CREAM              0xF5FFFAAA
#define X11_AZURE                   0xF0FFFFAA
#define X11_ALICEBLUE               0xF0F8FFAA
#define X11_ALICE_BLUE              0xF0F8FFAA
#define X11_LAVENDER                0xE6E6FAAA
#define X11_LAVENDERBLUSH           0xFFF0F5AA
#define X11_LAVENDER_BLUSH          0xFFF0F5AA
#define X11_MISTYROSE               0xFFE4E1AA
#define X11_MISTY_ROSE              0xFFE4E1AA
#define X11_WHITE                   0xFFFFFFAA
#define X11_BLACK                   0x000000AA
#define X11_DARKSLATEGRAY           0x2F4F4FAA
#define X11_DARK_SLATE_GRAY         0x2F4F4FAA
#define X11_DARKSLATEGREY           0x2F4F4FAA
#define X11_DARK_SLATE_GREY         0x2F4F4FAA
#define X11_DIMGRAY                 0x696969AA
#define X11_DIM_GRAY                0x696969AA
#define X11_DIMGREY                 0x696969AA
#define X11_DIM_GREY                0x696969AA
#define X11_SLATEGRAY               0x708090AA
#define X11_SLATE_GRAY              0x708090AA
#define X11_SLATEGREY               0x708090AA
#define X11_SLATE_GREY              0x708090AA
#define X11_LIGHTSLATEGRAY          0x778899AA
#define X11_LIGHT_SLATE_GRAY        0x778899AA
#define X11_LIGHTSLATEGREY          0x778899AA
#define X11_LIGHT_SLATE_GREY        0x778899AA
#define X11_GRAY                    0xBEBEBEAA
#define X11_GREY                    0xBEBEBEAA
#define X11_LIGHTGREY               0xD3D3D3AA
#define X11_LIGHT_GREY              0xD3D3D3AA
#define X11_LIGHTGRAY               0xD3D3D3AA
#define X11_LIGHT_GRAY              0xD3D3D3AA
#define X11_MIDNIGHTBLUE            0x191970AA
#define X11_MIDNIGHT_BLUE           0x191970AA
#define X11_NAVY                    0x000080AA
#define X11_NAVYBLUE                0x000080AA
#define X11_NAVY_BLUE               0x000080AA
#define X11_CORNFLOWERBLUE          0x6495EDAA
#define X11_CORNFLOWER_BLUE         0x6495EDAA
#define X11_DARKSLATEBLUE           0x483D8BAA
#define X11_DARK_SLATE_BLUE         0x483D8BAA
#define X11_SLATEBLUE               0x6A5ACDAA
#define X11_SLATE_BLUE              0x6A5ACDAA
#define X11_MEDIUMSLATEBLUE         0x7B68EEAA
#define X11_MEDIUM_SLATE_BLUE       0x7B68EEAA
#define X11_LIGHTSLATEBLUE          0x8470FFAA
#define X11_LIGHT_SLATE_BLUE        0x8470FFAA
#define X11_MEDIUMBLUE              0x0000CDAA
#define X11_MEDIUM_BLUE             0x0000CDAA
#define X11_ROYALBLUE               0x4169E1AA
#define X11_ROYAL_BLUE              0x4169E1AA
#define X11_BLUE                    0x0000FFAA
#define X11_DODGERBLUE              0x1E90FFAA
#define X11_DODGER_BLUE             0x1E90FFAA
#define X11_DEEPSKYBLUE             0x00BFFFAA
#define X11_DEEP_SKY_BLUE           0x00BFFFAA
#define X11_SKYBLUE                 0x87CEEBAA
#define X11_SKY_BLUE                0x87CEEBAA
#define X11_LIGHTSKYBLUE            0x87CEFAAA
#define X11_LIGHT_SKY_BLUE          0x87CEFAAA
#define X11_STEELBLUE               0x4682B4AA
#define X11_STEEL_BLUE              0x4682B4AA
#define X11_LIGHTSTEELBLUE          0xB0C4DEAA
#define X11_LIGHT_STEEL_BLUE        0xB0C4DEAA
#define X11_LIGHTBLUE               0xADD8E6AA
#define X11_LIGHT_BLUE              0xADD8E6AA
#define X11_POWDERBLUE              0xB0E0E6AA
#define X11_POWDER_BLUE             0xB0E0E6AA
#define X11_PALETURQUOISE           0xAFEEEEAA
#define X11_PALE_TURQUOISE          0xAFEEEEAA
#define X11_DARKTURQUOISE           0x00CED1AA
#define X11_DARK_TURQUOISE          0x00CED1AA
#define X11_MEDIUMTURQUOISE         0x48D1CCAA
#define X11_MEDIUM_TURQUOISE        0x48D1CCAA
#define X11_TURQUOISE               0x40E0D0AA
#define X11_CYAN                    0x00FFFFAA
#define X11_LIGHTCYAN               0xE0FFFFAA
#define X11_LIGHT_CYAN              0xE0FFFFAA
#define X11_CADETBLUE               0x5F9EA0AA
#define X11_CADET_BLUE              0x5F9EA0AA
#define X11_MEDIUMAQUAMARINE        0x66CDAAAA
#define X11_MEDIUM_AQUAMARINE       0x66CDAAAA
#define X11_AQUAMARINE              0x7FFFD4AA
#define X11_DARKGREEN               0x006400AA
#define X11_DARK_GREEN              0x006400AA
#define X11_DARKOLIVEGREEN          0x556B2FAA
#define X11_DARK_OLIVE_GREEN        0x556B2FAA
#define X11_DARKSEAGREEN            0x8FBC8FAA
#define X11_DARK_SEA_GREEN          0x8FBC8FAA
#define X11_SEAGREEN                0x2E8B57AA
#define X11_SEA_GREEN               0x2E8B57AA
#define X11_MEDIUMSEAGREEN          0x3CB371AA
#define X11_MEDIUM_SEA_GREEN        0x3CB371AA
#define X11_LIGHTSEAGREEN           0x20B2AAAA
#define X11_LIGHT_SEA_GREEN         0x20B2AAAA
#define X11_PALEGREEN               0x98FB98AA
#define X11_PALE_GREEN              0x98FB98AA
#define X11_SPRINGGREEN             0x00FF7FAA
#define X11_SPRING_GREEN            0x00FF7FAA
#define X11_LAWNGREEN               0x7CFC00AA
#define X11_LAWN_GREEN              0x7CFC00AA
#define X11_GREEN                   0x00FF00AA
#define X11_CHARTREUSE              0x7FFF00AA
#define X11_MEDIUMSPRINGGREEN       0x00FA9AAA
#define X11_MEDIUM_SPRING_GREEN     0x00FA9AAA
#define X11_GREENYELLOW             0xADFF2FAA
#define X11_GREEN_YELLOW            0xADFF2FAA
#define X11_LIMEGREEN               0x32CD32AA
#define X11_LIME_GREEN              0x32CD32AA
#define X11_YELLOWGREEN             0x9ACD32AA
#define X11_YELLOW_GREEN            0x9ACD32AA
#define X11_FORESTGREEN             0x228B22AA
#define X11_FOREST_GREEN            0x228B22AA
#define X11_OLIVEDRAB               0x6B8E23AA
#define X11_OLIVE_DRAB              0x6B8E23AA
#define X11_DARKKHAKI               0xBDB76BAA
#define X11_DARK_KHAKI              0xBDB76BAA
#define X11_KHAKI                   0xF0E68CAA
#define X11_PALEGOLDENROD           0xEEE8AAAA
#define X11_PALE_GOLDENROD          0xEEE8AAAA
#define X11_LIGHTGOLDENRODYELLOW    0xFAFAD2AA
#define X11_LIGHT_GOLDENROD_YELLOW  0xFAFAD2AA
#define X11_LIGHTYELLOW             0xFFFFE0AA
#define X11_LIGHT_YELLOW            0xFFFFE0AA
#define X11_YELLOW                  0xFFFF00AA
#define X11_GOLD                    0xFFD700AA
#define X11_LIGHTGOLDENROD          0xEEDD82AA
#define X11_LIGHT_GOLDENROD         0xEEDD82AA
#define X11_GOLDENROD               0xDAA520AA
#define X11_DARKGOLDENROD           0xB8860BAA
#define X11_DARK_GOLDENROD          0xB8860BAA
#define X11_ROSYBROWN               0xBC8F8FAA
#define X11_ROSY_BROWN              0xBC8F8FAA
#define X11_INDIANRED               0xCD5C5CAA
#define X11_INDIAN_RED              0xCD5C5CAA
#define X11_SADDLEBROWN             0x8B4513AA
#define X11_SADDLE_BROWN            0x8B4513AA
#define X11_SIENNA                  0xA0522DAA
#define X11_PERU                    0xCD853FAA
#define X11_BURLYWOOD               0xDEB887AA
#define X11_BEIGE                   0xF5F5DCAA
#define X11_WHEAT                   0xF5DEB3AA
#define X11_SANDYBROWN              0xF4A460AA
#define X11_SANDY_BROWN             0xF4A460AA
#define X11_TAN                     0xD2B48CAA
#define X11_CHOCOLATE               0xD2691EAA
#define X11_FIREBRICK               0xB22222AA
#define X11_BROWN                   0xA52A2AAA
#define X11_DARKSALMON              0xE9967AAA
#define X11_DARK_SALMON             0xE9967AAA
#define X11_SALMON                  0xFA8072AA
#define X11_LIGHTSALMON             0xFFA07AAA
#define X11_LIGHT_SALMON            0xFFA07AAA
#define X11_ORANGE                  0xFFA500AA
#define X11_DARKORANGE              0xFF8C00AA
#define X11_DARK_ORANGE             0xFF8C00AA
#define X11_CORAL                   0xFF7F50AA
#define X11_LIGHTCORAL              0xF08080AA
#define X11_LIGHT_CORAL             0xF08080AA
#define X11_TOMATO                  0xFF6347AA
#define X11_ORANGERED               0xFF4500AA
#define X11_ORANGE_RED              0xFF4500AA
#define X11_RED                     0xFF0000AA
#define X11_HOTPINK                 0xFF69B4AA
#define X11_HOT_PINK                0xFF69B4AA
#define X11_DEEPPINK                0xFF1493AA
#define X11_DEEP_PINK               0xFF1493AA
#define X11_PINK                    0xFFC0CBAA
#define X11_LIGHTPINK               0xFFB6C1AA
#define X11_LIGHT_PINK              0xFFB6C1AA
#define X11_PALEVIOLETRED           0xDB7093AA
#define X11_PALE_VIOLET_RED         0xDB7093AA
#define X11_MAROON                  0xB03060AA
#define X11_MEDIUMVIOLETRED         0xC71585AA
#define X11_MEDIUM_VIOLET_RED       0xC71585AA
#define X11_VIOLETRED               0xD02090AA
#define X11_VIOLET_RED              0xD02090AA
#define X11_MAGENTA                 0xFF00FFAA
#define X11_VIOLET                  0xEE82EEAA
#define X11_PLUM                    0xDDA0DDAA
#define X11_ORCHID                  0xDA70D6AA
#define X11_MEDIUMORCHID            0xBA55D3AA
#define X11_MEDIUM_ORCHID           0xBA55D3AA
#define X11_DARKORCHID              0x9932CCAA
#define X11_DARK_ORCHID             0x9932CCAA
#define X11_DARKVIOLET              0x9400D3AA
#define X11_DARK_VIOLET             0x9400D3AA
#define X11_BLUEVIOLET              0x8A2BE2AA
#define X11_BLUE_VIOLET             0x8A2BE2AA
#define X11_PURPLE                  0xA020F0AA
#define X11_MEDIUMPURPLE            0x9370DBAA
#define X11_MEDIUM_PURPLE           0x9370DBAA
#define X11_THISTLE                 0xD8BFD8AA
#define X11_SNOW1                   0xFFFAFAAA
#define X11_SNOW_1                  0xFFFAFAAA
#define X11_SNOW2                   0xEEE9E9AA
#define X11_SNOW_2                  0xEEE9E9AA
#define X11_SNOW3                   0xCDC9C9AA
#define X11_SNOW_3                  0xCDC9C9AA
#define X11_SNOW4                   0x8B8989AA
#define X11_SNOW_4                  0x8B8989AA
#define X11_SEASHELL1               0xFFF5EEAA
#define X11_SEASHELL_1              0xFFF5EEAA
#define X11_SEASHELL2               0xEEE5DEAA
#define X11_SEASHELL_2              0xEEE5DEAA
#define X11_SEASHELL3               0xCDC5BFAA
#define X11_SEASHELL_3              0xCDC5BFAA
#define X11_SEASHELL4               0x8B8682AA
#define X11_SEASHELL_4              0x8B8682AA
#define X11_ANTIQUEWHITE1           0xFFEFDBAA
#define X11_ANTIQUE_WHITE_1         0xFFEFDBAA
#define X11_ANTIQUEWHITE2           0xEEDFCCAA
#define X11_ANTIQUE_WHITE_2         0xEEDFCCAA
#define X11_ANTIQUEWHITE3           0xCDC0B0AA
#define X11_ANTIQUE_WHITE_3         0xCDC0B0AA
#define X11_ANTIQUEWHITE4           0x8B8378AA
#define X11_ANTIQUE_WHITE_4         0x8B8378AA
#define X11_BISQUE1                 0xFFE4C4AA
#define X11_BISQUE_1                0xFFE4C4AA
#define X11_BISQUE2                 0xEED5B7AA
#define X11_BISQUE_2                0xEED5B7AA
#define X11_BISQUE3                 0xCDB79EAA
#define X11_BISQUE_3                0xCDB79EAA
#define X11_BISQUE4                 0x8B7D6BAA
#define X11_BISQUE_4                0x8B7D6BAA
#define X11_PEACHPUFF1              0xFFDAB9AA
#define X11_PEACH_PUFF_1            0xFFDAB9AA
#define X11_PEACHPUFF2              0xEECBADAA
#define X11_PEACH_PUFF_2            0xEECBADAA
#define X11_PEACHPUFF3              0xCDAF95AA
#define X11_PEACH_PUFF_3            0xCDAF95AA
#define X11_PEACHPUFF4              0x8B7765AA
#define X11_PEACH_PUFF_4            0x8B7765AA
#define X11_NAVAJOWHITE1            0xFFDEADAA
#define X11_NAVAJO_WHITE_1          0xFFDEADAA
#define X11_NAVAJOWHITE2            0xEECFA1AA
#define X11_NAVAJO_WHITE_2          0xEECFA1AA
#define X11_NAVAJOWHITE3            0xCDB38BAA
#define X11_NAVAJO_WHITE_3          0xCDB38BAA
#define X11_NAVAJOWHITE4            0x8B795EAA
#define X11_NAVAJO_WHITE_4          0x8B795EAA
#define X11_LEMONCHIFFON1           0xFFFACDAA
#define X11_LEMON_CHIFFON_1         0xFFFACDAA
#define X11_LEMONCHIFFON2           0xEEE9BFAA
#define X11_LEMON_CHIFFON_2         0xEEE9BFAA
#define X11_LEMONCHIFFON3           0xCDC9A5AA
#define X11_LEMON_CHIFFON_3         0xCDC9A5AA
#define X11_LEMONCHIFFON4           0x8B8970AA
#define X11_LEMON_CHIFFON_4         0x8B8970AA
#define X11_CORNSILK1               0xFFF8DCAA
#define X11_CORNSILK_1              0xFFF8DCAA
#define X11_CORNSILK2               0xEEE8CDAA
#define X11_CORNSILK_2              0xEEE8CDAA
#define X11_CORNSILK3               0xCDC8B1AA
#define X11_CORNSILK_3              0xCDC8B1AA
#define X11_CORNSILK4               0x8B8878AA
#define X11_CORNSILK_4              0x8B8878AA
#define X11_IVORY1                  0xFFFFF0AA
#define X11_IVORY_1                 0xFFFFF0AA
#define X11_IVORY2                  0xEEEEE0AA
#define X11_IVORY_2                 0xEEEEE0AA
#define X11_IVORY3                  0xCDCDC1AA
#define X11_IVORY_3                 0xCDCDC1AA
#define X11_IVORY4                  0x8B8B83AA
#define X11_IVORY_4                 0x8B8B83AA
#define X11_HONEYDEW1               0xF0FFF0AA
#define X11_HONEYDEW_1              0xF0FFF0AA
#define X11_HONEYDEW2               0xE0EEE0AA
#define X11_HONEYDEW_2              0xE0EEE0AA
#define X11_HONEYDEW3               0xC1CDC1AA
#define X11_HONEYDEW_3              0xC1CDC1AA
#define X11_HONEYDEW4               0x838B83AA
#define X11_HONEYDEW_4              0x838B83AA
#define X11_LAVENDERBLUSH1          0xFFF0F5AA
#define X11_LAVENDER_BLUSH_1        0xFFF0F5AA
#define X11_LAVENDERBLUSH2          0xEEE0E5AA
#define X11_LAVENDER_BLUSH_2        0xEEE0E5AA
#define X11_LAVENDERBLUSH3          0xCDC1C5AA
#define X11_LAVENDER_BLUSH_3        0xCDC1C5AA
#define X11_LAVENDERBLUSH4          0x8B8386AA
#define X11_LAVENDER_BLUSH_4        0x8B8386AA
#define X11_MISTYROSE1              0xFFE4E1AA
#define X11_MISTY_ROSE_1            0xFFE4E1AA
#define X11_MISTYROSE2              0xEED5D2AA
#define X11_MISTY_ROSE_2            0xEED5D2AA
#define X11_MISTYROSE3              0xCDB7B5AA
#define X11_MISTY_ROSE_3            0xCDB7B5AA
#define X11_MISTYROSE4              0x8B7D7BAA
#define X11_MISTY_ROSE_4            0x8B7D7BAA
#define X11_AZURE1                  0xF0FFFFAA
#define X11_AZURE_1                 0xF0FFFFAA
#define X11_AZURE2                  0xE0EEEEAA
#define X11_AZURE_2                 0xE0EEEEAA
#define X11_AZURE3                  0xC1CDCDAA
#define X11_AZURE_3                 0xC1CDCDAA
#define X11_AZURE4                  0x838B8BAA
#define X11_AZURE_4                 0x838B8BAA
#define X11_SLATEBLUE1              0x836FFFAA
#define X11_SLATE_BLUE_1            0x836FFFAA
#define X11_SLATEBLUE2              0x7A67EEAA
#define X11_SLATE_BLUE_2            0x7A67EEAA
#define X11_SLATEBLUE3              0x6959CDAA
#define X11_SLATE_BLUE_3            0x6959CDAA
#define X11_SLATEBLUE4              0x473C8BAA
#define X11_SLATE_BLUE_4            0x473C8BAA
#define X11_ROYALBLUE1              0x4876FFAA
#define X11_ROYAL_BLUE_1            0x4876FFAA
#define X11_ROYALBLUE2              0x436EEEAA
#define X11_ROYAL_BLUE_2            0x436EEEAA
#define X11_ROYALBLUE3              0x3A5FCDAA
#define X11_ROYAL_BLUE_3            0x3A5FCDAA
#define X11_ROYALBLUE4              0x27408BAA
#define X11_ROYAL_BLUE_4            0x27408BAA
#define X11_BLUE1                   0x0000FFAA
#define X11_BLUE_1                  0x0000FFAA
#define X11_BLUE2                   0x0000EEAA
#define X11_BLUE_2                  0x0000EEAA
#define X11_BLUE3                   0x0000CDAA
#define X11_BLUE_3                  0x0000CDAA
#define X11_BLUE4                   0x00008BAA
#define X11_BLUE_4                  0x00008BAA
#define X11_DODGERBLUE1             0x1E90FFAA
#define X11_DODGER_BLUE_1           0x1E90FFAA
#define X11_DODGERBLUE2             0x1C86EEAA
#define X11_DODGER_BLUE_2           0x1C86EEAA
#define X11_DODGERBLUE3             0x1874CDAA
#define X11_DODGER_BLUE_3           0x1874CDAA
#define X11_DODGERBLUE4             0x104E8BAA
#define X11_DODGER_BLUE_4           0x104E8BAA
#define X11_STEELBLUE1              0x63B8FFAA
#define X11_STEEL_BLUE_1            0x63B8FFAA
#define X11_STEELBLUE2              0x5CACEEAA
#define X11_STEEL_BLUE_2            0x5CACEEAA
#define X11_STEELBLUE3              0x4F94CDAA
#define X11_STEEL_BLUE_3            0x4F94CDAA
#define X11_STEELBLUE4              0x36648BAA
#define X11_STEEL_BLUE_4            0x36648BAA
#define X11_DEEPSKYBLUE1            0x00BFFFAA
#define X11_DEEP_SKY_BLUE_1         0x00BFFFAA
#define X11_DEEPSKYBLUE2            0x00B2EEAA
#define X11_DEEP_SKY_BLUE_2         0x00B2EEAA
#define X11_DEEPSKYBLUE3            0x009ACDAA
#define X11_DEEP_SKY_BLUE_3         0x009ACDAA
#define X11_DEEPSKYBLUE4            0x00688BAA
#define X11_DEEP_SKY_BLUE_4         0x00688BAA
#define X11_SKYBLUE1                0x87CEFFAA
#define X11_SKY_BLUE_1              0x87CEFFAA
#define X11_SKYBLUE2                0x7EC0EEAA
#define X11_SKY_BLUE_2              0x7EC0EEAA
#define X11_SKYBLUE3                0x6CA6CDAA
#define X11_SKY_BLUE_3              0x6CA6CDAA
#define X11_SKYBLUE4                0x4A708BAA
#define X11_SKY_BLUE_4              0x4A708BAA
#define X11_LIGHTSKYBLUE1           0xB0E2FFAA
#define X11_LIGHT_SKY_BLUE_1        0xB0E2FFAA
#define X11_LIGHTSKYBLUE2           0xA4D3EEAA
#define X11_LIGHT_SKY_BLUE_2        0xA4D3EEAA
#define X11_LIGHTSKYBLUE3           0x8DB6CDAA
#define X11_LIGHT_SKY_BLUE_3        0x8DB6CDAA
#define X11_LIGHTSKYBLUE4           0x607B8BAA
#define X11_LIGHT_SKY_BLUE_4        0x607B8BAA
#define X11_SLATEGRAY1              0xC6E2FFAA
#define X11_SLATE_GRAY_1            0xC6E2FFAA
#define X11_SLATEGRAY2              0xB9D3EEAA
#define X11_SLATE_GRAY_2            0xB9D3EEAA
#define X11_SLATEGRAY3              0x9FB6CDAA
#define X11_SLATE_GRAY_3            0x9FB6CDAA
#define X11_SLATEGRAY4              0x6C7B8BAA
#define X11_SLATE_GRAY_4            0x6C7B8BAA
#define X11_SLATEGREY1              0xC6E2FFAA
#define X11_SLATE_GREY_1            0xC6E2FFAA
#define X11_SLATEGREY2              0xB9D3EEAA
#define X11_SLATE_GREY_2            0xB9D3EEAA
#define X11_SLATEGREY3              0x9FB6CDAA
#define X11_SLATE_GREY_3            0x9FB6CDAA
#define X11_SLATEGREY4              0x6C7B8BAA
#define X11_SLATE_GREY_4            0x6C7B8BAA
#define X11_LIGHTSTEELBLUE1         0xCAE1FFAA
#define X11_LIGHT_STEEL_BLUE_1      0xCAE1FFAA
#define X11_LIGHTSTEELBLUE2         0xBCD2EEAA
#define X11_LIGHT_STEEL_BLUE_2      0xBCD2EEAA
#define X11_LIGHTSTEELBLUE3         0xA2B5CDAA
#define X11_LIGHT_STEEL_BLUE_3      0xA2B5CDAA
#define X11_LIGHTSTEELBLUE4         0x6E7B8BAA
#define X11_LIGHT_STEEL_BLUE_4      0x6E7B8BAA
#define X11_LIGHTBLUE1              0xBFEFFFAA
#define X11_LIGHT_BLUE_1            0xBFEFFFAA
#define X11_LIGHTBLUE2              0xB2DFEEAA
#define X11_LIGHT_BLUE_2            0xB2DFEEAA
#define X11_LIGHTBLUE3              0x9AC0CDAA
#define X11_LIGHT_BLUE_3            0x9AC0CDAA
#define X11_LIGHTBLUE4              0x68838BAA
#define X11_LIGHT_BLUE_4            0x68838BAA
#define X11_LIGHTCYAN1              0xE0FFFFAA
#define X11_LIGHT_CYAN_1            0xE0FFFFAA
#define X11_LIGHTCYAN2              0xD1EEEEAA
#define X11_LIGHT_CYAN_2            0xD1EEEEAA
#define X11_LIGHTCYAN3              0xB4CDCDAA
#define X11_LIGHT_CYAN_3            0xB4CDCDAA
#define X11_LIGHTCYAN4              0x7A8B8BAA
#define X11_LIGHT_CYAN_4            0x7A8B8BAA
#define X11_PALETURQUOISE1          0xBBFFFFAA
#define X11_PALE_TURQUOISE_1        0xBBFFFFAA
#define X11_PALETURQUOISE2          0xAEEEEEAA
#define X11_PALE_TURQUOISE_2        0xAEEEEEAA
#define X11_PALETURQUOISE3          0x96CDCDAA
#define X11_PALE_TURQUOISE_3        0x96CDCDAA
#define X11_PALETURQUOISE4          0x668B8BAA
#define X11_PALE_TURQUOISE_4        0x668B8BAA
#define X11_CADETBLUE1              0x98F5FFAA
#define X11_CADET_BLUE_1            0x98F5FFAA
#define X11_CADETBLUE2              0x8EE5EEAA
#define X11_CADET_BLUE_2            0x8EE5EEAA
#define X11_CADETBLUE3              0x7AC5CDAA
#define X11_CADET_BLUE_3            0x7AC5CDAA
#define X11_CADETBLUE4              0x53868BAA
#define X11_CADET_BLUE_4            0x53868BAA
#define X11_TURQUOISE1              0x00F5FFAA
#define X11_TURQUOISE_1             0x00F5FFAA
#define X11_TURQUOISE2              0x00E5EEAA
#define X11_TURQUOISE_2             0x00E5EEAA
#define X11_TURQUOISE3              0x00C5CDAA
#define X11_TURQUOISE_3             0x00C5CDAA
#define X11_TURQUOISE4              0x00868BAA
#define X11_TURQUOISE_4             0x00868BAA
#define X11_CYAN1                   0x00FFFFAA
#define X11_CYAN_1                  0x00FFFFAA
#define X11_CYAN2                   0x00EEEEAA
#define X11_CYAN_2                  0x00EEEEAA
#define X11_CYAN3                   0x00CDCDAA
#define X11_CYAN_3                  0x00CDCDAA
#define X11_CYAN4                   0x008B8BAA
#define X11_CYAN_4                  0x008B8BAA
#define X11_DARKSLATEGRAY1          0x97FFFFAA
#define X11_DARK_SLATE_GRAY_1       0x97FFFFAA
#define X11_DARKSLATEGRAY2          0x8DEEEEAA
#define X11_DARK_SLATE_GRAY_2       0x8DEEEEAA
#define X11_DARKSLATEGRAY3          0x79CDCDAA
#define X11_DARK_SLATE_GRAY_3       0x79CDCDAA
#define X11_DARKSLATEGRAY4          0x528B8BAA
#define X11_DARK_SLATE_GRAY_4       0x528B8BAA
#define X11_DARKSLATEGREY1          0x97FFFFAA
#define X11_DARK_SLATE_GREY_1       0x97FFFFAA
#define X11_DARKSLATEGREY2          0x8DEEEEAA
#define X11_DARK_SLATE_GREY_2       0x8DEEEEAA
#define X11_DARKSLATEGREY3          0x79CDCDAA
#define X11_DARK_SLATE_GREY_3       0x79CDCDAA
#define X11_DARKSLATEGREY4          0x528B8BAA
#define X11_DARK_SLATE_GREY_4       0x528B8BAA
#define X11_AQUAMARINE1             0x7FFFD4AA
#define X11_AQUAMARINE_1            0x7FFFD4AA
#define X11_AQUAMARINE2             0x76EEC6AA
#define X11_AQUAMARINE_2            0x76EEC6AA
#define X11_AQUAMARINE3             0x66CDAAAA
#define X11_AQUAMARINE_3            0x66CDAAAA
#define X11_AQUAMARINE4             0x458B74AA
#define X11_AQUAMARINE_4            0x458B74AA
#define X11_DARKSEAGREEN1           0xC1FFC1AA
#define X11_DARK_SEA_GREEN_1        0xC1FFC1AA
#define X11_DARKSEAGREEN2           0xB4EEB4AA
#define X11_DARK_SEA_GREEN_2        0xB4EEB4AA
#define X11_DARKSEAGREEN3           0x9BCD9BAA
#define X11_DARK_SEA_GREEN_3        0x9BCD9BAA
#define X11_DARKSEAGREEN4           0x698B69AA
#define X11_DARK_SEA_GREEN_4        0x698B69AA
#define X11_SEAGREEN1               0x54FF9FAA
#define X11_SEA_GREEN_1             0x54FF9FAA
#define X11_SEAGREEN2               0x4EEE94AA
#define X11_SEA_GREEN_2             0x4EEE94AA
#define X11_SEAGREEN3               0x43CD80AA
#define X11_SEA_GREEN_3             0x43CD80AA
#define X11_SEAGREEN4               0x2E8B57AA
#define X11_SEA_GREEN_4             0x2E8B57AA
#define X11_PALEGREEN1              0x9AFF9AAA
#define X11_PALE_GREEN_1            0x9AFF9AAA
#define X11_PALEGREEN2              0x90EE90AA
#define X11_PALE_GREEN_2            0x90EE90AA
#define X11_PALEGREEN3              0x7CCD7CAA
#define X11_PALE_GREEN_3            0x7CCD7CAA
#define X11_PALEGREEN4              0x548B54AA
#define X11_PALE_GREEN_4            0x548B54AA
#define X11_SPRINGGREEN1            0x00FF7FAA
#define X11_SPRING_GREEN_1          0x00FF7FAA
#define X11_SPRINGGREEN2            0x00EE76AA
#define X11_SPRING_GREEN_2          0x00EE76AA
#define X11_SPRINGGREEN3            0x00CD66AA
#define X11_SPRING_GREEN_3          0x00CD66AA
#define X11_SPRINGGREEN4            0x008B45AA
#define X11_SPRING_GREEN_4          0x008B45AA
#define X11_GREEN1                  0x00FF00AA
#define X11_GREEN_1                 0x00FF00AA
#define X11_GREEN2                  0x00EE00AA
#define X11_GREEN_2                 0x00EE00AA
#define X11_GREEN3                  0x00CD00AA
#define X11_GREEN_3                 0x00CD00AA
#define X11_GREEN4                  0x008B00AA
#define X11_GREEN_4                 0x008B00AA
#define X11_CHARTREUSE1             0x7FFF00AA
#define X11_CHARTREUSE_1            0x7FFF00AA
#define X11_CHARTREUSE2             0x76EE00AA
#define X11_CHARTREUSE_2            0x76EE00AA
#define X11_CHARTREUSE3             0x66CD00AA
#define X11_CHARTREUSE_3            0x66CD00AA
#define X11_CHARTREUSE4             0x458B00AA
#define X11_CHARTREUSE_4            0x458B00AA
#define X11_OLIVEDRAB1              0xC0FF3EAA
#define X11_OLIVE_DRAB_1            0xC0FF3EAA
#define X11_OLIVEDRAB2              0xB3EE3AAA
#define X11_OLIVE_DRAB_2            0xB3EE3AAA
#define X11_OLIVEDRAB3              0x9ACD32AA
#define X11_OLIVE_DRAB_3            0x9ACD32AA
#define X11_OLIVEDRAB4              0x698B22AA
#define X11_OLIVE_DRAB_4            0x698B22AA
#define X11_DARKOLIVEGREEN1         0xCAFF70AA
#define X11_DARK_OLIVE_GREEN_1      0xCAFF70AA
#define X11_DARKOLIVEGREEN2         0xBCEE68AA
#define X11_DARK_OLIVE_GREEN_2      0xBCEE68AA
#define X11_DARKOLIVEGREEN3         0xA2CD5AAA
#define X11_DARK_OLIVE_GREEN_3      0xA2CD5AAA
#define X11_DARKOLIVEGREEN4         0x6E8B3DAA
#define X11_DARK_OLIVE_GREEN_4      0x6E8B3DAA
#define X11_KHAKI1                  0xFFF68FAA
#define X11_KHAKI_1                 0xFFF68FAA
#define X11_KHAKI2                  0xEEE685AA
#define X11_KHAKI_2                 0xEEE685AA
#define X11_KHAKI3                  0xCDC673AA
#define X11_KHAKI_3                 0xCDC673AA
#define X11_KHAKI4                  0x8B864EAA
#define X11_KHAKI_4                 0x8B864EAA
#define X11_LIGHTGOLDENROD1         0xFFEC8BAA
#define X11_LIGHT_GOLDENROD_1       0xFFEC8BAA
#define X11_LIGHTGOLDENROD2         0xEEDC82AA
#define X11_LIGHT_GOLDENROD_2       0xEEDC82AA
#define X11_LIGHTGOLDENROD3         0xCDBE70AA
#define X11_LIGHT_GOLDENROD_3       0xCDBE70AA
#define X11_LIGHTGOLDENROD4         0x8B814CAA
#define X11_LIGHT_GOLDENROD_4       0x8B814CAA
#define X11_LIGHTYELLOW1            0xFFFFE0AA
#define X11_LIGHT_YELLOW_1          0xFFFFE0AA
#define X11_LIGHTYELLOW2            0xEEEED1AA
#define X11_LIGHT_YELLOW_2          0xEEEED1AA
#define X11_LIGHTYELLOW3            0xCDCDB4AA
#define X11_LIGHT_YELLOW_3          0xCDCDB4AA
#define X11_LIGHTYELLOW4            0x8B8B7AAA
#define X11_LIGHT_YELLOW_4          0x8B8B7AAA
#define X11_YELLOW1                 0xFFFF00AA
#define X11_YELLOW_1                0xFFFF00AA
#define X11_YELLOW2                 0xEEEE00AA
#define X11_YELLOW_2                0xEEEE00AA
#define X11_YELLOW3                 0xCDCD00AA
#define X11_YELLOW_3                0xCDCD00AA
#define X11_YELLOW4                 0x8B8B00AA
#define X11_YELLOW_4                0x8B8B00AA
#define X11_GOLD1                   0xFFD700AA
#define X11_GOLD_1                  0xFFD700AA
#define X11_GOLD2                   0xEEC900AA
#define X11_GOLD_2                  0xEEC900AA
#define X11_GOLD3                   0xCDAD00AA
#define X11_GOLD_3                  0xCDAD00AA
#define X11_GOLD4                   0x8B7500AA
#define X11_GOLD_4                  0x8B7500AA
#define X11_GOLDENROD1              0xFFC125AA
#define X11_GOLDENROD_1             0xFFC125AA
#define X11_GOLDENROD2              0xEEB422AA
#define X11_GOLDENROD_2             0xEEB422AA
#define X11_GOLDENROD3              0xCD9B1DAA
#define X11_GOLDENROD_3             0xCD9B1DAA
#define X11_GOLDENROD4              0x8B6914AA
#define X11_GOLDENROD_4             0x8B6914AA
#define X11_DARKGOLDENROD1          0xFFB90FAA
#define X11_DARK_GOLDENROD_1        0xFFB90FAA
#define X11_DARKGOLDENROD2          0xEEAD0EAA
#define X11_DARK_GOLDENROD_2        0xEEAD0EAA
#define X11_DARKGOLDENROD3          0xCD950CAA
#define X11_DARK_GOLDENROD_3        0xCD950CAA
#define X11_DARKGOLDENROD4          0x8B6508AA
#define X11_DARK_GOLDENROD_4        0x8B6508AA
#define X11_ROSYBROWN1              0xFFC1C1AA
#define X11_ROSY_BROWN_1            0xFFC1C1AA
#define X11_ROSYBROWN2              0xEEB4B4AA
#define X11_ROSY_BROWN_2            0xEEB4B4AA
#define X11_ROSYBROWN3              0xCD9B9BAA
#define X11_ROSY_BROWN_3            0xCD9B9BAA
#define X11_ROSYBROWN4              0x8B6969AA
#define X11_ROSY_BROWN_4            0x8B6969AA
#define X11_INDIANRED1              0xFF6A6AAA
#define X11_INDIAN_RED_1            0xFF6A6AAA
#define X11_INDIANRED2              0xEE6363AA
#define X11_INDIAN_RED_2            0xEE6363AA
#define X11_INDIANRED3              0xCD5555AA
#define X11_INDIAN_RED_3            0xCD5555AA
#define X11_INDIANRED4              0x8B3A3AAA
#define X11_INDIAN_RED_4            0x8B3A3AAA
#define X11_SIENNA1                 0xFF8247AA
#define X11_SIENNA_1                0xFF8247AA
#define X11_SIENNA2                 0xEE7942AA
#define X11_SIENNA_2                0xEE7942AA
#define X11_SIENNA3                 0xCD6839AA
#define X11_SIENNA_3                0xCD6839AA
#define X11_SIENNA4                 0x8B4726AA
#define X11_SIENNA_4                0x8B4726AA
#define X11_BURLYWOOD1              0xFFD39BAA
#define X11_BURLYWOOD_1             0xFFD39BAA
#define X11_BURLYWOOD2              0xEEC591AA
#define X11_BURLYWOOD_2             0xEEC591AA
#define X11_BURLYWOOD3              0xCDAA7DAA
#define X11_BURLYWOOD_3             0xCDAA7DAA
#define X11_BURLYWOOD4              0x8B7355AA
#define X11_BURLYWOOD_4             0x8B7355AA
#define X11_WHEAT1                  0xFFE7BAAA
#define X11_WHEAT_1                 0xFFE7BAAA
#define X11_WHEAT2                  0xEED8AEAA
#define X11_WHEAT_2                 0xEED8AEAA
#define X11_WHEAT3                  0xCDBA96AA
#define X11_WHEAT_3                 0xCDBA96AA
#define X11_WHEAT4                  0x8B7E66AA
#define X11_WHEAT_4                 0x8B7E66AA
#define X11_TAN1                    0xFFA54FAA
#define X11_TAN_1                   0xFFA54FAA
#define X11_TAN2                    0xEE9A49AA
#define X11_TAN_2                   0xEE9A49AA
#define X11_TAN3                    0xCD853FAA
#define X11_TAN_3                   0xCD853FAA
#define X11_TAN4                    0x8B5A2BAA
#define X11_TAN_4                   0x8B5A2BAA
#define X11_CHOCOLATE1              0xFF7F24AA
#define X11_CHOCOLATE_1             0xFF7F24AA
#define X11_CHOCOLATE2              0xEE7621AA
#define X11_CHOCOLATE_2             0xEE7621AA
#define X11_CHOCOLATE3              0xCD661DAA
#define X11_CHOCOLATE_3             0xCD661DAA
#define X11_CHOCOLATE4              0x8B4513AA
#define X11_CHOCOLATE_4             0x8B4513AA
#define X11_FIREBRICK1              0xFF3030AA
#define X11_FIREBRICK_1             0xFF3030AA
#define X11_FIREBRICK2              0xEE2C2CAA
#define X11_FIREBRICK_2             0xEE2C2CAA
#define X11_FIREBRICK3              0xCD2626AA
#define X11_FIREBRICK_3             0xCD2626AA
#define X11_FIREBRICK4              0x8B1A1AAA
#define X11_FIREBRICK_4             0x8B1A1AAA
#define X11_BROWN1                  0xFF4040AA
#define X11_BROWN_1                 0xFF4040AA
#define X11_BROWN2                  0xEE3B3BAA
#define X11_BROWN_2                 0xEE3B3BAA
#define X11_BROWN3                  0xCD3333AA
#define X11_BROWN_3                 0xCD3333AA
#define X11_BROWN4                  0x8B2323AA
#define X11_BROWN_4                 0x8B2323AA
#define X11_SALMON1                 0xFF8C69AA
#define X11_SALMON_1                0xFF8C69AA
#define X11_SALMON2                 0xEE8262AA
#define X11_SALMON_2                0xEE8262AA
#define X11_SALMON3                 0xCD7054AA
#define X11_SALMON_3                0xCD7054AA
#define X11_SALMON4                 0x8B4C39AA
#define X11_SALMON_4                0x8B4C39AA
#define X11_LIGHTSALMON1            0xFFA07AAA
#define X11_LIGHT_SALMON_1          0xFFA07AAA
#define X11_LIGHTSALMON2            0xEE9572AA
#define X11_LIGHT_SALMON_2          0xEE9572AA
#define X11_LIGHTSALMON3            0xCD8162AA
#define X11_LIGHT_SALMON_3          0xCD8162AA
#define X11_LIGHTSALMON4            0x8B5742AA
#define X11_LIGHT_SALMON_4          0x8B5742AA
#define X11_ORANGE1                 0xFFA500AA
#define X11_ORANGE_1                0xFFA500AA
#define X11_ORANGE2                 0xEE9A00AA
#define X11_ORANGE_2                0xEE9A00AA
#define X11_ORANGE3                 0xCD8500AA
#define X11_ORANGE_3                0xCD8500AA
#define X11_ORANGE4                 0x8B5A00AA
#define X11_ORANGE_4                0x8B5A00AA
#define X11_DARKORANGE1             0xFF7F00AA
#define X11_DARK_ORANGE_1           0xFF7F00AA
#define X11_DARKORANGE2             0xEE7600AA
#define X11_DARK_ORANGE_2           0xEE7600AA
#define X11_DARKORANGE3             0xCD6600AA
#define X11_DARK_ORANGE_3           0xCD6600AA
#define X11_DARKORANGE4             0x8B4500AA
#define X11_DARK_ORANGE_4           0x8B4500AA
#define X11_CORAL1                  0xFF7256AA
#define X11_CORAL_1                 0xFF7256AA
#define X11_CORAL2                  0xEE6A50AA
#define X11_CORAL_2                 0xEE6A50AA
#define X11_CORAL3                  0xCD5B45AA
#define X11_CORAL_3                 0xCD5B45AA
#define X11_CORAL4                  0x8B3E2FAA
#define X11_CORAL_4                 0x8B3E2FAA
#define X11_TOMATO1                 0xFF6347AA
#define X11_TOMATO_1                0xFF6347AA
#define X11_TOMATO2                 0xEE5C42AA
#define X11_TOMATO_2                0xEE5C42AA
#define X11_TOMATO3                 0xCD4F39AA
#define X11_TOMATO_3                0xCD4F39AA
#define X11_TOMATO4                 0x8B3626AA
#define X11_TOMATO_4                0x8B3626AA
#define X11_ORANGERED1              0xFF4500AA
#define X11_ORANGE_RED_1            0xFF4500AA
#define X11_ORANGERED2              0xEE4000AA
#define X11_ORANGE_RED_2            0xEE4000AA
#define X11_ORANGERED3              0xCD3700AA
#define X11_ORANGE_RED_3            0xCD3700AA
#define X11_ORANGERED4              0x8B2500AA
#define X11_ORANGE_RED_4            0x8B2500AA
#define X11_RED1                    0xFF0000AA
#define X11_RED_1                   0xFF0000AA
#define X11_RED2                    0xEE0000AA
#define X11_RED_2                   0xEE0000AA
#define X11_RED3                    0xCD0000AA
#define X11_RED_3                   0xCD0000AA
#define X11_RED4                    0x8B0000AA
#define X11_RED_4                   0x8B0000AA
#define X11_DEEPPINK1               0xFF1493AA
#define X11_DEEP_PINK_1             0xFF1493AA
#define X11_DEEPPINK2               0xEE1289AA
#define X11_DEEP_PINK_2             0xEE1289AA
#define X11_DEEPPINK3               0xCD1076AA
#define X11_DEEP_PINK_3             0xCD1076AA
#define X11_DEEPPINK4               0x8B0A50AA
#define X11_DEEP_PINK_4             0x8B0A50AA
#define X11_HOTPINK1                0xFF6EB4AA
#define X11_HOT_PINK_1              0xFF6EB4AA
#define X11_HOTPINK2                0xEE6AA7AA
#define X11_HOT_PINK_2              0xEE6AA7AA
#define X11_HOTPINK3                0xCD6090AA
#define X11_HOT_PINK_3              0xCD6090AA
#define X11_HOTPINK4                0x8B3A62AA
#define X11_HOT_PINK_4              0x8B3A62AA
#define X11_PINK1                   0xFFB5C5AA
#define X11_PINK_1                  0xFFB5C5AA
#define X11_PINK2                   0xEEA9B8AA
#define X11_PINK_2                  0xEEA9B8AA
#define X11_PINK3                   0xCD919EAA
#define X11_PINK_3                  0xCD919EAA
#define X11_PINK4                   0x8B636CAA
#define X11_PINK_4                  0x8B636CAA
#define X11_LIGHTPINK1              0xFFAEB9AA
#define X11_LIGHT_PINK_1            0xFFAEB9AA
#define X11_LIGHTPINK2              0xEEA2ADAA
#define X11_LIGHT_PINK_2            0xEEA2ADAA
#define X11_LIGHTPINK3              0xCD8C95AA
#define X11_LIGHT_PINK_3            0xCD8C95AA
#define X11_LIGHTPINK4              0x8B5F65AA
#define X11_LIGHT_PINK_4            0x8B5F65AA
#define X11_PALEVIOLETRED1          0xFF82ABAA
#define X11_PALE_VIOLET_RED_1       0xFF82ABAA
#define X11_PALEVIOLETRED2          0xEE799FAA
#define X11_PALE_VIOLET_RED_2       0xEE799FAA
#define X11_PALEVIOLETRED3          0xCD6889AA
#define X11_PALE_VIOLET_RED_3       0xCD6889AA
#define X11_PALEVIOLETRED4          0x8B475DAA
#define X11_PALE_VIOLET_RED_4       0x8B475DAA
#define X11_MAROON1                 0xFF34B3AA
#define X11_MAROON_1                0xFF34B3AA
#define X11_MAROON2                 0xEE30A7AA
#define X11_MAROON_2                0xEE30A7AA
#define X11_MAROON3                 0xCD2990AA
#define X11_MAROON_3                0xCD2990AA
#define X11_MAROON4                 0x8B1C62AA
#define X11_MAROON_4                0x8B1C62AA
#define X11_VIOLETRED1              0xFF3E96AA
#define X11_VIOLET_RED_1            0xFF3E96AA
#define X11_VIOLETRED2              0xEE3A8CAA
#define X11_VIOLET_RED_2            0xEE3A8CAA
#define X11_VIOLETRED3              0xCD3278AA
#define X11_VIOLET_RED_3            0xCD3278AA
#define X11_VIOLETRED4              0x8B2252AA
#define X11_VIOLET_RED_4            0x8B2252AA
#define X11_MAGENTA1                0xFF00FFAA
#define X11_MAGENTA_1               0xFF00FFAA
#define X11_MAGENTA2                0xEE00EEAA
#define X11_MAGENTA_2               0xEE00EEAA
#define X11_MAGENTA3                0xCD00CDAA
#define X11_MAGENTA_3               0xCD00CDAA
#define X11_MAGENTA4                0x8B008BAA
#define X11_MAGENTA_4               0x8B008BAA
#define X11_ORCHID1                 0xFF83FAAA
#define X11_ORCHID_1                0xFF83FAAA
#define X11_ORCHID2                 0xEE7AE9AA
#define X11_ORCHID_2                0xEE7AE9AA
#define X11_ORCHID3                 0xCD69C9AA
#define X11_ORCHID_3                0xCD69C9AA
#define X11_ORCHID4                 0x8B4789AA
#define X11_ORCHID_4                0x8B4789AA
#define X11_PLUM1                   0xFFBBFFAA
#define X11_PLUM_1                  0xFFBBFFAA
#define X11_PLUM2                   0xEEAEEEAA
#define X11_PLUM_2                  0xEEAEEEAA
#define X11_PLUM3                   0xCD96CDAA
#define X11_PLUM_3                  0xCD96CDAA
#define X11_PLUM4                   0x8B668BAA
#define X11_PLUM_4                  0x8B668BAA
#define X11_MEDIUMORCHID1           0xE066FFAA
#define X11_MEDIUM_ORCHID_1         0xE066FFAA
#define X11_MEDIUMORCHID2           0xD15FEEAA
#define X11_MEDIUM_ORCHID_2         0xD15FEEAA
#define X11_MEDIUMORCHID3           0xB452CDAA
#define X11_MEDIUM_ORCHID_3         0xB452CDAA
#define X11_MEDIUMORCHID4           0x7A378BAA
#define X11_MEDIUM_ORCHID_4         0x7A378BAA
#define X11_DARKORCHID1             0xBF3EFFAA
#define X11_DARK_ORCHID_1           0xBF3EFFAA
#define X11_DARKORCHID2             0xB23AEEAA
#define X11_DARK_ORCHID_2           0xB23AEEAA
#define X11_DARKORCHID3             0x9A32CDAA
#define X11_DARK_ORCHID_3           0x9A32CDAA
#define X11_DARKORCHID4             0x68228BAA
#define X11_DARK_ORCHID_4           0x68228BAA
#define X11_PURPLE1                 0x9B30FFAA
#define X11_PURPLE_1                0x9B30FFAA
#define X11_PURPLE2                 0x912CEEAA
#define X11_PURPLE_2                0x912CEEAA
#define X11_PURPLE3                 0x7D26CDAA
#define X11_PURPLE_3                0x7D26CDAA
#define X11_PURPLE4                 0x551A8BAA
#define X11_PURPLE_4                0x551A8BAA
#define X11_MEDIUMPURPLE1           0xAB82FFAA
#define X11_MEDIUM_PURPLE_1         0xAB82FFAA
#define X11_MEDIUMPURPLE2           0x9F79EEAA
#define X11_MEDIUM_PURPLE_2         0x9F79EEAA
#define X11_MEDIUMPURPLE3           0x8968CDAA
#define X11_MEDIUM_PURPLE_3         0x8968CDAA
#define X11_MEDIUMPURPLE4           0x5D478BAA
#define X11_MEDIUM_PURPLE_4         0x5D478BAA
#define X11_THISTLE1                0xFFE1FFAA
#define X11_THISTLE_1               0xFFE1FFAA
#define X11_THISTLE2                0xEED2EEAA
#define X11_THISTLE_2               0xEED2EEAA
#define X11_THISTLE3                0xCDB5CDAA
#define X11_THISTLE_3               0xCDB5CDAA
#define X11_THISTLE4                0x8B7B8BAA
#define X11_THISTLE_4               0x8B7B8BAA
#define X11_GRAY0                   0x000000AA
#define X11_GRAY_0                  0x000000AA
#define X11_GREY0                   0x000000AA
#define X11_GREY_0                  0x000000AA
#define X11_GRAY1                   0x030303AA
#define X11_GRAY_1                  0x030303AA
#define X11_GREY1                   0x030303AA
#define X11_GREY_1                  0x030303AA
#define X11_GRAY2                   0x050505AA
#define X11_GRAY_2                  0x050505AA
#define X11_GREY2                   0x050505AA
#define X11_GREY_2                  0x050505AA
#define X11_GRAY3                   0x080808AA
#define X11_GRAY_3                  0x080808AA
#define X11_GREY3                   0x080808AA
#define X11_GREY_3                  0x080808AA
#define X11_GRAY4                   0x0A0A0AAA
#define X11_GRAY_4                  0x0A0A0AAA
#define X11_GREY4                   0x0A0A0AAA
#define X11_GREY_4                  0x0A0A0AAA
#define X11_GRAY5                   0x0D0D0DAA
#define X11_GRAY_5                  0x0D0D0DAA
#define X11_GREY5                   0x0D0D0DAA
#define X11_GREY_5                  0x0D0D0DAA
#define X11_GRAY6                   0x0F0F0FAA
#define X11_GRAY_6                  0x0F0F0FAA
#define X11_GREY6                   0x0F0F0FAA
#define X11_GREY_6                  0x0F0F0FAA
#define X11_GRAY7                   0x121212AA
#define X11_GRAY_7                  0x121212AA
#define X11_GREY7                   0x121212AA
#define X11_GREY_7                  0x121212AA
#define X11_GRAY8                   0x141414AA
#define X11_GRAY_8                  0x141414AA
#define X11_GREY8                   0x141414AA
#define X11_GREY_8                  0x141414AA
#define X11_GRAY9                   0x171717AA
#define X11_GRAY_9                  0x171717AA
#define X11_GREY9                   0x171717AA
#define X11_GREY_9                  0x171717AA
#define X11_GRAY10                  0x1A1A1AAA
#define X11_GRAY_10                 0x1A1A1AAA
#define X11_GREY10                  0x1A1A1AAA
#define X11_GREY_10                 0x1A1A1AAA
#define X11_GRAY11                  0x1C1C1CAA
#define X11_GRAY_11                 0x1C1C1CAA
#define X11_GREY11                  0x1C1C1CAA
#define X11_GREY_11                 0x1C1C1CAA
#define X11_GRAY12                  0x1F1F1FAA
#define X11_GRAY_12                 0x1F1F1FAA
#define X11_GREY12                  0x1F1F1FAA
#define X11_GREY_12                 0x1F1F1FAA
#define X11_GRAY13                  0x212121AA
#define X11_GRAY_13                 0x212121AA
#define X11_GREY13                  0x212121AA
#define X11_GREY_13                 0x212121AA
#define X11_GRAY14                  0x242424AA
#define X11_GRAY_14                 0x242424AA
#define X11_GREY14                  0x242424AA
#define X11_GREY_14                 0x242424AA
#define X11_GRAY15                  0x262626AA
#define X11_GRAY_15                 0x262626AA
#define X11_GREY15                  0x262626AA
#define X11_GREY_15                 0x262626AA
#define X11_GRAY16                  0x292929AA
#define X11_GRAY_16                 0x292929AA
#define X11_GREY16                  0x292929AA
#define X11_GREY_16                 0x292929AA
#define X11_GRAY17                  0x2B2B2BAA
#define X11_GRAY_17                 0x2B2B2BAA
#define X11_GREY17                  0x2B2B2BAA
#define X11_GREY_17                 0x2B2B2BAA
#define X11_GRAY18                  0x2E2E2EAA
#define X11_GRAY_18                 0x2E2E2EAA
#define X11_GREY18                  0x2E2E2EAA
#define X11_GREY_18                 0x2E2E2EAA
#define X11_GRAY19                  0x303030AA
#define X11_GRAY_19                 0x303030AA
#define X11_GREY19                  0x303030AA
#define X11_GREY_19                 0x303030AA
#define X11_GRAY20                  0x333333AA
#define X11_GRAY_20                 0x333333AA
#define X11_GREY20                  0x333333AA
#define X11_GREY_20                 0x333333AA
#define X11_GRAY21                  0x363636AA
#define X11_GRAY_21                 0x363636AA
#define X11_GREY21                  0x363636AA
#define X11_GREY_21                 0x363636AA
#define X11_GRAY22                  0x383838AA
#define X11_GRAY_22                 0x383838AA
#define X11_GREY22                  0x383838AA
#define X11_GREY_22                 0x383838AA
#define X11_GRAY23                  0x3B3B3BAA
#define X11_GRAY_23                 0x3B3B3BAA
#define X11_GREY23                  0x3B3B3BAA
#define X11_GREY_23                 0x3B3B3BAA
#define X11_GRAY24                  0x3D3D3DAA
#define X11_GRAY_24                 0x3D3D3DAA
#define X11_GREY24                  0x3D3D3DAA
#define X11_GREY_24                 0x3D3D3DAA
#define X11_GRAY25                  0x404040AA
#define X11_GRAY_25                 0x404040AA
#define X11_GREY25                  0x404040AA
#define X11_GREY_25                 0x404040AA
#define X11_GRAY26                  0x424242AA
#define X11_GRAY_26                 0x424242AA
#define X11_GREY26                  0x424242AA
#define X11_GREY_26                 0x424242AA
#define X11_GRAY27                  0x454545AA
#define X11_GRAY_27                 0x454545AA
#define X11_GREY27                  0x454545AA
#define X11_GREY_27                 0x454545AA
#define X11_GRAY28                  0x474747AA
#define X11_GRAY_28                 0x474747AA
#define X11_GREY28                  0x474747AA
#define X11_GREY_28                 0x474747AA
#define X11_GRAY29                  0x4A4A4AAA
#define X11_GRAY_29                 0x4A4A4AAA
#define X11_GREY29                  0x4A4A4AAA
#define X11_GREY_29                 0x4A4A4AAA
#define X11_GRAY30                  0x4D4D4DAA
#define X11_GRAY_30                 0x4D4D4DAA
#define X11_GREY30                  0x4D4D4DAA
#define X11_GREY_30                 0x4D4D4DAA
#define X11_GRAY31                  0x4F4F4FAA
#define X11_GRAY_31                 0x4F4F4FAA
#define X11_GREY31                  0x4F4F4FAA
#define X11_GREY_31                 0x4F4F4FAA
#define X11_GRAY32                  0x525252AA
#define X11_GRAY_32                 0x525252AA
#define X11_GREY32                  0x525252AA
#define X11_GREY_32                 0x525252AA
#define X11_GRAY33                  0x545454AA
#define X11_GRAY_33                 0x545454AA
#define X11_GREY33                  0x545454AA
#define X11_GREY_33                 0x545454AA
#define X11_GRAY34                  0x575757AA
#define X11_GRAY_34                 0x575757AA
#define X11_GREY34                  0x575757AA
#define X11_GREY_34                 0x575757AA
#define X11_GRAY35                  0x595959AA
#define X11_GRAY_35                 0x595959AA
#define X11_GREY35                  0x595959AA
#define X11_GREY_35                 0x595959AA
#define X11_GRAY36                  0x5C5C5CAA
#define X11_GRAY_36                 0x5C5C5CAA
#define X11_GREY36                  0x5C5C5CAA
#define X11_GREY_36                 0x5C5C5CAA
#define X11_GRAY37                  0x5E5E5EAA
#define X11_GRAY_37                 0x5E5E5EAA
#define X11_GREY37                  0x5E5E5EAA
#define X11_GREY_37                 0x5E5E5EAA
#define X11_GRAY38                  0x616161AA
#define X11_GRAY_38                 0x616161AA
#define X11_GREY38                  0x616161AA
#define X11_GREY_38                 0x616161AA
#define X11_GRAY39                  0x636363AA
#define X11_GRAY_39                 0x636363AA
#define X11_GREY39                  0x636363AA
#define X11_GREY_39                 0x636363AA
#define X11_GRAY40                  0x666666AA
#define X11_GRAY_40                 0x666666AA
#define X11_GREY40                  0x666666AA
#define X11_GREY_40                 0x666666AA
#define X11_GRAY41                  0x696969AA
#define X11_GRAY_41                 0x696969AA
#define X11_GREY41                  0x696969AA
#define X11_GREY_41                 0x696969AA
#define X11_GRAY42                  0x6B6B6BAA
#define X11_GRAY_42                 0x6B6B6BAA
#define X11_GREY42                  0x6B6B6BAA
#define X11_GREY_42                 0x6B6B6BAA
#define X11_GRAY43                  0x6E6E6EAA
#define X11_GRAY_43                 0x6E6E6EAA
#define X11_GREY43                  0x6E6E6EAA
#define X11_GREY_43                 0x6E6E6EAA
#define X11_GRAY44                  0x707070AA
#define X11_GRAY_44                 0x707070AA
#define X11_GREY44                  0x707070AA
#define X11_GREY_44                 0x707070AA
#define X11_GRAY45                  0x737373AA
#define X11_GRAY_45                 0x737373AA
#define X11_GREY45                  0x737373AA
#define X11_GREY_45                 0x737373AA
#define X11_GRAY46                  0x757575AA
#define X11_GRAY_46                 0x757575AA
#define X11_GREY46                  0x757575AA
#define X11_GREY_46                 0x757575AA
#define X11_GRAY47                  0x787878AA
#define X11_GRAY_47                 0x787878AA
#define X11_GREY47                  0x787878AA
#define X11_GREY_47                 0x787878AA
#define X11_GRAY48                  0x7A7A7AAA
#define X11_GRAY_48                 0x7A7A7AAA
#define X11_GREY48                  0x7A7A7AAA
#define X11_GREY_48                 0x7A7A7AAA
#define X11_GRAY49                  0x7D7D7DAA
#define X11_GRAY_49                 0x7D7D7DAA
#define X11_GREY49                  0x7D7D7DAA
#define X11_GREY_49                 0x7D7D7DAA
#define X11_GRAY50                  0x7F7F7FAA
#define X11_GRAY_50                 0x7F7F7FAA
#define X11_GREY50                  0x7F7F7FAA
#define X11_GREY_50                 0x7F7F7FAA
#define X11_GRAY51                  0x828282AA
#define X11_GRAY_51                 0x828282AA
#define X11_GREY51                  0x828282AA
#define X11_GREY_51                 0x828282AA
#define X11_GRAY52                  0x858585AA
#define X11_GRAY_52                 0x858585AA
#define X11_GREY52                  0x858585AA
#define X11_GREY_52                 0x858585AA
#define X11_GRAY53                  0x878787AA
#define X11_GRAY_53                 0x878787AA
#define X11_GREY53                  0x878787AA
#define X11_GREY_53                 0x878787AA
#define X11_GRAY54                  0x8A8A8AAA
#define X11_GRAY_54                 0x8A8A8AAA
#define X11_GREY54                  0x8A8A8AAA
#define X11_GREY_54                 0x8A8A8AAA
#define X11_GRAY55                  0x8C8C8CAA
#define X11_GRAY_55                 0x8C8C8CAA
#define X11_GREY55                  0x8C8C8CAA
#define X11_GREY_55                 0x8C8C8CAA
#define X11_GRAY56                  0x8F8F8FAA
#define X11_GRAY_56                 0x8F8F8FAA
#define X11_GREY56                  0x8F8F8FAA
#define X11_GREY_56                 0x8F8F8FAA
#define X11_GRAY57                  0x919191AA
#define X11_GRAY_57                 0x919191AA
#define X11_GREY57                  0x919191AA
#define X11_GREY_57                 0x919191AA
#define X11_GRAY58                  0x949494AA
#define X11_GRAY_58                 0x949494AA
#define X11_GREY58                  0x949494AA
#define X11_GREY_58                 0x949494AA
#define X11_GRAY59                  0x969696AA
#define X11_GRAY_59                 0x969696AA
#define X11_GREY59                  0x969696AA
#define X11_GREY_59                 0x969696AA
#define X11_GRAY60                  0x999999AA
#define X11_GRAY_60                 0x999999AA
#define X11_GREY60                  0x999999AA
#define X11_GREY_60                 0x999999AA
#define X11_GRAY61                  0x9C9C9CAA
#define X11_GRAY_61                 0x9C9C9CAA
#define X11_GREY61                  0x9C9C9CAA
#define X11_GREY_61                 0x9C9C9CAA
#define X11_GRAY62                  0x9E9E9EAA
#define X11_GRAY_62                 0x9E9E9EAA
#define X11_GREY62                  0x9E9E9EAA
#define X11_GREY_62                 0x9E9E9EAA
#define X11_GRAY63                  0xA1A1A1AA
#define X11_GRAY_63                 0xA1A1A1AA
#define X11_GREY63                  0xA1A1A1AA
#define X11_GREY_63                 0xA1A1A1AA
#define X11_GRAY64                  0xA3A3A3AA
#define X11_GRAY_64                 0xA3A3A3AA
#define X11_GREY64                  0xA3A3A3AA
#define X11_GREY_64                 0xA3A3A3AA
#define X11_GRAY65                  0xA6A6A6AA
#define X11_GRAY_65                 0xA6A6A6AA
#define X11_GREY65                  0xA6A6A6AA
#define X11_GREY_65                 0xA6A6A6AA
#define X11_GRAY66                  0xA8A8A8AA
#define X11_GRAY_66                 0xA8A8A8AA
#define X11_GREY66                  0xA8A8A8AA
#define X11_GREY_66                 0xA8A8A8AA
#define X11_GRAY67                  0xABABABAA
#define X11_GRAY_67                 0xABABABAA
#define X11_GREY67                  0xABABABAA
#define X11_GREY_67                 0xABABABAA
#define X11_GRAY68                  0xADADADAA
#define X11_GRAY_68                 0xADADADAA
#define X11_GREY68                  0xADADADAA
#define X11_GREY_68                 0xADADADAA
#define X11_GRAY69                  0xB0B0B0AA
#define X11_GRAY_69                 0xB0B0B0AA
#define X11_GREY69                  0xB0B0B0AA
#define X11_GREY_69                 0xB0B0B0AA
#define X11_GRAY70                  0xB3B3B3AA
#define X11_GRAY_70                 0xB3B3B3AA
#define X11_GREY70                  0xB3B3B3AA
#define X11_GREY_70                 0xB3B3B3AA
#define X11_GRAY71                  0xB5B5B5AA
#define X11_GRAY_71                 0xB5B5B5AA
#define X11_GREY71                  0xB5B5B5AA
#define X11_GREY_71                 0xB5B5B5AA
#define X11_GRAY72                  0xB8B8B8AA
#define X11_GRAY_72                 0xB8B8B8AA
#define X11_GREY72                  0xB8B8B8AA
#define X11_GREY_72                 0xB8B8B8AA
#define X11_GRAY73                  0xBABABAAA
#define X11_GRAY_73                 0xBABABAAA
#define X11_GREY73                  0xBABABAAA
#define X11_GREY_73                 0xBABABAAA
#define X11_GRAY74                  0xBDBDBDAA
#define X11_GRAY_74                 0xBDBDBDAA
#define X11_GREY74                  0xBDBDBDAA
#define X11_GREY_74                 0xBDBDBDAA
#define X11_GRAY75                  0xBFBFBFAA
#define X11_GRAY_75                 0xBFBFBFAA
#define X11_GREY75                  0xBFBFBFAA
#define X11_GREY_75                 0xBFBFBFAA
#define X11_GRAY76                  0xC2C2C2AA
#define X11_GRAY_76                 0xC2C2C2AA
#define X11_GREY76                  0xC2C2C2AA
#define X11_GREY_76                 0xC2C2C2AA
#define X11_GRAY77                  0xC4C4C4AA
#define X11_GRAY_77                 0xC4C4C4AA
#define X11_GREY77                  0xC4C4C4AA
#define X11_GREY_77                 0xC4C4C4AA
#define X11_GRAY78                  0xC7C7C7AA
#define X11_GRAY_78                 0xC7C7C7AA
#define X11_GREY78                  0xC7C7C7AA
#define X11_GREY_78                 0xC7C7C7AA
#define X11_GRAY79                  0xC9C9C9AA
#define X11_GRAY_79                 0xC9C9C9AA
#define X11_GREY79                  0xC9C9C9AA
#define X11_GREY_79                 0xC9C9C9AA
#define X11_GRAY80                  0xCCCCCCAA
#define X11_GRAY_80                 0xCCCCCCAA
#define X11_GREY80                  0xCCCCCCAA
#define X11_GREY_80                 0xCCCCCCAA
#define X11_GRAY81                  0xCFCFCFAA
#define X11_GRAY_81                 0xCFCFCFAA
#define X11_GREY81                  0xCFCFCFAA
#define X11_GREY_81                 0xCFCFCFAA
#define X11_GRAY82                  0xD1D1D1AA
#define X11_GRAY_82                 0xD1D1D1AA
#define X11_GREY82                  0xD1D1D1AA
#define X11_GREY_82                 0xD1D1D1AA
#define X11_GRAY83                  0xD4D4D4AA
#define X11_GRAY_83                 0xD4D4D4AA
#define X11_GREY83                  0xD4D4D4AA
#define X11_GREY_83                 0xD4D4D4AA
#define X11_GRAY84                  0xD6D6D6AA
#define X11_GRAY_84                 0xD6D6D6AA
#define X11_GREY84                  0xD6D6D6AA
#define X11_GREY_84                 0xD6D6D6AA
#define X11_GRAY85                  0xD9D9D9AA
#define X11_GRAY_85                 0xD9D9D9AA
#define X11_GREY85                  0xD9D9D9AA
#define X11_GREY_85                 0xD9D9D9AA
#define X11_GRAY86                  0xDBDBDBAA
#define X11_GRAY_86                 0xDBDBDBAA
#define X11_GREY86                  0xDBDBDBAA
#define X11_GREY_86                 0xDBDBDBAA
#define X11_GRAY87                  0xDEDEDEAA
#define X11_GRAY_87                 0xDEDEDEAA
#define X11_GREY87                  0xDEDEDEAA
#define X11_GREY_87                 0xDEDEDEAA
#define X11_GRAY88                  0xE0E0E0AA
#define X11_GRAY_88                 0xE0E0E0AA
#define X11_GREY88                  0xE0E0E0AA
#define X11_GREY_88                 0xE0E0E0AA
#define X11_GRAY89                  0xE3E3E3AA
#define X11_GRAY_89                 0xE3E3E3AA
#define X11_GREY89                  0xE3E3E3AA
#define X11_GREY_89                 0xE3E3E3AA
#define X11_GRAY90                  0xE5E5E5AA
#define X11_GRAY_90                 0xE5E5E5AA
#define X11_GREY90                  0xE5E5E5AA
#define X11_GREY_90                 0xE5E5E5AA
#define X11_GRAY91                  0xE8E8E8AA
#define X11_GRAY_91                 0xE8E8E8AA
#define X11_GREY91                  0xE8E8E8AA
#define X11_GREY_91                 0xE8E8E8AA
#define X11_GRAY92                  0xEBEBEBAA
#define X11_GRAY_92                 0xEBEBEBAA
#define X11_GREY92                  0xEBEBEBAA
#define X11_GREY_92                 0xEBEBEBAA
#define X11_GRAY93                  0xEDEDEDAA
#define X11_GRAY_93                 0xEDEDEDAA
#define X11_GREY93                  0xEDEDEDAA
#define X11_GREY_93                 0xEDEDEDAA
#define X11_GRAY94                  0xF0F0F0AA
#define X11_GRAY_94                 0xF0F0F0AA
#define X11_GREY94                  0xF0F0F0AA
#define X11_GREY_94                 0xF0F0F0AA
#define X11_GRAY95                  0xF2F2F2AA
#define X11_GRAY_95                 0xF2F2F2AA
#define X11_GREY95                  0xF2F2F2AA
#define X11_GREY_95                 0xF2F2F2AA
#define X11_GRAY96                  0xF5F5F5AA
#define X11_GRAY_96                 0xF5F5F5AA
#define X11_GREY96                  0xF5F5F5AA
#define X11_GREY_96                 0xF5F5F5AA
#define X11_GRAY97                  0xF7F7F7AA
#define X11_GRAY_97                 0xF7F7F7AA
#define X11_GREY97                  0xF7F7F7AA
#define X11_GREY_97                 0xF7F7F7AA
#define X11_GRAY98                  0xFAFAFAAA
#define X11_GRAY_98                 0xFAFAFAAA
#define X11_GREY98                  0xFAFAFAAA
#define X11_GREY_98                 0xFAFAFAAA
#define X11_GRAY99                  0xFCFCFCAA
#define X11_GRAY_99                 0xFCFCFCAA
#define X11_GREY99                  0xFCFCFCAA
#define X11_GREY_99                 0xFCFCFCAA
#define X11_GRAY100                 0xFFFFFFAA
#define X11_GRAY_100                0xFFFFFFAA
#define X11_GREY100                 0xFFFFFFAA
#define X11_GREY_100                0xFFFFFFAA
#define X11_DARKGREY                0xA9A9A9AA
#define X11_DARK_GREY               0xA9A9A9AA
#define X11_DARKGRAY                0xA9A9A9AA
#define X11_DARK_GRAY               0xA9A9A9AA
#define X11_DARKBLUE                0x00008BAA
#define X11_DARK_BLUE               0x00008BAA
#define X11_DARKCYAN                0x008B8BAA
#define X11_DARK_CYAN               0x008B8BAA
#define X11_DARKMAGENTA             0x8B008BAA
#define X11_DARK_MAGENTA            0x8B008BAA
#define X11_DARKRED                 0x8B0000AA
#define X11_DARK_RED                0x8B0000AA
#define X11_LIGHTGREEN              0x90EE90AA
#define X11_LIGHT_GREEN             0x90EE90AA

// Embeddable colours.
#define SNOW                        "{FFFAFA}"
#define GHOSTWHITE                  "{F8F8FF}"
#define GHOST_WHITE                 "{F8F8FF}"
#define WHITESMOKE                  "{F5F5F5}"
#define WHITE_SMOKE                 "{F5F5F5}"
#define GAINSBORO                   "{DCDCDC}"
#define FLORALWHITE                 "{FFFAF0}"
#define FLORAL_WHITE                "{FFFAF0}"
#define OLDLACE                     "{FDF5E6}"
#define OLD_LACE                    "{FDF5E6}"
#define LINEN                       "{FAF0E6}"
#define ANTIQUEWHITE                "{FAEBD7}"
#define ANTIQUE_WHITE               "{FAEBD7}"
#define PAPAYAWHIP                  "{FFEFD5}"
#define PAPAYA_WHIP                 "{FFEFD5}"
#define BLANCHEDALMOND              "{FFEBCD}"
#define BLANCHED_ALMOND             "{FFEBCD}"
#define BISQUE                      "{FFE4C4}"
#define PEACHPUFF                   "{FFDAB9}"
#define PEACH_PUFF                  "{FFDAB9}"
#define NAVAJOWHITE                 "{FFDEAD}"
#define NAVAJO_WHITE                "{FFDEAD}"
#define MOCCASIN                    "{FFE4B5}"
#define CORNSILK                    "{FFF8DC}"
#define IVORY                       "{FFFFF0}"
#define LEMONCHIFFON                "{FFFACD}"
#define LEMON_CHIFFON               "{FFFACD}"
#define SEASHELL                    "{FFF5EE}"
#define HONEYDEW                    "{F0FFF0}"
#define MINTCREAM                   "{F5FFFA}"
#define MINT_CREAM                  "{F5FFFA}"
#define AZURE                       "{F0FFFF}"
#define ALICEBLUE                   "{F0F8FF}"
#define ALICE_BLUE                  "{F0F8FF}"
#define LAVENDER                    "{E6E6FA}"
#define LAVENDERBLUSH               "{FFF0F5}"
#define LAVENDER_BLUSH              "{FFF0F5}"
#define MISTYROSE                   "{FFE4E1}"
#define MISTY_ROSE                  "{FFE4E1}"
#define WHITE                       "{FFFFFF}"
#define BLACK                       "{000000}"
#define DARKSLATEGRAY               "{2F4F4F}"
#define DARK_SLATE_GRAY             "{2F4F4F}"
#define DARKSLATEGREY               "{2F4F4F}"
#define DARK_SLATE_GREY             "{2F4F4F}"
#define DIMGRAY                     "{696969}"
#define DIM_GRAY                    "{696969}"
#define DIMGREY                     "{696969}"
#define DIM_GREY                    "{696969}"
#define SLATEGRAY                   "{708090}"
#define SLATE_GRAY                  "{708090}"
#define SLATEGREY                   "{708090}"
#define SLATE_GREY                  "{708090}"
#define LIGHTSLATEGRAY              "{778899}"
#define LIGHT_SLATE_GRAY            "{778899}"
#define LIGHTSLATEGREY              "{778899}"
#define LIGHT_SLATE_GREY            "{778899}"
#define GRAY                        "{BEBEBE}"
#define GREY                        "{BEBEBE}"
#define LIGHTGREY                   "{D3D3D3}"
#define LIGHT_GREY                  "{D3D3D3}"
#define LIGHTGRAY                   "{D3D3D3}"
#define LIGHT_GRAY                  "{D3D3D3}"
#define MIDNIGHTBLUE                "{191970}"
#define MIDNIGHT_BLUE               "{191970}"
#define NAVY                        "{000080}"
#define NAVYBLUE                    "{000080}"
#define NAVY_BLUE                   "{000080}"
#define CORNFLOWERBLUE              "{6495ED}"
#define CORNFLOWER_BLUE             "{6495ED}"
#define DARKSLATEBLUE               "{483D8B}"
#define DARK_SLATE_BLUE             "{483D8B}"
#define SLATEBLUE                   "{6A5ACD}"
#define SLATE_BLUE                  "{6A5ACD}"
#define MEDIUMSLATEBLUE             "{7B68EE}"
#define MEDIUM_SLATE_BLUE           "{7B68EE}"
#define LIGHTSLATEBLUE              "{8470FF}"
#define LIGHT_SLATE_BLUE            "{8470FF}"
#define MEDIUMBLUE                  "{0000CD}"
#define MEDIUM_BLUE                 "{0000CD}"
#define ROYALBLUE                   "{4169E1}"
#define ROYAL_BLUE                  "{4169E1}"
#define BLUE                        "{0000FF}"
#define DODGERBLUE                  "{1E90FF}"
#define DODGER_BLUE                 "{1E90FF}"
#define DEEPSKYBLUE                 "{00BFFF}"
#define DEEP_SKY_BLUE               "{00BFFF}"
#define SKYBLUE                     "{87CEEB}"
#define SKY_BLUE                    "{87CEEB}"
#define LIGHTSKYBLUE                "{87CEFA}"
#define LIGHT_SKY_BLUE              "{87CEFA}"
#define STEELBLUE                   "{4682B4}"
#define STEEL_BLUE                  "{4682B4}"
#define LIGHTSTEELBLUE              "{B0C4DE}"
#define LIGHT_STEEL_BLUE            "{B0C4DE}"
#define LIGHTBLUE                   "{ADD8E6}"
#define LIGHT_BLUE                  "{ADD8E6}"
#define POWDERBLUE                  "{B0E0E6}"
#define POWDER_BLUE                 "{B0E0E6}"
#define PALETURQUOISE               "{AFEEEE}"
#define PALE_TURQUOISE              "{AFEEEE}"
#define DARKTURQUOISE               "{00CED1}"
#define DARK_TURQUOISE              "{00CED1}"
#define MEDIUMTURQUOISE             "{48D1CC}"
#define MEDIUM_TURQUOISE            "{48D1CC}"
#define TURQUOISE                   "{40E0D0}"
#define CYAN                        "{00FFFF}"
#define LIGHTCYAN                   "{E0FFFF}"
#define LIGHT_CYAN                  "{E0FFFF}"
#define CADETBLUE                   "{5F9EA0}"
#define CADET_BLUE                  "{5F9EA0}"
#define MEDIUMAQUAMARINE            "{66CDAA}"
#define MEDIUM_AQUAMARINE           "{66CDAA}"
#define AQUAMARINE                  "{7FFFD4}"
#define DARKGREEN                   "{006400}"
#define DARK_GREEN                  "{006400}"
#define DARKOLIVEGREEN              "{556B2F}"
#define DARK_OLIVE_GREEN            "{556B2F}"
#define DARKSEAGREEN                "{8FBC8F}"
#define DARK_SEA_GREEN              "{8FBC8F}"
#define SEAGREEN                    "{2E8B57}"
#define SEA_GREEN                   "{2E8B57}"
#define MEDIUMSEAGREEN              "{3CB371}"
#define MEDIUM_SEA_GREEN            "{3CB371}"
#define LIGHTSEAGREEN               "{20B2AA}"
#define LIGHT_SEA_GREEN             "{20B2AA}"
#define PALEGREEN                   "{98FB98}"
#define PALE_GREEN                  "{98FB98}"
#define SPRINGGREEN                 "{00FF7F}"
#define SPRING_GREEN                "{00FF7F}"
#define LAWNGREEN                   "{7CFC00}"
#define LAWN_GREEN                  "{7CFC00}"
#define GREEN                       "{00FF00}"
#define CHARTREUSE                  "{7FFF00}"
#define MEDIUMSPRINGGREEN           "{00FA9A}"
#define MEDIUM_SPRING_GREEN         "{00FA9A}"
#define GREENYELLOW                 "{ADFF2F}"
#define GREEN_YELLOW                "{ADFF2F}"
#define LIMEGREEN                   "{32CD32}"
#define LIME_GREEN                  "{32CD32}"
#define YELLOWGREEN                 "{9ACD32}"
#define YELLOW_GREEN                "{9ACD32}"
#define FORESTGREEN                 "{228B22}"
#define FOREST_GREEN                "{228B22}"
#define OLIVEDRAB                   "{6B8E23}"
#define OLIVE_DRAB                  "{6B8E23}"
#define DARKKHAKI                   "{BDB76B}"
#define DARK_KHAKI                  "{BDB76B}"
#define KHAKI                       "{F0E68C}"
#define PALEGOLDENROD               "{EEE8AA}"
#define PALE_GOLDENROD              "{EEE8AA}"
#define LIGHTGOLDENRODYELLOW        "{FAFAD2}"
#define LIGHT_GOLDENROD_YELLOW      "{FAFAD2}"
#define LIGHTYELLOW                 "{FFFFE0}"
#define LIGHT_YELLOW                "{FFFFE0}"
#define YELLOW                      "{FFFF00}"
#define GOLD                        "{FFD700}"
#define LIGHTGOLDENROD              "{EEDD82}"
#define LIGHT_GOLDENROD             "{EEDD82}"
#define GOLDENROD                   "{DAA520}"
#define DARKGOLDENROD               "{B8860B}"
#define DARK_GOLDENROD              "{B8860B}"
#define ROSYBROWN                   "{BC8F8F}"
#define ROSY_BROWN                  "{BC8F8F}"
#define INDIANRED                   "{CD5C5C}"
#define INDIAN_RED                  "{CD5C5C}"
#define SADDLEBROWN                 "{8B4513}"
#define SADDLE_BROWN                "{8B4513}"
#define SIENNA                      "{A0522D}"
#define PERU                        "{CD853F}"
#define BURLYWOOD                   "{DEB887}"
#define BEIGE                       "{F5F5DC}"
#define WHEAT                       "{F5DEB3}"
#define SANDYBROWN                  "{F4A460}"
#define SANDY_BROWN                 "{F4A460}"
#define TAN                         "{D2B48C}"
#define CHOCOLATE                   "{D2691E}"
#define FIREBRICK                   "{B22222}"
#define BROWN                       "{A52A2A}"
#define DARKSALMON                  "{E9967A}"
#define DARK_SALMON                 "{E9967A}"
#define SALMON                      "{FA8072}"
#define LIGHTSALMON                 "{FFA07A}"
#define LIGHT_SALMON                "{FFA07A}"
#define ORANGE                      "{FFA500}"
#define DARKORANGE                  "{FF8C00}"
#define DARK_ORANGE                 "{FF8C00}"
#define CORAL                       "{FF7F50}"
#define LIGHTCORAL                  "{F08080}"
#define LIGHT_CORAL                 "{F08080}"
#define TOMATO                      "{FF6347}"
#define ORANGERED                   "{FF4500}"
#define ORANGE_RED                  "{FF4500}"
#define RED                         "{FF0000}"
#define HOTPINK                     "{FF69B4}"
#define HOT_PINK                    "{FF69B4}"
#define DEEPPINK                    "{FF1493}"
#define DEEP_PINK                   "{FF1493}"
#define PINK                        "{FFC0CB}"
#define LIGHTPINK                   "{FFB6C1}"
#define LIGHT_PINK                  "{FFB6C1}"
#define PALEVIOLETRED               "{DB7093}"
#define PALE_VIOLET_RED             "{DB7093}"
#define MAROON                      "{B03060}"
#define MEDIUMVIOLETRED             "{C71585}"
#define MEDIUM_VIOLET_RED           "{C71585}"
#define VIOLETRED                   "{D02090}"
#define VIOLET_RED                  "{D02090}"
#define MAGENTA                     "{FF00FF}"
#define VIOLET                      "{EE82EE}"
#define PLUM                        "{DDA0DD}"
#define ORCHID                      "{DA70D6}"
#define MEDIUMORCHID                "{BA55D3}"
#define MEDIUM_ORCHID               "{BA55D3}"
#define DARKORCHID                  "{9932CC}"
#define DARK_ORCHID                 "{9932CC}"
#define DARKVIOLET                  "{9400D3}"
#define DARK_VIOLET                 "{9400D3}"
#define BLUEVIOLET                  "{8A2BE2}"
#define BLUE_VIOLET                 "{8A2BE2}"
#define PURPLE                      "{A020F0}"
#define MEDIUMPURPLE                "{9370DB}"
#define MEDIUM_PURPLE               "{9370DB}"
#define THISTLE                     "{D8BFD8}"
#define SNOW1                       "{FFFAFA}"
#define SNOW_1                      "{FFFAFA}"
#define SNOW2                       "{EEE9E9}"
#define SNOW_2                      "{EEE9E9}"
#define SNOW3                       "{CDC9C9}"
#define SNOW_3                      "{CDC9C9}"
#define SNOW4                       "{8B8989}"
#define SNOW_4                      "{8B8989}"
#define SEASHELL1                   "{FFF5EE}"
#define SEASHELL_1                  "{FFF5EE}"
#define SEASHELL2                   "{EEE5DE}"
#define SEASHELL_2                  "{EEE5DE}"
#define SEASHELL3                   "{CDC5BF}"
#define SEASHELL_3                  "{CDC5BF}"
#define SEASHELL4                   "{8B8682}"
#define SEASHELL_4                  "{8B8682}"
#define ANTIQUEWHITE1               "{FFEFDB}"
#define ANTIQUE_WHITE_1             "{FFEFDB}"
#define ANTIQUEWHITE2               "{EEDFCC}"
#define ANTIQUE_WHITE_2             "{EEDFCC}"
#define ANTIQUEWHITE3               "{CDC0B0}"
#define ANTIQUE_WHITE_3             "{CDC0B0}"
#define ANTIQUEWHITE4               "{8B8378}"
#define ANTIQUE_WHITE_4             "{8B8378}"
#define BISQUE1                     "{FFE4C4}"
#define BISQUE_1                    "{FFE4C4}"
#define BISQUE2                     "{EED5B7}"
#define BISQUE_2                    "{EED5B7}"
#define BISQUE3                     "{CDB79E}"
#define BISQUE_3                    "{CDB79E}"
#define BISQUE4                     "{8B7D6B}"
#define BISQUE_4                    "{8B7D6B}"
#define PEACHPUFF1                  "{FFDAB9}"
#define PEACH_PUFF_1                "{FFDAB9}"
#define PEACHPUFF2                  "{EECBAD}"
#define PEACH_PUFF_2                "{EECBAD}"
#define PEACHPUFF3                  "{CDAF95}"
#define PEACH_PUFF_3                "{CDAF95}"
#define PEACHPUFF4                  "{8B7765}"
#define PEACH_PUFF_4                "{8B7765}"
#define NAVAJOWHITE1                "{FFDEAD}"
#define NAVAJO_WHITE_1              "{FFDEAD}"
#define NAVAJOWHITE2                "{EECFA1}"
#define NAVAJO_WHITE_2              "{EECFA1}"
#define NAVAJOWHITE3                "{CDB38B}"
#define NAVAJO_WHITE_3              "{CDB38B}"
#define NAVAJOWHITE4                "{8B795E}"
#define NAVAJO_WHITE_4              "{8B795E}"
#define LEMONCHIFFON1               "{FFFACD}"
#define LEMON_CHIFFON_1             "{FFFACD}"
#define LEMONCHIFFON2               "{EEE9BF}"
#define LEMON_CHIFFON_2             "{EEE9BF}"
#define LEMONCHIFFON3               "{CDC9A5}"
#define LEMON_CHIFFON_3             "{CDC9A5}"
#define LEMONCHIFFON4               "{8B8970}"
#define LEMON_CHIFFON_4             "{8B8970}"
#define CORNSILK1                   "{FFF8DC}"
#define CORNSILK_1                  "{FFF8DC}"
#define CORNSILK2                   "{EEE8CD}"
#define CORNSILK_2                  "{EEE8CD}"
#define CORNSILK3                   "{CDC8B1}"
#define CORNSILK_3                  "{CDC8B1}"
#define CORNSILK4                   "{8B8878}"
#define CORNSILK_4                  "{8B8878}"
#define IVORY1                      "{FFFFF0}"
#define IVORY_1                     "{FFFFF0}"
#define IVORY2                      "{EEEEE0}"
#define IVORY_2                     "{EEEEE0}"
#define IVORY3                      "{CDCDC1}"
#define IVORY_3                     "{CDCDC1}"
#define IVORY4                      "{8B8B83}"
#define IVORY_4                     "{8B8B83}"
#define HONEYDEW1                   "{F0FFF0}"
#define HONEYDEW_1                  "{F0FFF0}"
#define HONEYDEW2                   "{E0EEE0}"
#define HONEYDEW_2                  "{E0EEE0}"
#define HONEYDEW3                   "{C1CDC1}"
#define HONEYDEW_3                  "{C1CDC1}"
#define HONEYDEW4                   "{838B83}"
#define HONEYDEW_4                  "{838B83}"
#define LAVENDERBLUSH1              "{FFF0F5}"
#define LAVENDER_BLUSH_1            "{FFF0F5}"
#define LAVENDERBLUSH2              "{EEE0E5}"
#define LAVENDER_BLUSH_2            "{EEE0E5}"
#define LAVENDERBLUSH3              "{CDC1C5}"
#define LAVENDER_BLUSH_3            "{CDC1C5}"
#define LAVENDERBLUSH4              "{8B8386}"
#define LAVENDER_BLUSH_4            "{8B8386}"
#define MISTYROSE1                  "{FFE4E1}"
#define MISTY_ROSE_1                "{FFE4E1}"
#define MISTYROSE2                  "{EED5D2}"
#define MISTY_ROSE_2                "{EED5D2}"
#define MISTYROSE3                  "{CDB7B5}"
#define MISTY_ROSE_3                "{CDB7B5}"
#define MISTYROSE4                  "{8B7D7B}"
#define MISTY_ROSE_4                "{8B7D7B}"
#define AZURE1                      "{F0FFFF}"
#define AZURE_1                     "{F0FFFF}"
#define AZURE2                      "{E0EEEE}"
#define AZURE_2                     "{E0EEEE}"
#define AZURE3                      "{C1CDCD}"
#define AZURE_3                     "{C1CDCD}"
#define AZURE4                      "{838B8B}"
#define AZURE_4                     "{838B8B}"
#define SLATEBLUE1                  "{836FFF}"
#define SLATE_BLUE_1                "{836FFF}"
#define SLATEBLUE2                  "{7A67EE}"
#define SLATE_BLUE_2                "{7A67EE}"
#define SLATEBLUE3                  "{6959CD}"
#define SLATE_BLUE_3                "{6959CD}"
#define SLATEBLUE4                  "{473C8B}"
#define SLATE_BLUE_4                "{473C8B}"
#define ROYALBLUE1                  "{4876FF}"
#define ROYAL_BLUE_1                "{4876FF}"
#define ROYALBLUE2                  "{436EEE}"
#define ROYAL_BLUE_2                "{436EEE}"
#define ROYALBLUE3                  "{3A5FCD}"
#define ROYAL_BLUE_3                "{3A5FCD}"
#define ROYALBLUE4                  "{27408B}"
#define ROYAL_BLUE_4                "{27408B}"
#define BLUE1                       "{0000FF}"
#define BLUE_1                      "{0000FF}"
#define BLUE2                       "{0000EE}"
#define BLUE_2                      "{0000EE}"
#define BLUE3                       "{0000CD}"
#define BLUE_3                      "{0000CD}"
#define BLUE4                       "{00008B}"
#define BLUE_4                      "{00008B}"
#define DODGERBLUE1                 "{1E90FF}"
#define DODGER_BLUE_1               "{1E90FF}"
#define DODGERBLUE2                 "{1C86EE}"
#define DODGER_BLUE_2               "{1C86EE}"
#define DODGERBLUE3                 "{1874CD}"
#define DODGER_BLUE_3               "{1874CD}"
#define DODGERBLUE4                 "{104E8B}"
#define DODGER_BLUE_4               "{104E8B}"
#define STEELBLUE1                  "{63B8FF}"
#define STEEL_BLUE_1                "{63B8FF}"
#define STEELBLUE2                  "{5CACEE}"
#define STEEL_BLUE_2                "{5CACEE}"
#define STEELBLUE3                  "{4F94CD}"
#define STEEL_BLUE_3                "{4F94CD}"
#define STEELBLUE4                  "{36648B}"
#define STEEL_BLUE_4                "{36648B}"
#define DEEPSKYBLUE1                "{00BFFF}"
#define DEEP_SKY_BLUE_1             "{00BFFF}"
#define DEEPSKYBLUE2                "{00B2EE}"
#define DEEP_SKY_BLUE_2             "{00B2EE}"
#define DEEPSKYBLUE3                "{009ACD}"
#define DEEP_SKY_BLUE_3             "{009ACD}"
#define DEEPSKYBLUE4                "{00688B}"
#define DEEP_SKY_BLUE_4             "{00688B}"
#define SKYBLUE1                    "{87CEFF}"
#define SKY_BLUE_1                  "{87CEFF}"
#define SKYBLUE2                    "{7EC0EE}"
#define SKY_BLUE_2                  "{7EC0EE}"
#define SKYBLUE3                    "{6CA6CD}"
#define SKY_BLUE_3                  "{6CA6CD}"
#define SKYBLUE4                    "{4A708B}"
#define SKY_BLUE_4                  "{4A708B}"
#define LIGHTSKYBLUE1               "{B0E2FF}"
#define LIGHT_SKY_BLUE_1            "{B0E2FF}"
#define LIGHTSKYBLUE2               "{A4D3EE}"
#define LIGHT_SKY_BLUE_2            "{A4D3EE}"
#define LIGHTSKYBLUE3               "{8DB6CD}"
#define LIGHT_SKY_BLUE_3            "{8DB6CD}"
#define LIGHTSKYBLUE4               "{607B8B}"
#define LIGHT_SKY_BLUE_4            "{607B8B}"
#define SLATEGRAY1                  "{C6E2FF}"
#define SLATE_GRAY_1                "{C6E2FF}"
#define SLATEGRAY2                  "{B9D3EE}"
#define SLATE_GRAY_2                "{B9D3EE}"
#define SLATEGRAY3                  "{9FB6CD}"
#define SLATE_GRAY_3                "{9FB6CD}"
#define SLATEGRAY4                  "{6C7B8B}"
#define SLATE_GRAY_4                "{6C7B8B}"
#define SLATEGREY1                  "{C6E2FF}"
#define SLATE_GREY_1                "{C6E2FF}"
#define SLATEGREY2                  "{B9D3EE}"
#define SLATE_GREY_2                "{B9D3EE}"
#define SLATEGREY3                  "{9FB6CD}"
#define SLATE_GREY_3                "{9FB6CD}"
#define SLATEGREY4                  "{6C7B8B}"
#define SLATE_GREY_4                "{6C7B8B}"
#define LIGHTSTEELBLUE1             "{CAE1FF}"
#define LIGHT_STEEL_BLUE_1          "{CAE1FF}"
#define LIGHTSTEELBLUE2             "{BCD2EE}"
#define LIGHT_STEEL_BLUE_2          "{BCD2EE}"
#define LIGHTSTEELBLUE3             "{A2B5CD}"
#define LIGHT_STEEL_BLUE_3          "{A2B5CD}"
#define LIGHTSTEELBLUE4             "{6E7B8B}"
#define LIGHT_STEEL_BLUE_4          "{6E7B8B}"
#define LIGHTBLUE1                  "{BFEFFF}"
#define LIGHT_BLUE_1                "{BFEFFF}"
#define LIGHTBLUE2                  "{B2DFEE}"
#define LIGHT_BLUE_2                "{B2DFEE}"
#define LIGHTBLUE3                  "{9AC0CD}"
#define LIGHT_BLUE_3                "{9AC0CD}"
#define LIGHTBLUE4                  "{68838B}"
#define LIGHT_BLUE_4                "{68838B}"
#define LIGHTCYAN1                  "{E0FFFF}"
#define LIGHT_CYAN_1                "{E0FFFF}"
#define LIGHTCYAN2                  "{D1EEEE}"
#define LIGHT_CYAN_2                "{D1EEEE}"
#define LIGHTCYAN3                  "{B4CDCD}"
#define LIGHT_CYAN_3                "{B4CDCD}"
#define LIGHTCYAN4                  "{7A8B8B}"
#define LIGHT_CYAN_4                "{7A8B8B}"
#define PALETURQUOISE1              "{BBFFFF}"
#define PALE_TURQUOISE_1            "{BBFFFF}"
#define PALETURQUOISE2              "{AEEEEE}"
#define PALE_TURQUOISE_2            "{AEEEEE}"
#define PALETURQUOISE3              "{96CDCD}"
#define PALE_TURQUOISE_3            "{96CDCD}"
#define PALETURQUOISE4              "{668B8B}"
#define PALE_TURQUOISE_4            "{668B8B}"
#define CADETBLUE1                  "{98F5FF}"
#define CADET_BLUE_1                "{98F5FF}"
#define CADETBLUE2                  "{8EE5EE}"
#define CADET_BLUE_2                "{8EE5EE}"
#define CADETBLUE3                  "{7AC5CD}"
#define CADET_BLUE_3                "{7AC5CD}"
#define CADETBLUE4                  "{53868B}"
#define CADET_BLUE_4                "{53868B}"
#define TURQUOISE1                  "{00F5FF}"
#define TURQUOISE_1                 "{00F5FF}"
#define TURQUOISE2                  "{00E5EE}"
#define TURQUOISE_2                 "{00E5EE}"
#define TURQUOISE3                  "{00C5CD}"
#define TURQUOISE_3                 "{00C5CD}"
#define TURQUOISE4                  "{00868B}"
#define TURQUOISE_4                 "{00868B}"
#define CYAN1                       "{00FFFF}"
#define CYAN_1                      "{00FFFF}"
#define CYAN2                       "{00EEEE}"
#define CYAN_2                      "{00EEEE}"
#define CYAN3                       "{00CDCD}"
#define CYAN_3                      "{00CDCD}"
#define CYAN4                       "{008B8B}"
#define CYAN_4                      "{008B8B}"
#define DARKSLATEGRAY1              "{97FFFF}"
#define DARK_SLATE_GRAY_1           "{97FFFF}"
#define DARKSLATEGRAY2              "{8DEEEE}"
#define DARK_SLATE_GRAY_2           "{8DEEEE}"
#define DARKSLATEGRAY3              "{79CDCD}"
#define DARK_SLATE_GRAY_3           "{79CDCD}"
#define DARKSLATEGRAY4              "{528B8B}"
#define DARK_SLATE_GRAY_4           "{528B8B}"
#define DARKSLATEGREY1              "{97FFFF}"
#define DARK_SLATE_GREY_1           "{97FFFF}"
#define DARKSLATEGREY2              "{8DEEEE}"
#define DARK_SLATE_GREY_2           "{8DEEEE}"
#define DARKSLATEGREY3              "{79CDCD}"
#define DARK_SLATE_GREY_3           "{79CDCD}"
#define DARKSLATEGREY4              "{528B8B}"
#define DARK_SLATE_GREY_4           "{528B8B}"
#define AQUAMARINE1                 "{7FFFD4}"
#define AQUAMARINE_1                "{7FFFD4}"
#define AQUAMARINE2                 "{76EEC6}"
#define AQUAMARINE_2                "{76EEC6}"
#define AQUAMARINE3                 "{66CDAA}"
#define AQUAMARINE_3                "{66CDAA}"
#define AQUAMARINE4                 "{458B74}"
#define AQUAMARINE_4                "{458B74}"
#define DARKSEAGREEN1               "{C1FFC1}"
#define DARK_SEA_GREEN_1            "{C1FFC1}"
#define DARKSEAGREEN2               "{B4EEB4}"
#define DARK_SEA_GREEN_2            "{B4EEB4}"
#define DARKSEAGREEN3               "{9BCD9B}"
#define DARK_SEA_GREEN_3            "{9BCD9B}"
#define DARKSEAGREEN4               "{698B69}"
#define DARK_SEA_GREEN_4            "{698B69}"
#define SEAGREEN1                   "{54FF9F}"
#define SEA_GREEN_1                 "{54FF9F}"
#define SEAGREEN2                   "{4EEE94}"
#define SEA_GREEN_2                 "{4EEE94}"
#define SEAGREEN3                   "{43CD80}"
#define SEA_GREEN_3                 "{43CD80}"
#define SEAGREEN4                   "{2E8B57}"
#define SEA_GREEN_4                 "{2E8B57}"
#define PALEGREEN1                  "{9AFF9A}"
#define PALE_GREEN_1                "{9AFF9A}"
#define PALEGREEN2                  "{90EE90}"
#define PALE_GREEN_2                "{90EE90}"
#define PALEGREEN3                  "{7CCD7C}"
#define PALE_GREEN_3                "{7CCD7C}"
#define PALEGREEN4                  "{548B54}"
#define PALE_GREEN_4                "{548B54}"
#define SPRINGGREEN1                "{00FF7F}"
#define SPRING_GREEN_1              "{00FF7F}"
#define SPRINGGREEN2                "{00EE76}"
#define SPRING_GREEN_2              "{00EE76}"
#define SPRINGGREEN3                "{00CD66}"
#define SPRING_GREEN_3              "{00CD66}"
#define SPRINGGREEN4                "{008B45}"
#define SPRING_GREEN_4              "{008B45}"
#define GREEN1                      "{00FF00}"
#define GREEN_1                     "{00FF00}"
#define GREEN2                      "{00EE00}"
#define GREEN_2                     "{00EE00}"
#define GREEN3                      "{00CD00}"
#define GREEN_3                     "{00CD00}"
#define GREEN4                      "{008B00}"
#define GREEN_4                     "{008B00}"
#define CHARTREUSE1                 "{7FFF00}"
#define CHARTREUSE_1                "{7FFF00}"
#define CHARTREUSE2                 "{76EE00}"
#define CHARTREUSE_2                "{76EE00}"
#define CHARTREUSE3                 "{66CD00}"
#define CHARTREUSE_3                "{66CD00}"
#define CHARTREUSE4                 "{458B00}"
#define CHARTREUSE_4                "{458B00}"
#define OLIVEDRAB1                  "{C0FF3E}"
#define OLIVE_DRAB_1                "{C0FF3E}"
#define OLIVEDRAB2                  "{B3EE3A}"
#define OLIVE_DRAB_2                "{B3EE3A}"
#define OLIVEDRAB3                  "{9ACD32}"
#define OLIVE_DRAB_3                "{9ACD32}"
#define OLIVEDRAB4                  "{698B22}"
#define OLIVE_DRAB_4                "{698B22}"
#define DARKOLIVEGREEN1             "{CAFF70}"
#define DARK_OLIVE_GREEN_1          "{CAFF70}"
#define DARKOLIVEGREEN2             "{BCEE68}"
#define DARK_OLIVE_GREEN_2          "{BCEE68}"
#define DARKOLIVEGREEN3             "{A2CD5A}"
#define DARK_OLIVE_GREEN_3          "{A2CD5A}"
#define DARKOLIVEGREEN4             "{6E8B3D}"
#define DARK_OLIVE_GREEN_4          "{6E8B3D}"
#define KHAKI1                      "{FFF68F}"
#define KHAKI_1                     "{FFF68F}"
#define KHAKI2                      "{EEE685}"
#define KHAKI_2                     "{EEE685}"
#define KHAKI3                      "{CDC673}"
#define KHAKI_3                     "{CDC673}"
#define KHAKI4                      "{8B864E}"
#define KHAKI_4                     "{8B864E}"
#define LIGHTGOLDENROD1             "{FFEC8B}"
#define LIGHT_GOLDENROD_1           "{FFEC8B}"
#define LIGHTGOLDENROD2             "{EEDC82}"
#define LIGHT_GOLDENROD_2           "{EEDC82}"
#define LIGHTGOLDENROD3             "{CDBE70}"
#define LIGHT_GOLDENROD_3           "{CDBE70}"
#define LIGHTGOLDENROD4             "{8B814C}"
#define LIGHT_GOLDENROD_4           "{8B814C}"
#define LIGHTYELLOW1                "{FFFFE0}"
#define LIGHT_YELLOW_1              "{FFFFE0}"
#define LIGHTYELLOW2                "{EEEED1}"
#define LIGHT_YELLOW_2              "{EEEED1}"
#define LIGHTYELLOW3                "{CDCDB4}"
#define LIGHT_YELLOW_3              "{CDCDB4}"
#define LIGHTYELLOW4                "{8B8B7A}"
#define LIGHT_YELLOW_4              "{8B8B7A}"
#define YELLOW1                     "{FFFF00}"
#define YELLOW_1                    "{FFFF00}"
#define YELLOW2                     "{EEEE00}"
#define YELLOW_2                    "{EEEE00}"
#define YELLOW3                     "{CDCD00}"
#define YELLOW_3                    "{CDCD00}"
#define YELLOW4                     "{8B8B00}"
#define YELLOW_4                    "{8B8B00}"
#define GOLD1                       "{FFD700}"
#define GOLD_1                      "{FFD700}"
#define GOLD2                       "{EEC900}"
#define GOLD_2                      "{EEC900}"
#define GOLD3                       "{CDAD00}"
#define GOLD_3                      "{CDAD00}"
#define GOLD4                       "{8B7500}"
#define GOLD_4                      "{8B7500}"
#define GOLDENROD1                  "{FFC125}"
#define GOLDENROD_1                 "{FFC125}"
#define GOLDENROD2                  "{EEB422}"
#define GOLDENROD_2                 "{EEB422}"
#define GOLDENROD3                  "{CD9B1D}"
#define GOLDENROD_3                 "{CD9B1D}"
#define GOLDENROD4                  "{8B6914}"
#define GOLDENROD_4                 "{8B6914}"
#define DARKGOLDENROD1              "{FFB90F}"
#define DARK_GOLDENROD_1            "{FFB90F}"
#define DARKGOLDENROD2              "{EEAD0E}"
#define DARK_GOLDENROD_2            "{EEAD0E}"
#define DARKGOLDENROD3              "{CD950C}"
#define DARK_GOLDENROD_3            "{CD950C}"
#define DARKGOLDENROD4              "{8B6508}"
#define DARK_GOLDENROD_4            "{8B6508}"
#define ROSYBROWN1                  "{FFC1C1}"
#define ROSY_BROWN_1                "{FFC1C1}"
#define ROSYBROWN2                  "{EEB4B4}"
#define ROSY_BROWN_2                "{EEB4B4}"
#define ROSYBROWN3                  "{CD9B9B}"
#define ROSY_BROWN_3                "{CD9B9B}"
#define ROSYBROWN4                  "{8B6969}"
#define ROSY_BROWN_4                "{8B6969}"
#define INDIANRED1                  "{FF6A6A}"
#define INDIAN_RED_1                "{FF6A6A}"
#define INDIANRED2                  "{EE6363}"
#define INDIAN_RED_2                "{EE6363}"
#define INDIANRED3                  "{CD5555}"
#define INDIAN_RED_3                "{CD5555}"
#define INDIANRED4                  "{8B3A3A}"
#define INDIAN_RED_4                "{8B3A3A}"
#define SIENNA1                     "{FF8247}"
#define SIENNA_1                    "{FF8247}"
#define SIENNA2                     "{EE7942}"
#define SIENNA_2                    "{EE7942}"
#define SIENNA3                     "{CD6839}"
#define SIENNA_3                    "{CD6839}"
#define SIENNA4                     "{8B4726}"
#define SIENNA_4                    "{8B4726}"
#define BURLYWOOD1                  "{FFD39B}"
#define BURLYWOOD_1                 "{FFD39B}"
#define BURLYWOOD2                  "{EEC591}"
#define BURLYWOOD_2                 "{EEC591}"
#define BURLYWOOD3                  "{CDAA7D}"
#define BURLYWOOD_3                 "{CDAA7D}"
#define BURLYWOOD4                  "{8B7355}"
#define BURLYWOOD_4                 "{8B7355}"
#define WHEAT1                      "{FFE7BA}"
#define WHEAT_1                     "{FFE7BA}"
#define WHEAT2                      "{EED8AE}"
#define WHEAT_2                     "{EED8AE}"
#define WHEAT3                      "{CDBA96}"
#define WHEAT_3                     "{CDBA96}"
#define WHEAT4                      "{8B7E66}"
#define WHEAT_4                     "{8B7E66}"
#define TAN1                        "{FFA54F}"
#define TAN_1                       "{FFA54F}"
#define TAN2                        "{EE9A49}"
#define TAN_2                       "{EE9A49}"
#define TAN3                        "{CD853F}"
#define TAN_3                       "{CD853F}"
#define TAN4                        "{8B5A2B}"
#define TAN_4                       "{8B5A2B}"
#define CHOCOLATE1                  "{FF7F24}"
#define CHOCOLATE_1                 "{FF7F24}"
#define CHOCOLATE2                  "{EE7621}"
#define CHOCOLATE_2                 "{EE7621}"
#define CHOCOLATE3                  "{CD661D}"
#define CHOCOLATE_3                 "{CD661D}"
#define CHOCOLATE4                  "{8B4513}"
#define CHOCOLATE_4                 "{8B4513}"
#define FIREBRICK1                  "{FF3030}"
#define FIREBRICK_1                 "{FF3030}"
#define FIREBRICK2                  "{EE2C2C}"
#define FIREBRICK_2                 "{EE2C2C}"
#define FIREBRICK3                  "{CD2626}"
#define FIREBRICK_3                 "{CD2626}"
#define FIREBRICK4                  "{8B1A1A}"
#define FIREBRICK_4                 "{8B1A1A}"
#define BROWN1                      "{FF4040}"
#define BROWN_1                     "{FF4040}"
#define BROWN2                      "{EE3B3B}"
#define BROWN_2                     "{EE3B3B}"
#define BROWN3                      "{CD3333}"
#define BROWN_3                     "{CD3333}"
#define BROWN4                      "{8B2323}"
#define BROWN_4                     "{8B2323}"
#define SALMON1                     "{FF8C69}"
#define SALMON_1                    "{FF8C69}"
#define SALMON2                     "{EE8262}"
#define SALMON_2                    "{EE8262}"
#define SALMON3                     "{CD7054}"
#define SALMON_3                    "{CD7054}"
#define SALMON4                     "{8B4C39}"
#define SALMON_4                    "{8B4C39}"
#define LIGHTSALMON1                "{FFA07A}"
#define LIGHT_SALMON_1              "{FFA07A}"
#define LIGHTSALMON2                "{EE9572}"
#define LIGHT_SALMON_2              "{EE9572}"
#define LIGHTSALMON3                "{CD8162}"
#define LIGHT_SALMON_3              "{CD8162}"
#define LIGHTSALMON4                "{8B5742}"
#define LIGHT_SALMON_4              "{8B5742}"
#define ORANGE1                     "{FFA500}"
#define ORANGE_1                    "{FFA500}"
#define ORANGE2                     "{EE9A00}"
#define ORANGE_2                    "{EE9A00}"
#define ORANGE3                     "{CD8500}"
#define ORANGE_3                    "{CD8500}"
#define ORANGE4                     "{8B5A00}"
#define ORANGE_4                    "{8B5A00}"
#define DARKORANGE1                 "{FF7F00}"
#define DARK_ORANGE_1               "{FF7F00}"
#define DARKORANGE2                 "{EE7600}"
#define DARK_ORANGE_2               "{EE7600}"
#define DARKORANGE3                 "{CD6600}"
#define DARK_ORANGE_3               "{CD6600}"
#define DARKORANGE4                 "{8B4500}"
#define DARK_ORANGE_4               "{8B4500}"
#define CORAL1                      "{FF7256}"
#define CORAL_1                     "{FF7256}"
#define CORAL2                      "{EE6A50}"
#define CORAL_2                     "{EE6A50}"
#define CORAL3                      "{CD5B45}"
#define CORAL_3                     "{CD5B45}"
#define CORAL4                      "{8B3E2F}"
#define CORAL_4                     "{8B3E2F}"
#define TOMATO1                     "{FF6347}"
#define TOMATO_1                    "{FF6347}"
#define TOMATO2                     "{EE5C42}"
#define TOMATO_2                    "{EE5C42}"
#define TOMATO3                     "{CD4F39}"
#define TOMATO_3                    "{CD4F39}"
#define TOMATO4                     "{8B3626}"
#define TOMATO_4                    "{8B3626}"
#define ORANGERED1                  "{FF4500}"
#define ORANGE_RED_1                "{FF4500}"
#define ORANGERED2                  "{EE4000}"
#define ORANGE_RED_2                "{EE4000}"
#define ORANGERED3                  "{CD3700}"
#define ORANGE_RED_3                "{CD3700}"
#define ORANGERED4                  "{8B2500}"
#define ORANGE_RED_4                "{8B2500}"
#define RED1                        "{FF0000}"
#define RED_1                       "{FF0000}"
#define RED2                        "{EE0000}"
#define RED_2                       "{EE0000}"
#define RED3                        "{CD0000}"
#define RED_3                       "{CD0000}"
#define RED4                        "{8B0000}"
#define RED_4                       "{8B0000}"
#define DEEPPINK1                   "{FF1493}"
#define DEEP_PINK_1                 "{FF1493}"
#define DEEPPINK2                   "{EE1289}"
#define DEEP_PINK_2                 "{EE1289}"
#define DEEPPINK3                   "{CD1076}"
#define DEEP_PINK_3                 "{CD1076}"
#define DEEPPINK4                   "{8B0A50}"
#define DEEP_PINK_4                 "{8B0A50}"
#define HOTPINK1                    "{FF6EB4}"
#define HOT_PINK_1                  "{FF6EB4}"
#define HOTPINK2                    "{EE6AA7}"
#define HOT_PINK_2                  "{EE6AA7}"
#define HOTPINK3                    "{CD6090}"
#define HOT_PINK_3                  "{CD6090}"
#define HOTPINK4                    "{8B3A62}"
#define HOT_PINK_4                  "{8B3A62}"
#define PINK1                       "{FFB5C5}"
#define PINK_1                      "{FFB5C5}"
#define PINK2                       "{EEA9B8}"
#define PINK_2                      "{EEA9B8}"
#define PINK3                       "{CD919E}"
#define PINK_3                      "{CD919E}"
#define PINK4                       "{8B636C}"
#define PINK_4                      "{8B636C}"
#define LIGHTPINK1                  "{FFAEB9}"
#define LIGHT_PINK_1                "{FFAEB9}"
#define LIGHTPINK2                  "{EEA2AD}"
#define LIGHT_PINK_2                "{EEA2AD}"
#define LIGHTPINK3                  "{CD8C95}"
#define LIGHT_PINK_3                "{CD8C95}"
#define LIGHTPINK4                  "{8B5F65}"
#define LIGHT_PINK_4                "{8B5F65}"
#define PALEVIOLETRED1              "{FF82AB}"
#define PALE_VIOLET_RED_1           "{FF82AB}"
#define PALEVIOLETRED2              "{EE799F}"
#define PALE_VIOLET_RED_2           "{EE799F}"
#define PALEVIOLETRED3              "{CD6889}"
#define PALE_VIOLET_RED_3           "{CD6889}"
#define PALEVIOLETRED4              "{8B475D}"
#define PALE_VIOLET_RED_4           "{8B475D}"
#define MAROON1                     "{FF34B3}"
#define MAROON_1                    "{FF34B3}"
#define MAROON2                     "{EE30A7}"
#define MAROON_2                    "{EE30A7}"
#define MAROON3                     "{CD2990}"
#define MAROON_3                    "{CD2990}"
#define MAROON4                     "{8B1C62}"
#define MAROON_4                    "{8B1C62}"
#define VIOLETRED1                  "{FF3E96}"
#define VIOLET_RED_1                "{FF3E96}"
#define VIOLETRED2                  "{EE3A8C}"
#define VIOLET_RED_2                "{EE3A8C}"
#define VIOLETRED3                  "{CD3278}"
#define VIOLET_RED_3                "{CD3278}"
#define VIOLETRED4                  "{8B2252}"
#define VIOLET_RED_4                "{8B2252}"
#define MAGENTA1                    "{FF00FF}"
#define MAGENTA_1                   "{FF00FF}"
#define MAGENTA2                    "{EE00EE}"
#define MAGENTA_2                   "{EE00EE}"
#define MAGENTA3                    "{CD00CD}"
#define MAGENTA_3                   "{CD00CD}"
#define MAGENTA4                    "{8B008B}"
#define MAGENTA_4                   "{8B008B}"
#define ORCHID1                     "{FF83FA}"
#define ORCHID_1                    "{FF83FA}"
#define ORCHID2                     "{EE7AE9}"
#define ORCHID_2                    "{EE7AE9}"
#define ORCHID3                     "{CD69C9}"
#define ORCHID_3                    "{CD69C9}"
#define ORCHID4                     "{8B4789}"
#define ORCHID_4                    "{8B4789}"
#define PLUM1                       "{FFBBFF}"
#define PLUM_1                      "{FFBBFF}"
#define PLUM2                       "{EEAEEE}"
#define PLUM_2                      "{EEAEEE}"
#define PLUM3                       "{CD96CD}"
#define PLUM_3                      "{CD96CD}"
#define PLUM4                       "{8B668B}"
#define PLUM_4                      "{8B668B}"
#define MEDIUMORCHID1               "{E066FF}"
#define MEDIUM_ORCHID_1             "{E066FF}"
#define MEDIUMORCHID2               "{D15FEE}"
#define MEDIUM_ORCHID_2             "{D15FEE}"
#define MEDIUMORCHID3               "{B452CD}"
#define MEDIUM_ORCHID_3             "{B452CD}"
#define MEDIUMORCHID4               "{7A378B}"
#define MEDIUM_ORCHID_4             "{7A378B}"
#define DARKORCHID1                 "{BF3EFF}"
#define DARK_ORCHID_1               "{BF3EFF}"
#define DARKORCHID2                 "{B23AEE}"
#define DARK_ORCHID_2               "{B23AEE}"
#define DARKORCHID3                 "{9A32CD}"
#define DARK_ORCHID_3               "{9A32CD}"
#define DARKORCHID4                 "{68228B}"
#define DARK_ORCHID_4               "{68228B}"
#define PURPLE1                     "{9B30FF}"
#define PURPLE_1                    "{9B30FF}"
#define PURPLE2                     "{912CEE}"
#define PURPLE_2                    "{912CEE}"
#define PURPLE3                     "{7D26CD}"
#define PURPLE_3                    "{7D26CD}"
#define PURPLE4                     "{551A8B}"
#define PURPLE_4                    "{551A8B}"
#define MEDIUMPURPLE1               "{AB82FF}"
#define MEDIUM_PURPLE_1             "{AB82FF}"
#define MEDIUMPURPLE2               "{9F79EE}"
#define MEDIUM_PURPLE_2             "{9F79EE}"
#define MEDIUMPURPLE3               "{8968CD}"
#define MEDIUM_PURPLE_3             "{8968CD}"
#define MEDIUMPURPLE4               "{5D478B}"
#define MEDIUM_PURPLE_4             "{5D478B}"
#define THISTLE1                    "{FFE1FF}"
#define THISTLE_1                   "{FFE1FF}"
#define THISTLE2                    "{EED2EE}"
#define THISTLE_2                   "{EED2EE}"
#define THISTLE3                    "{CDB5CD}"
#define THISTLE_3                   "{CDB5CD}"
#define THISTLE4                    "{8B7B8B}"
#define THISTLE_4                   "{8B7B8B}"
#define GRAY0                       "{000000}"
#define GRAY_0                      "{000000}"
#define GREY0                       "{000000}"
#define GREY_0                      "{000000}"
#define GRAY1                       "{030303}"
#define GRAY_1                      "{030303}"
#define GREY1                       "{030303}"
#define GREY_1                      "{030303}"
#define GRAY2                       "{050505}"
#define GRAY_2                      "{050505}"
#define GREY2                       "{050505}"
#define GREY_2                      "{050505}"
#define GRAY3                       "{080808}"
#define GRAY_3                      "{080808}"
#define GREY3                       "{080808}"
#define GREY_3                      "{080808}"
#define GRAY4                       "{0A0A0A}"
#define GRAY_4                      "{0A0A0A}"
#define GREY4                       "{0A0A0A}"
#define GREY_4                      "{0A0A0A}"
#define GRAY5                       "{0D0D0D}"
#define GRAY_5                      "{0D0D0D}"
#define GREY5                       "{0D0D0D}"
#define GREY_5                      "{0D0D0D}"
#define GRAY6                       "{0F0F0F}"
#define GRAY_6                      "{0F0F0F}"
#define GREY6                       "{0F0F0F}"
#define GREY_6                      "{0F0F0F}"
#define GRAY7                       "{121212}"
#define GRAY_7                      "{121212}"
#define GREY7                       "{121212}"
#define GREY_7                      "{121212}"
#define GRAY8                       "{141414}"
#define GRAY_8                      "{141414}"
#define GREY8                       "{141414}"
#define GREY_8                      "{141414}"
#define GRAY9                       "{171717}"
#define GRAY_9                      "{171717}"
#define GREY9                       "{171717}"
#define GREY_9                      "{171717}"
#define GRAY10                      "{1A1A1A}"
#define GRAY_10                     "{1A1A1A}"
#define GREY10                      "{1A1A1A}"
#define GREY_10                     "{1A1A1A}"
#define GRAY11                      "{1C1C1C}"
#define GRAY_11                     "{1C1C1C}"
#define GREY11                      "{1C1C1C}"
#define GREY_11                     "{1C1C1C}"
#define GRAY12                      "{1F1F1F}"
#define GRAY_12                     "{1F1F1F}"
#define GREY12                      "{1F1F1F}"
#define GREY_12                     "{1F1F1F}"
#define GRAY13                      "{212121}"
#define GRAY_13                     "{212121}"
#define GREY13                      "{212121}"
#define GREY_13                     "{212121}"
#define GRAY14                      "{242424}"
#define GRAY_14                     "{242424}"
#define GREY14                      "{242424}"
#define GREY_14                     "{242424}"
#define GRAY15                      "{262626}"
#define GRAY_15                     "{262626}"
#define GREY15                      "{262626}"
#define GREY_15                     "{262626}"
#define GRAY16                      "{292929}"
#define GRAY_16                     "{292929}"
#define GREY16                      "{292929}"
#define GREY_16                     "{292929}"
#define GRAY17                      "{2B2B2B}"
#define GRAY_17                     "{2B2B2B}"
#define GREY17                      "{2B2B2B}"
#define GREY_17                     "{2B2B2B}"
#define GRAY18                      "{2E2E2E}"
#define GRAY_18                     "{2E2E2E}"
#define GREY18                      "{2E2E2E}"
#define GREY_18                     "{2E2E2E}"
#define GRAY19                      "{303030}"
#define GRAY_19                     "{303030}"
#define GREY19                      "{303030}"
#define GREY_19                     "{303030}"
#define GRAY20                      "{333333}"
#define GRAY_20                     "{333333}"
#define GREY20                      "{333333}"
#define GREY_20                     "{333333}"
#define GRAY21                      "{363636}"
#define GRAY_21                     "{363636}"
#define GREY21                      "{363636}"
#define GREY_21                     "{363636}"
#define GRAY22                      "{383838}"
#define GRAY_22                     "{383838}"
#define GREY22                      "{383838}"
#define GREY_22                     "{383838}"
#define GRAY23                      "{3B3B3B}"
#define GRAY_23                     "{3B3B3B}"
#define GREY23                      "{3B3B3B}"
#define GREY_23                     "{3B3B3B}"
#define GRAY24                      "{3D3D3D}"
#define GRAY_24                     "{3D3D3D}"
#define GREY24                      "{3D3D3D}"
#define GREY_24                     "{3D3D3D}"
#define GRAY25                      "{404040}"
#define GRAY_25                     "{404040}"
#define GREY25                      "{404040}"
#define GREY_25                     "{404040}"
#define GRAY26                      "{424242}"
#define GRAY_26                     "{424242}"
#define GREY26                      "{424242}"
#define GREY_26                     "{424242}"
#define GRAY27                      "{454545}"
#define GRAY_27                     "{454545}"
#define GREY27                      "{454545}"
#define GREY_27                     "{454545}"
#define GRAY28                      "{474747}"
#define GRAY_28                     "{474747}"
#define GREY28                      "{474747}"
#define GREY_28                     "{474747}"
#define GRAY29                      "{4A4A4A}"
#define GRAY_29                     "{4A4A4A}"
#define GREY29                      "{4A4A4A}"
#define GREY_29                     "{4A4A4A}"
#define GRAY30                      "{4D4D4D}"
#define GRAY_30                     "{4D4D4D}"
#define GREY30                      "{4D4D4D}"
#define GREY_30                     "{4D4D4D}"
#define GRAY31                      "{4F4F4F}"
#define GRAY_31                     "{4F4F4F}"
#define GREY31                      "{4F4F4F}"
#define GREY_31                     "{4F4F4F}"
#define GRAY32                      "{525252}"
#define GRAY_32                     "{525252}"
#define GREY32                      "{525252}"
#define GREY_32                     "{525252}"
#define GRAY33                      "{545454}"
#define GRAY_33                     "{545454}"
#define GREY33                      "{545454}"
#define GREY_33                     "{545454}"
#define GRAY34                      "{575757}"
#define GRAY_34                     "{575757}"
#define GREY34                      "{575757}"
#define GREY_34                     "{575757}"
#define GRAY35                      "{595959}"
#define GRAY_35                     "{595959}"
#define GREY35                      "{595959}"
#define GREY_35                     "{595959}"
#define GRAY36                      "{5C5C5C}"
#define GRAY_36                     "{5C5C5C}"
#define GREY36                      "{5C5C5C}"
#define GREY_36                     "{5C5C5C}"
#define GRAY37                      "{5E5E5E}"
#define GRAY_37                     "{5E5E5E}"
#define GREY37                      "{5E5E5E}"
#define GREY_37                     "{5E5E5E}"
#define GRAY38                      "{616161}"
#define GRAY_38                     "{616161}"
#define GREY38                      "{616161}"
#define GREY_38                     "{616161}"
#define GRAY39                      "{636363}"
#define GRAY_39                     "{636363}"
#define GREY39                      "{636363}"
#define GREY_39                     "{636363}"
#define GRAY40                      "{666666}"
#define GRAY_40                     "{666666}"
#define GREY40                      "{666666}"
#define GREY_40                     "{666666}"
#define GRAY41                      "{696969}"
#define GRAY_41                     "{696969}"
#define GREY41                      "{696969}"
#define GREY_41                     "{696969}"
#define GRAY42                      "{6B6B6B}"
#define GRAY_42                     "{6B6B6B}"
#define GREY42                      "{6B6B6B}"
#define GREY_42                     "{6B6B6B}"
#define GRAY43                      "{6E6E6E}"
#define GRAY_43                     "{6E6E6E}"
#define GREY43                      "{6E6E6E}"
#define GREY_43                     "{6E6E6E}"
#define GRAY44                      "{707070}"
#define GRAY_44                     "{707070}"
#define GREY44                      "{707070}"
#define GREY_44                     "{707070}"
#define GRAY45                      "{737373}"
#define GRAY_45                     "{737373}"
#define GREY45                      "{737373}"
#define GREY_45                     "{737373}"
#define GRAY46                      "{757575}"
#define GRAY_46                     "{757575}"
#define GREY46                      "{757575}"
#define GREY_46                     "{757575}"
#define GRAY47                      "{787878}"
#define GRAY_47                     "{787878}"
#define GREY47                      "{787878}"
#define GREY_47                     "{787878}"
#define GRAY48                      "{7A7A7A}"
#define GRAY_48                     "{7A7A7A}"
#define GREY48                      "{7A7A7A}"
#define GREY_48                     "{7A7A7A}"
#define GRAY49                      "{7D7D7D}"
#define GRAY_49                     "{7D7D7D}"
#define GREY49                      "{7D7D7D}"
#define GREY_49                     "{7D7D7D}"
#define GRAY50                      "{7F7F7F}"
#define GRAY_50                     "{7F7F7F}"
#define GREY50                      "{7F7F7F}"
#define GREY_50                     "{7F7F7F}"
#define GRAY51                      "{828282}"
#define GRAY_51                     "{828282}"
#define GREY51                      "{828282}"
#define GREY_51                     "{828282}"
#define GRAY52                      "{858585}"
#define GRAY_52                     "{858585}"
#define GREY52                      "{858585}"
#define GREY_52                     "{858585}"
#define GRAY53                      "{878787}"
#define GRAY_53                     "{878787}"
#define GREY53                      "{878787}"
#define GREY_53                     "{878787}"
#define GRAY54                      "{8A8A8A}"
#define GRAY_54                     "{8A8A8A}"
#define GREY54                      "{8A8A8A}"
#define GREY_54                     "{8A8A8A}"
#define GRAY55                      "{8C8C8C}"
#define GRAY_55                     "{8C8C8C}"
#define GREY55                      "{8C8C8C}"
#define GREY_55                     "{8C8C8C}"
#define GRAY56                      "{8F8F8F}"
#define GRAY_56                     "{8F8F8F}"
#define GREY56                      "{8F8F8F}"
#define GREY_56                     "{8F8F8F}"
#define GRAY57                      "{919191}"
#define GRAY_57                     "{919191}"
#define GREY57                      "{919191}"
#define GREY_57                     "{919191}"
#define GRAY58                      "{949494}"
#define GRAY_58                     "{949494}"
#define GREY58                      "{949494}"
#define GREY_58                     "{949494}"
#define GRAY59                      "{969696}"
#define GRAY_59                     "{969696}"
#define GREY59                      "{969696}"
#define GREY_59                     "{969696}"
#define GRAY60                      "{999999}"
#define GRAY_60                     "{999999}"
#define GREY60                      "{999999}"
#define GREY_60                     "{999999}"
#define GRAY61                      "{9C9C9C}"
#define GRAY_61                     "{9C9C9C}"
#define GREY61                      "{9C9C9C}"
#define GREY_61                     "{9C9C9C}"
#define GRAY62                      "{9E9E9E}"
#define GRAY_62                     "{9E9E9E}"
#define GREY62                      "{9E9E9E}"
#define GREY_62                     "{9E9E9E}"
#define GRAY63                      "{A1A1A1}"
#define GRAY_63                     "{A1A1A1}"
#define GREY63                      "{A1A1A1}"
#define GREY_63                     "{A1A1A1}"
#define GRAY64                      "{A3A3A3}"
#define GRAY_64                     "{A3A3A3}"
#define GREY64                      "{A3A3A3}"
#define GREY_64                     "{A3A3A3}"
#define GRAY65                      "{A6A6A6}"
#define GRAY_65                     "{A6A6A6}"
#define GREY65                      "{A6A6A6}"
#define GREY_65                     "{A6A6A6}"
#define GRAY66                      "{A8A8A8}"
#define GRAY_66                     "{A8A8A8}"
#define GREY66                      "{A8A8A8}"
#define GREY_66                     "{A8A8A8}"
#define GRAY67                      "{ABABAB}"
#define GRAY_67                     "{ABABAB}"
#define GREY67                      "{ABABAB}"
#define GREY_67                     "{ABABAB}"
#define GRAY68                      "{ADADAD}"
#define GRAY_68                     "{ADADAD}"
#define GREY68                      "{ADADAD}"
#define GREY_68                     "{ADADAD}"
#define GRAY69                      "{B0B0B0}"
#define GRAY_69                     "{B0B0B0}"
#define GREY69                      "{B0B0B0}"
#define GREY_69                     "{B0B0B0}"
#define GRAY70                      "{B3B3B3}"
#define GRAY_70                     "{B3B3B3}"
#define GREY70                      "{B3B3B3}"
#define GREY_70                     "{B3B3B3}"
#define GRAY71                      "{B5B5B5}"
#define GRAY_71                     "{B5B5B5}"
#define GREY71                      "{B5B5B5}"
#define GREY_71                     "{B5B5B5}"
#define GRAY72                      "{B8B8B8}"
#define GRAY_72                     "{B8B8B8}"
#define GREY72                      "{B8B8B8}"
#define GREY_72                     "{B8B8B8}"
#define GRAY73                      "{BABABA}"
#define GRAY_73                     "{BABABA}"
#define GREY73                      "{BABABA}"
#define GREY_73                     "{BABABA}"
#define GRAY74                      "{BDBDBD}"
#define GRAY_74                     "{BDBDBD}"
#define GREY74                      "{BDBDBD}"
#define GREY_74                     "{BDBDBD}"
#define GRAY75                      "{BFBFBF}"
#define GRAY_75                     "{BFBFBF}"
#define GREY75                      "{BFBFBF}"
#define GREY_75                     "{BFBFBF}"
#define GRAY76                      "{C2C2C2}"
#define GRAY_76                     "{C2C2C2}"
#define GREY76                      "{C2C2C2}"
#define GREY_76                     "{C2C2C2}"
#define GRAY77                      "{C4C4C4}"
#define GRAY_77                     "{C4C4C4}"
#define GREY77                      "{C4C4C4}"
#define GREY_77                     "{C4C4C4}"
#define GRAY78                      "{C7C7C7}"
#define GRAY_78                     "{C7C7C7}"
#define GREY78                      "{C7C7C7}"
#define GREY_78                     "{C7C7C7}"
#define GRAY79                      "{C9C9C9}"
#define GRAY_79                     "{C9C9C9}"
#define GREY79                      "{C9C9C9}"
#define GREY_79                     "{C9C9C9}"
#define GRAY80                      "{CCCCCC}"
#define GRAY_80                     "{CCCCCC}"
#define GREY80                      "{CCCCCC}"
#define GREY_80                     "{CCCCCC}"
#define GRAY81                      "{CFCFCF}"
#define GRAY_81                     "{CFCFCF}"
#define GREY81                      "{CFCFCF}"
#define GREY_81                     "{CFCFCF}"
#define GRAY82                      "{D1D1D1}"
#define GRAY_82                     "{D1D1D1}"
#define GREY82                      "{D1D1D1}"
#define GREY_82                     "{D1D1D1}"
#define GRAY83                      "{D4D4D4}"
#define GRAY_83                     "{D4D4D4}"
#define GREY83                      "{D4D4D4}"
#define GREY_83                     "{D4D4D4}"
#define GRAY84                      "{D6D6D6}"
#define GRAY_84                     "{D6D6D6}"
#define GREY84                      "{D6D6D6}"
#define GREY_84                     "{D6D6D6}"
#define GRAY85                      "{D9D9D9}"
#define GRAY_85                     "{D9D9D9}"
#define GREY85                      "{D9D9D9}"
#define GREY_85                     "{D9D9D9}"
#define GRAY86                      "{DBDBDB}"
#define GRAY_86                     "{DBDBDB}"
#define GREY86                      "{DBDBDB}"
#define GREY_86                     "{DBDBDB}"
#define GRAY87                      "{DEDEDE}"
#define GRAY_87                     "{DEDEDE}"
#define GREY87                      "{DEDEDE}"
#define GREY_87                     "{DEDEDE}"
#define GRAY88                      "{E0E0E0}"
#define GRAY_88                     "{E0E0E0}"
#define GREY88                      "{E0E0E0}"
#define GREY_88                     "{E0E0E0}"
#define GRAY89                      "{E3E3E3}"
#define GRAY_89                     "{E3E3E3}"
#define GREY89                      "{E3E3E3}"
#define GREY_89                     "{E3E3E3}"
#define GRAY90                      "{E5E5E5}"
#define GRAY_90                     "{E5E5E5}"
#define GREY90                      "{E5E5E5}"
#define GREY_90                     "{E5E5E5}"
#define GRAY91                      "{E8E8E8}"
#define GRAY_91                     "{E8E8E8}"
#define GREY91                      "{E8E8E8}"
#define GREY_91                     "{E8E8E8}"
#define GRAY92                      "{EBEBEB}"
#define GRAY_92                     "{EBEBEB}"
#define GREY92                      "{EBEBEB}"
#define GREY_92                     "{EBEBEB}"
#define GRAY93                      "{EDEDED}"
#define GRAY_93                     "{EDEDED}"
#define GREY93                      "{EDEDED}"
#define GREY_93                     "{EDEDED}"
#define GRAY94                      "{F0F0F0}"
#define GRAY_94                     "{F0F0F0}"
#define GREY94                      "{F0F0F0}"
#define GREY_94                     "{F0F0F0}"
#define GRAY95                      "{F2F2F2}"
#define GRAY_95                     "{F2F2F2}"
#define GREY95                      "{F2F2F2}"
#define GREY_95                     "{F2F2F2}"
#define GRAY96                      "{F5F5F5}"
#define GRAY_96                     "{F5F5F5}"
#define GREY96                      "{F5F5F5}"
#define GREY_96                     "{F5F5F5}"
#define GRAY97                      "{F7F7F7}"
#define GRAY_97                     "{F7F7F7}"
#define GREY97                      "{F7F7F7}"
#define GREY_97                     "{F7F7F7}"
#define GRAY98                      "{FAFAFA}"
#define GRAY_98                     "{FAFAFA}"
#define GREY98                      "{FAFAFA}"
#define GREY_98                     "{FAFAFA}"
#define GRAY99                      "{FCFCFC}"
#define GRAY_99                     "{FCFCFC}"
#define GREY99                      "{FCFCFC}"
#define GREY_99                     "{FCFCFC}"
#define GRAY100                     "{FFFFFF}"
#define GRAY_100                    "{FFFFFF}"
#define GREY100                     "{FFFFFF}"
#define GREY_100                    "{FFFFFF}"
#define DARKGREY                    "{A9A9A9}"
#define DARK_GREY                   "{A9A9A9}"
#define DARKGRAY                    "{A9A9A9}"
#define DARK_GRAY                   "{A9A9A9}"
#define DARKBLUE                    "{00008B}"
#define DARK_BLUE                   "{00008B}"
#define DARKCYAN                    "{008B8B}"
#define DARK_CYAN                   "{008B8B}"
#define DARKMAGENTA                 "{8B008B}"
#define DARK_MAGENTA                "{8B008B}"
#define DARKRED                     "{8B0000}"
#define DARK_RED                    "{8B0000}"
#define LIGHTGREEN                  "{90EE90}"
#define LIGHT_GREEN                 "{90EE90}"

// Colours without alpha.
#define Y_SNOW                      0xFFFAFA00
#define Y_GHOSTWHITE                0xF8F8FF00
#define Y_GHOST_WHITE               0xF8F8FF00
#define Y_WHITESMOKE                0xF5F5F500
#define Y_WHITE_SMOKE               0xF5F5F500
#define Y_GAINSBORO                 0xDCDCDC00
#define Y_FLORALWHITE               0xFFFAF000
#define Y_FLORAL_WHITE              0xFFFAF000
#define Y_OLDLACE                   0xFDF5E600
#define Y_OLD_LACE                  0xFDF5E600
#define Y_LINEN                     0xFAF0E600
#define Y_ANTIQUEWHITE              0xFAEBD700
#define Y_ANTIQUE_WHITE             0xFAEBD700
#define Y_PAPAYAWHIP                0xFFEFD500
#define Y_PAPAYA_WHIP               0xFFEFD500
#define Y_BLANCHEDALMOND            0xFFEBCD00
#define Y_BLANCHED_ALMOND           0xFFEBCD00
#define Y_BISQUE                    0xFFE4C400
#define Y_PEACHPUFF                 0xFFDAB900
#define Y_PEACH_PUFF                0xFFDAB900
#define Y_NAVAJOWHITE               0xFFDEAD00
#define Y_NAVAJO_WHITE              0xFFDEAD00
#define Y_MOCCASIN                  0xFFE4B500
#define Y_CORNSILK                  0xFFF8DC00
#define Y_IVORY                     0xFFFFF000
#define Y_LEMONCHIFFON              0xFFFACD00
#define Y_LEMON_CHIFFON             0xFFFACD00
#define Y_SEASHELL                  0xFFF5EE00
#define Y_HONEYDEW                  0xF0FFF000
#define Y_MINTCREAM                 0xF5FFFA00
#define Y_MINT_CREAM                0xF5FFFA00
#define Y_AZURE                     0xF0FFFF00
#define Y_ALICEBLUE                 0xF0F8FF00
#define Y_ALICE_BLUE                0xF0F8FF00
#define Y_LAVENDER                  0xE6E6FA00
#define Y_LAVENDERBLUSH             0xFFF0F500
#define Y_LAVENDER_BLUSH            0xFFF0F500
#define Y_MISTYROSE                 0xFFE4E100
#define Y_MISTY_ROSE                0xFFE4E100
#define Y_WHITE                     0xFFFFFF00
#define Y_BLACK                     0x00000000
#define Y_DARKSLATEGRAY             0x2F4F4F00
#define Y_DARK_SLATE_GRAY           0x2F4F4F00
#define Y_DARKSLATEGREY             0x2F4F4F00
#define Y_DARK_SLATE_GREY           0x2F4F4F00
#define Y_DIMGRAY                   0x69696900
#define Y_DIM_GRAY                  0x69696900
#define Y_DIMGREY                   0x69696900
#define Y_DIM_GREY                  0x69696900
#define Y_SLATEGRAY                 0x70809000
#define Y_SLATE_GRAY                0x70809000
#define Y_SLATEGREY                 0x70809000
#define Y_SLATE_GREY                0x70809000
#define Y_LIGHTSLATEGRAY            0x77889900
#define Y_LIGHT_SLATE_GRAY          0x77889900
#define Y_LIGHTSLATEGREY            0x77889900
#define Y_LIGHT_SLATE_GREY          0x77889900
#define Y_GRAY                      0xBEBEBE00
#define Y_GREY                      0xBEBEBE00
#define Y_LIGHTGREY                 0xD3D3D300
#define Y_LIGHT_GREY                0xD3D3D300
#define Y_LIGHTGRAY                 0xD3D3D300
#define Y_LIGHT_GRAY                0xD3D3D300
#define Y_MIDNIGHTBLUE              0x19197000
#define Y_MIDNIGHT_BLUE             0x19197000
#define Y_NAVY                      0x00008000
#define Y_NAVYBLUE                  0x00008000
#define Y_NAVY_BLUE                 0x00008000
#define Y_CORNFLOWERBLUE            0x6495ED00
#define Y_CORNFLOWER_BLUE           0x6495ED00
#define Y_DARKSLATEBLUE             0x483D8B00
#define Y_DARK_SLATE_BLUE           0x483D8B00
#define Y_SLATEBLUE                 0x6A5ACD00
#define Y_SLATE_BLUE                0x6A5ACD00
#define Y_MEDIUMSLATEBLUE           0x7B68EE00
#define Y_MEDIUM_SLATE_BLUE         0x7B68EE00
#define Y_LIGHTSLATEBLUE            0x8470FF00
#define Y_LIGHT_SLATE_BLUE          0x8470FF00
#define Y_MEDIUMBLUE                0x0000CD00
#define Y_MEDIUM_BLUE               0x0000CD00
#define Y_ROYALBLUE                 0x4169E100
#define Y_ROYAL_BLUE                0x4169E100
#define Y_BLUE                      0x0000FF00
#define Y_DODGERBLUE                0x1E90FF00
#define Y_DODGER_BLUE               0x1E90FF00
#define Y_DEEPSKYBLUE               0x00BFFF00
#define Y_DEEP_SKY_BLUE             0x00BFFF00
#define Y_SKYBLUE                   0x87CEEB00
#define Y_SKY_BLUE                  0x87CEEB00
#define Y_LIGHTSKYBLUE              0x87CEFA00
#define Y_LIGHT_SKY_BLUE            0x87CEFA00
#define Y_STEELBLUE                 0x4682B400
#define Y_STEEL_BLUE                0x4682B400
#define Y_LIGHTSTEELBLUE            0xB0C4DE00
#define Y_LIGHT_STEEL_BLUE          0xB0C4DE00
#define Y_LIGHTBLUE                 0xADD8E600
#define Y_LIGHT_BLUE                0xADD8E600
#define Y_POWDERBLUE                0xB0E0E600
#define Y_POWDER_BLUE               0xB0E0E600
#define Y_PALETURQUOISE             0xAFEEEE00
#define Y_PALE_TURQUOISE            0xAFEEEE00
#define Y_DARKTURQUOISE             0x00CED100
#define Y_DARK_TURQUOISE            0x00CED100
#define Y_MEDIUMTURQUOISE           0x48D1CC00
#define Y_MEDIUM_TURQUOISE          0x48D1CC00
#define Y_TURQUOISE                 0x40E0D000
#define Y_CYAN                      0x00FFFF00
#define Y_LIGHTCYAN                 0xE0FFFF00
#define Y_LIGHT_CYAN                0xE0FFFF00
#define Y_CADETBLUE                 0x5F9EA000
#define Y_CADET_BLUE                0x5F9EA000
#define Y_MEDIUMAQUAMARINE          0x66CDAA00
#define Y_MEDIUM_AQUAMARINE         0x66CDAA00
#define Y_AQUAMARINE                0x7FFFD400
#define Y_DARKGREEN                 0x00640000
#define Y_DARK_GREEN                0x00640000
#define Y_DARKOLIVEGREEN            0x556B2F00
#define Y_DARK_OLIVE_GREEN          0x556B2F00
#define Y_DARKSEAGREEN              0x8FBC8F00
#define Y_DARK_SEA_GREEN            0x8FBC8F00
#define Y_SEAGREEN                  0x2E8B5700
#define Y_SEA_GREEN                 0x2E8B5700
#define Y_MEDIUMSEAGREEN            0x3CB37100
#define Y_MEDIUM_SEA_GREEN          0x3CB37100
#define Y_LIGHTSEAGREEN             0x20B2AA00
#define Y_LIGHT_SEA_GREEN           0x20B2AA00
#define Y_PALEGREEN                 0x98FB9800
#define Y_PALE_GREEN                0x98FB9800
#define Y_SPRINGGREEN               0x00FF7F00
#define Y_SPRING_GREEN              0x00FF7F00
#define Y_LAWNGREEN                 0x7CFC0000
#define Y_LAWN_GREEN                0x7CFC0000
#define Y_GREEN                     0x00FF0000
#define Y_CHARTREUSE                0x7FFF0000
#define Y_MEDIUMSPRINGGREEN         0x00FA9A00
#define Y_MEDIUM_SPRING_GREEN       0x00FA9A00
#define Y_GREENYELLOW               0xADFF2F00
#define Y_GREEN_YELLOW              0xADFF2F00
#define Y_LIMEGREEN                 0x32CD3200
#define Y_LIME_GREEN                0x32CD3200
#define Y_YELLOWGREEN               0x9ACD3200
#define Y_YELLOW_GREEN              0x9ACD3200
#define Y_FORESTGREEN               0x228B2200
#define Y_FOREST_GREEN              0x228B2200
#define Y_OLIVEDRAB                 0x6B8E2300
#define Y_OLIVE_DRAB                0x6B8E2300
#define Y_DARKKHAKI                 0xBDB76B00
#define Y_DARK_KHAKI                0xBDB76B00
#define Y_KHAKI                     0xF0E68C00
#define Y_PALEGOLDENROD             0xEEE8AA00
#define Y_PALE_GOLDENROD            0xEEE8AA00
#define Y_LIGHTGOLDENRODYELLOW      0xFAFAD200
#define Y_LIGHT_GOLDENROD_YELLOW    0xFAFAD200
#define Y_LIGHTYELLOW               0xFFFFE000
#define Y_LIGHT_YELLOW              0xFFFFE000
#define Y_YELLOW                    0xFFFF0000
#define Y_GOLD                      0xFFD70000
#define Y_LIGHTGOLDENROD            0xEEDD8200
#define Y_LIGHT_GOLDENROD           0xEEDD8200
#define Y_GOLDENROD                 0xDAA52000
#define Y_DARKGOLDENROD             0xB8860B00
#define Y_DARK_GOLDENROD            0xB8860B00
#define Y_ROSYBROWN                 0xBC8F8F00
#define Y_ROSY_BROWN                0xBC8F8F00
#define Y_INDIANRED                 0xCD5C5C00
#define Y_INDIAN_RED                0xCD5C5C00
#define Y_SADDLEBROWN               0x8B451300
#define Y_SADDLE_BROWN              0x8B451300
#define Y_SIENNA                    0xA0522D00
#define Y_PERU                      0xCD853F00
#define Y_BURLYWOOD                 0xDEB88700
#define Y_BEIGE                     0xF5F5DC00
#define Y_WHEAT                     0xF5DEB300
#define Y_SANDYBROWN                0xF4A46000
#define Y_SANDY_BROWN               0xF4A46000
#define Y_TAN                       0xD2B48C00
#define Y_CHOCOLATE                 0xD2691E00
#define Y_FIREBRICK                 0xB2222200
#define Y_BROWN                     0xA52A2A00
#define Y_DARKSALMON                0xE9967A00
#define Y_DARK_SALMON               0xE9967A00
#define Y_SALMON                    0xFA807200
#define Y_LIGHTSALMON               0xFFA07A00
#define Y_LIGHT_SALMON              0xFFA07A00
#define Y_ORANGE                    0xFFA50000
#define Y_DARKORANGE                0xFF8C0000
#define Y_DARK_ORANGE               0xFF8C0000
#define Y_CORAL                     0xFF7F5000
#define Y_LIGHTCORAL                0xF0808000
#define Y_LIGHT_CORAL               0xF0808000
#define Y_TOMATO                    0xFF634700
#define Y_ORANGERED                 0xFF450000
#define Y_ORANGE_RED                0xFF450000
#define Y_RED                       0xFF000000
#define Y_HOTPINK                   0xFF69B400
#define Y_HOT_PINK                  0xFF69B400
#define Y_DEEPPINK                  0xFF149300
#define Y_DEEP_PINK                 0xFF149300
#define Y_PINK                      0xFFC0CB00
#define Y_LIGHTPINK                 0xFFB6C100
#define Y_LIGHT_PINK                0xFFB6C100
#define Y_PALEVIOLETRED             0xDB709300
#define Y_PALE_VIOLET_RED           0xDB709300
#define Y_MAROON                    0xB0306000
#define Y_MEDIUMVIOLETRED           0xC7158500
#define Y_MEDIUM_VIOLET_RED         0xC7158500
#define Y_VIOLETRED                 0xD0209000
#define Y_VIOLET_RED                0xD0209000
#define Y_MAGENTA                   0xFF00FF00
#define Y_VIOLET                    0xEE82EE00
#define Y_PLUM                      0xDDA0DD00
#define Y_ORCHID                    0xDA70D600
#define Y_MEDIUMORCHID              0xBA55D300
#define Y_MEDIUM_ORCHID             0xBA55D300
#define Y_DARKORCHID                0x9932CC00
#define Y_DARK_ORCHID               0x9932CC00
#define Y_DARKVIOLET                0x9400D300
#define Y_DARK_VIOLET               0x9400D300
#define Y_BLUEVIOLET                0x8A2BE200
#define Y_BLUE_VIOLET               0x8A2BE200
#define Y_PURPLE                    0xA020F000
#define Y_MEDIUMPURPLE              0x9370DB00
#define Y_MEDIUM_PURPLE             0x9370DB00
#define Y_THISTLE                   0xD8BFD800
#define Y_SNOW1                     0xFFFAFA00
#define Y_SNOW_1                    0xFFFAFA00
#define Y_SNOW2                     0xEEE9E900
#define Y_SNOW_2                    0xEEE9E900
#define Y_SNOW3                     0xCDC9C900
#define Y_SNOW_3                    0xCDC9C900
#define Y_SNOW4                     0x8B898900
#define Y_SNOW_4                    0x8B898900
#define Y_SEASHELL1                 0xFFF5EE00
#define Y_SEASHELL_1                0xFFF5EE00
#define Y_SEASHELL2                 0xEEE5DE00
#define Y_SEASHELL_2                0xEEE5DE00
#define Y_SEASHELL3                 0xCDC5BF00
#define Y_SEASHELL_3                0xCDC5BF00
#define Y_SEASHELL4                 0x8B868200
#define Y_SEASHELL_4                0x8B868200
#define Y_ANTIQUEWHITE1             0xFFEFDB00
#define Y_ANTIQUE_WHITE_1           0xFFEFDB00
#define Y_ANTIQUEWHITE2             0xEEDFCC00
#define Y_ANTIQUE_WHITE_2           0xEEDFCC00
#define Y_ANTIQUEWHITE3             0xCDC0B000
#define Y_ANTIQUE_WHITE_3           0xCDC0B000
#define Y_ANTIQUEWHITE4             0x8B837800
#define Y_ANTIQUE_WHITE_4           0x8B837800
#define Y_BISQUE1                   0xFFE4C400
#define Y_BISQUE_1                  0xFFE4C400
#define Y_BISQUE2                   0xEED5B700
#define Y_BISQUE_2                  0xEED5B700
#define Y_BISQUE3                   0xCDB79E00
#define Y_BISQUE_3                  0xCDB79E00
#define Y_BISQUE4                   0x8B7D6B00
#define Y_BISQUE_4                  0x8B7D6B00
#define Y_PEACHPUFF1                0xFFDAB900
#define Y_PEACH_PUFF_1              0xFFDAB900
#define Y_PEACHPUFF2                0xEECBAD00
#define Y_PEACH_PUFF_2              0xEECBAD00
#define Y_PEACHPUFF3                0xCDAF9500
#define Y_PEACH_PUFF_3              0xCDAF9500
#define Y_PEACHPUFF4                0x8B776500
#define Y_PEACH_PUFF_4              0x8B776500
#define Y_NAVAJOWHITE1              0xFFDEAD00
#define Y_NAVAJO_WHITE_1            0xFFDEAD00
#define Y_NAVAJOWHITE2              0xEECFA100
#define Y_NAVAJO_WHITE_2            0xEECFA100
#define Y_NAVAJOWHITE3              0xCDB38B00
#define Y_NAVAJO_WHITE_3            0xCDB38B00
#define Y_NAVAJOWHITE4              0x8B795E00
#define Y_NAVAJO_WHITE_4            0x8B795E00
#define Y_LEMONCHIFFON1             0xFFFACD00
#define Y_LEMON_CHIFFON_1           0xFFFACD00
#define Y_LEMONCHIFFON2             0xEEE9BF00
#define Y_LEMON_CHIFFON_2           0xEEE9BF00
#define Y_LEMONCHIFFON3             0xCDC9A500
#define Y_LEMON_CHIFFON_3           0xCDC9A500
#define Y_LEMONCHIFFON4             0x8B897000
#define Y_LEMON_CHIFFON_4           0x8B897000
#define Y_CORNSILK1                 0xFFF8DC00
#define Y_CORNSILK_1                0xFFF8DC00
#define Y_CORNSILK2                 0xEEE8CD00
#define Y_CORNSILK_2                0xEEE8CD00
#define Y_CORNSILK3                 0xCDC8B100
#define Y_CORNSILK_3                0xCDC8B100
#define Y_CORNSILK4                 0x8B887800
#define Y_CORNSILK_4                0x8B887800
#define Y_IVORY1                    0xFFFFF000
#define Y_IVORY_1                   0xFFFFF000
#define Y_IVORY2                    0xEEEEE000
#define Y_IVORY_2                   0xEEEEE000
#define Y_IVORY3                    0xCDCDC100
#define Y_IVORY_3                   0xCDCDC100
#define Y_IVORY4                    0x8B8B8300
#define Y_IVORY_4                   0x8B8B8300
#define Y_HONEYDEW1                 0xF0FFF000
#define Y_HONEYDEW_1                0xF0FFF000
#define Y_HONEYDEW2                 0xE0EEE000
#define Y_HONEYDEW_2                0xE0EEE000
#define Y_HONEYDEW3                 0xC1CDC100
#define Y_HONEYDEW_3                0xC1CDC100
#define Y_HONEYDEW4                 0x838B8300
#define Y_HONEYDEW_4                0x838B8300
#define Y_LAVENDERBLUSH1            0xFFF0F500
#define Y_LAVENDER_BLUSH_1          0xFFF0F500
#define Y_LAVENDERBLUSH2            0xEEE0E500
#define Y_LAVENDER_BLUSH_2          0xEEE0E500
#define Y_LAVENDERBLUSH3            0xCDC1C500
#define Y_LAVENDER_BLUSH_3          0xCDC1C500
#define Y_LAVENDERBLUSH4            0x8B838600
#define Y_LAVENDER_BLUSH_4          0x8B838600
#define Y_MISTYROSE1                0xFFE4E100
#define Y_MISTY_ROSE_1              0xFFE4E100
#define Y_MISTYROSE2                0xEED5D200
#define Y_MISTY_ROSE_2              0xEED5D200
#define Y_MISTYROSE3                0xCDB7B500
#define Y_MISTY_ROSE_3              0xCDB7B500
#define Y_MISTYROSE4                0x8B7D7B00
#define Y_MISTY_ROSE_4              0x8B7D7B00
#define Y_AZURE1                    0xF0FFFF00
#define Y_AZURE_1                   0xF0FFFF00
#define Y_AZURE2                    0xE0EEEE00
#define Y_AZURE_2                   0xE0EEEE00
#define Y_AZURE3                    0xC1CDCD00
#define Y_AZURE_3                   0xC1CDCD00
#define Y_AZURE4                    0x838B8B00
#define Y_AZURE_4                   0x838B8B00
#define Y_SLATEBLUE1                0x836FFF00
#define Y_SLATE_BLUE_1              0x836FFF00
#define Y_SLATEBLUE2                0x7A67EE00
#define Y_SLATE_BLUE_2              0x7A67EE00
#define Y_SLATEBLUE3                0x6959CD00
#define Y_SLATE_BLUE_3              0x6959CD00
#define Y_SLATEBLUE4                0x473C8B00
#define Y_SLATE_BLUE_4              0x473C8B00
#define Y_ROYALBLUE1                0x4876FF00
#define Y_ROYAL_BLUE_1              0x4876FF00
#define Y_ROYALBLUE2                0x436EEE00
#define Y_ROYAL_BLUE_2              0x436EEE00
#define Y_ROYALBLUE3                0x3A5FCD00
#define Y_ROYAL_BLUE_3              0x3A5FCD00
#define Y_ROYALBLUE4                0x27408B00
#define Y_ROYAL_BLUE_4              0x27408B00
#define Y_BLUE1                     0x0000FF00
#define Y_BLUE_1                    0x0000FF00
#define Y_BLUE2                     0x0000EE00
#define Y_BLUE_2                    0x0000EE00
#define Y_BLUE3                     0x0000CD00
#define Y_BLUE_3                    0x0000CD00
#define Y_BLUE4                     0x00008B00
#define Y_BLUE_4                    0x00008B00
#define Y_DODGERBLUE1               0x1E90FF00
#define Y_DODGER_BLUE_1             0x1E90FF00
#define Y_DODGERBLUE2               0x1C86EE00
#define Y_DODGER_BLUE_2             0x1C86EE00
#define Y_DODGERBLUE3               0x1874CD00
#define Y_DODGER_BLUE_3             0x1874CD00
#define Y_DODGERBLUE4               0x104E8B00
#define Y_DODGER_BLUE_4             0x104E8B00
#define Y_STEELBLUE1                0x63B8FF00
#define Y_STEEL_BLUE_1              0x63B8FF00
#define Y_STEELBLUE2                0x5CACEE00
#define Y_STEEL_BLUE_2              0x5CACEE00
#define Y_STEELBLUE3                0x4F94CD00
#define Y_STEEL_BLUE_3              0x4F94CD00
#define Y_STEELBLUE4                0x36648B00
#define Y_STEEL_BLUE_4              0x36648B00
#define Y_DEEPSKYBLUE1              0x00BFFF00
#define Y_DEEP_SKY_BLUE_1           0x00BFFF00
#define Y_DEEPSKYBLUE2              0x00B2EE00
#define Y_DEEP_SKY_BLUE_2           0x00B2EE00
#define Y_DEEPSKYBLUE3              0x009ACD00
#define Y_DEEP_SKY_BLUE_3           0x009ACD00
#define Y_DEEPSKYBLUE4              0x00688B00
#define Y_DEEP_SKY_BLUE_4           0x00688B00
#define Y_SKYBLUE1                  0x87CEFF00
#define Y_SKY_BLUE_1                0x87CEFF00
#define Y_SKYBLUE2                  0x7EC0EE00
#define Y_SKY_BLUE_2                0x7EC0EE00
#define Y_SKYBLUE3                  0x6CA6CD00
#define Y_SKY_BLUE_3                0x6CA6CD00
#define Y_SKYBLUE4                  0x4A708B00
#define Y_SKY_BLUE_4                0x4A708B00
#define Y_LIGHTSKYBLUE1             0xB0E2FF00
#define Y_LIGHT_SKY_BLUE_1          0xB0E2FF00
#define Y_LIGHTSKYBLUE2             0xA4D3EE00
#define Y_LIGHT_SKY_BLUE_2          0xA4D3EE00
#define Y_LIGHTSKYBLUE3             0x8DB6CD00
#define Y_LIGHT_SKY_BLUE_3          0x8DB6CD00
#define Y_LIGHTSKYBLUE4             0x607B8B00
#define Y_LIGHT_SKY_BLUE_4          0x607B8B00
#define Y_SLATEGRAY1                0xC6E2FF00
#define Y_SLATE_GRAY_1              0xC6E2FF00
#define Y_SLATEGRAY2                0xB9D3EE00
#define Y_SLATE_GRAY_2              0xB9D3EE00
#define Y_SLATEGRAY3                0x9FB6CD00
#define Y_SLATE_GRAY_3              0x9FB6CD00
#define Y_SLATEGRAY4                0x6C7B8B00
#define Y_SLATE_GRAY_4              0x6C7B8B00
#define Y_SLATEGREY1                0xC6E2FF00
#define Y_SLATE_GREY_1              0xC6E2FF00
#define Y_SLATEGREY2                0xB9D3EE00
#define Y_SLATE_GREY_2              0xB9D3EE00
#define Y_SLATEGREY3                0x9FB6CD00
#define Y_SLATE_GREY_3              0x9FB6CD00
#define Y_SLATEGREY4                0x6C7B8B00
#define Y_SLATE_GREY_4              0x6C7B8B00
#define Y_LIGHTSTEELBLUE1           0xCAE1FF00
#define Y_LIGHT_STEEL_BLUE_1        0xCAE1FF00
#define Y_LIGHTSTEELBLUE2           0xBCD2EE00
#define Y_LIGHT_STEEL_BLUE_2        0xBCD2EE00
#define Y_LIGHTSTEELBLUE3           0xA2B5CD00
#define Y_LIGHT_STEEL_BLUE_3        0xA2B5CD00
#define Y_LIGHTSTEELBLUE4           0x6E7B8B00
#define Y_LIGHT_STEEL_BLUE_4        0x6E7B8B00
#define Y_LIGHTBLUE1                0xBFEFFF00
#define Y_LIGHT_BLUE_1              0xBFEFFF00
#define Y_LIGHTBLUE2                0xB2DFEE00
#define Y_LIGHT_BLUE_2              0xB2DFEE00
#define Y_LIGHTBLUE3                0x9AC0CD00
#define Y_LIGHT_BLUE_3              0x9AC0CD00
#define Y_LIGHTBLUE4                0x68838B00
#define Y_LIGHT_BLUE_4              0x68838B00
#define Y_LIGHTCYAN1                0xE0FFFF00
#define Y_LIGHT_CYAN_1              0xE0FFFF00
#define Y_LIGHTCYAN2                0xD1EEEE00
#define Y_LIGHT_CYAN_2              0xD1EEEE00
#define Y_LIGHTCYAN3                0xB4CDCD00
#define Y_LIGHT_CYAN_3              0xB4CDCD00
#define Y_LIGHTCYAN4                0x7A8B8B00
#define Y_LIGHT_CYAN_4              0x7A8B8B00
#define Y_PALETURQUOISE1            0xBBFFFF00
#define Y_PALE_TURQUOISE_1          0xBBFFFF00
#define Y_PALETURQUOISE2            0xAEEEEE00
#define Y_PALE_TURQUOISE_2          0xAEEEEE00
#define Y_PALETURQUOISE3            0x96CDCD00
#define Y_PALE_TURQUOISE_3          0x96CDCD00
#define Y_PALETURQUOISE4            0x668B8B00
#define Y_PALE_TURQUOISE_4          0x668B8B00
#define Y_CADETBLUE1                0x98F5FF00
#define Y_CADET_BLUE_1              0x98F5FF00
#define Y_CADETBLUE2                0x8EE5EE00
#define Y_CADET_BLUE_2              0x8EE5EE00
#define Y_CADETBLUE3                0x7AC5CD00
#define Y_CADET_BLUE_3              0x7AC5CD00
#define Y_CADETBLUE4                0x53868B00
#define Y_CADET_BLUE_4              0x53868B00
#define Y_TURQUOISE1                0x00F5FF00
#define Y_TURQUOISE_1               0x00F5FF00
#define Y_TURQUOISE2                0x00E5EE00
#define Y_TURQUOISE_2               0x00E5EE00
#define Y_TURQUOISE3                0x00C5CD00
#define Y_TURQUOISE_3               0x00C5CD00
#define Y_TURQUOISE4                0x00868B00
#define Y_TURQUOISE_4               0x00868B00
#define Y_CYAN1                     0x00FFFF00
#define Y_CYAN_1                    0x00FFFF00
#define Y_CYAN2                     0x00EEEE00
#define Y_CYAN_2                    0x00EEEE00
#define Y_CYAN3                     0x00CDCD00
#define Y_CYAN_3                    0x00CDCD00
#define Y_CYAN4                     0x008B8B00
#define Y_CYAN_4                    0x008B8B00
#define Y_DARKSLATEGRAY1            0x97FFFF00
#define Y_DARK_SLATE_GRAY_1         0x97FFFF00
#define Y_DARKSLATEGRAY2            0x8DEEEE00
#define Y_DARK_SLATE_GRAY_2         0x8DEEEE00
#define Y_DARKSLATEGRAY3            0x79CDCD00
#define Y_DARK_SLATE_GRAY_3         0x79CDCD00
#define Y_DARKSLATEGRAY4            0x528B8B00
#define Y_DARK_SLATE_GRAY_4         0x528B8B00
#define Y_DARKSLATEGREY1            0x97FFFF00
#define Y_DARK_SLATE_GREY_1         0x97FFFF00
#define Y_DARKSLATEGREY2            0x8DEEEE00
#define Y_DARK_SLATE_GREY_2         0x8DEEEE00
#define Y_DARKSLATEGREY3            0x79CDCD00
#define Y_DARK_SLATE_GREY_3         0x79CDCD00
#define Y_DARKSLATEGREY4            0x528B8B00
#define Y_DARK_SLATE_GREY_4         0x528B8B00
#define Y_AQUAMARINE1               0x7FFFD400
#define Y_AQUAMARINE_1              0x7FFFD400
#define Y_AQUAMARINE2               0x76EEC600
#define Y_AQUAMARINE_2              0x76EEC600
#define Y_AQUAMARINE3               0x66CDAA00
#define Y_AQUAMARINE_3              0x66CDAA00
#define Y_AQUAMARINE4               0x458B7400
#define Y_AQUAMARINE_4              0x458B7400
#define Y_DARKSEAGREEN1             0xC1FFC100
#define Y_DARK_SEA_GREEN_1          0xC1FFC100
#define Y_DARKSEAGREEN2             0xB4EEB400
#define Y_DARK_SEA_GREEN_2          0xB4EEB400
#define Y_DARKSEAGREEN3             0x9BCD9B00
#define Y_DARK_SEA_GREEN_3          0x9BCD9B00
#define Y_DARKSEAGREEN4             0x698B6900
#define Y_DARK_SEA_GREEN_4          0x698B6900
#define Y_SEAGREEN1                 0x54FF9F00
#define Y_SEA_GREEN_1               0x54FF9F00
#define Y_SEAGREEN2                 0x4EEE9400
#define Y_SEA_GREEN_2               0x4EEE9400
#define Y_SEAGREEN3                 0x43CD8000
#define Y_SEA_GREEN_3               0x43CD8000
#define Y_SEAGREEN4                 0x2E8B5700
#define Y_SEA_GREEN_4               0x2E8B5700
#define Y_PALEGREEN1                0x9AFF9A00
#define Y_PALE_GREEN_1              0x9AFF9A00
#define Y_PALEGREEN2                0x90EE9000
#define Y_PALE_GREEN_2              0x90EE9000
#define Y_PALEGREEN3                0x7CCD7C00
#define Y_PALE_GREEN_3              0x7CCD7C00
#define Y_PALEGREEN4                0x548B5400
#define Y_PALE_GREEN_4              0x548B5400
#define Y_SPRINGGREEN1              0x00FF7F00
#define Y_SPRING_GREEN_1            0x00FF7F00
#define Y_SPRINGGREEN2              0x00EE7600
#define Y_SPRING_GREEN_2            0x00EE7600
#define Y_SPRINGGREEN3              0x00CD6600
#define Y_SPRING_GREEN_3            0x00CD6600
#define Y_SPRINGGREEN4              0x008B4500
#define Y_SPRING_GREEN_4            0x008B4500
#define Y_GREEN1                    0x00FF0000
#define Y_GREEN_1                   0x00FF0000
#define Y_GREEN2                    0x00EE0000
#define Y_GREEN_2                   0x00EE0000
#define Y_GREEN3                    0x00CD0000
#define Y_GREEN_3                   0x00CD0000
#define Y_GREEN4                    0x008B0000
#define Y_GREEN_4                   0x008B0000
#define Y_CHARTREUSE1               0x7FFF0000
#define Y_CHARTREUSE_1              0x7FFF0000
#define Y_CHARTREUSE2               0x76EE0000
#define Y_CHARTREUSE_2              0x76EE0000
#define Y_CHARTREUSE3               0x66CD0000
#define Y_CHARTREUSE_3              0x66CD0000
#define Y_CHARTREUSE4               0x458B0000
#define Y_CHARTREUSE_4              0x458B0000
#define Y_OLIVEDRAB1                0xC0FF3E00
#define Y_OLIVE_DRAB_1              0xC0FF3E00
#define Y_OLIVEDRAB2                0xB3EE3A00
#define Y_OLIVE_DRAB_2              0xB3EE3A00
#define Y_OLIVEDRAB3                0x9ACD3200
#define Y_OLIVE_DRAB_3              0x9ACD3200
#define Y_OLIVEDRAB4                0x698B2200
#define Y_OLIVE_DRAB_4              0x698B2200
#define Y_DARKOLIVEGREEN1           0xCAFF7000
#define Y_DARK_OLIVE_GREEN_1        0xCAFF7000
#define Y_DARKOLIVEGREEN2           0xBCEE6800
#define Y_DARK_OLIVE_GREEN_2        0xBCEE6800
#define Y_DARKOLIVEGREEN3           0xA2CD5A00
#define Y_DARK_OLIVE_GREEN_3        0xA2CD5A00
#define Y_DARKOLIVEGREEN4           0x6E8B3D00
#define Y_DARK_OLIVE_GREEN_4        0x6E8B3D00
#define Y_KHAKI1                    0xFFF68F00
#define Y_KHAKI_1                   0xFFF68F00
#define Y_KHAKI2                    0xEEE68500
#define Y_KHAKI_2                   0xEEE68500
#define Y_KHAKI3                    0xCDC67300
#define Y_KHAKI_3                   0xCDC67300
#define Y_KHAKI4                    0x8B864E00
#define Y_KHAKI_4                   0x8B864E00
#define Y_LIGHTGOLDENROD1           0xFFEC8B00
#define Y_LIGHT_GOLDENROD_1         0xFFEC8B00
#define Y_LIGHTGOLDENROD2           0xEEDC8200
#define Y_LIGHT_GOLDENROD_2         0xEEDC8200
#define Y_LIGHTGOLDENROD3           0xCDBE7000
#define Y_LIGHT_GOLDENROD_3         0xCDBE7000
#define Y_LIGHTGOLDENROD4           0x8B814C00
#define Y_LIGHT_GOLDENROD_4         0x8B814C00
#define Y_LIGHTYELLOW1              0xFFFFE000
#define Y_LIGHT_YELLOW_1            0xFFFFE000
#define Y_LIGHTYELLOW2              0xEEEED100
#define Y_LIGHT_YELLOW_2            0xEEEED100
#define Y_LIGHTYELLOW3              0xCDCDB400
#define Y_LIGHT_YELLOW_3            0xCDCDB400
#define Y_LIGHTYELLOW4              0x8B8B7A00
#define Y_LIGHT_YELLOW_4            0x8B8B7A00
#define Y_YELLOW1                   0xFFFF0000
#define Y_YELLOW_1                  0xFFFF0000
#define Y_YELLOW2                   0xEEEE0000
#define Y_YELLOW_2                  0xEEEE0000
#define Y_YELLOW3                   0xCDCD0000
#define Y_YELLOW_3                  0xCDCD0000
#define Y_YELLOW4                   0x8B8B0000
#define Y_YELLOW_4                  0x8B8B0000
#define Y_GOLD1                     0xFFD70000
#define Y_GOLD_1                    0xFFD70000
#define Y_GOLD2                     0xEEC90000
#define Y_GOLD_2                    0xEEC90000
#define Y_GOLD3                     0xCDAD0000
#define Y_GOLD_3                    0xCDAD0000
#define Y_GOLD4                     0x8B750000
#define Y_GOLD_4                    0x8B750000
#define Y_GOLDENROD1                0xFFC12500
#define Y_GOLDENROD_1               0xFFC12500
#define Y_GOLDENROD2                0xEEB42200
#define Y_GOLDENROD_2               0xEEB42200
#define Y_GOLDENROD3                0xCD9B1D00
#define Y_GOLDENROD_3               0xCD9B1D00
#define Y_GOLDENROD4                0x8B691400
#define Y_GOLDENROD_4               0x8B691400
#define Y_DARKGOLDENROD1            0xFFB90F00
#define Y_DARK_GOLDENROD_1          0xFFB90F00
#define Y_DARKGOLDENROD2            0xEEAD0E00
#define Y_DARK_GOLDENROD_2          0xEEAD0E00
#define Y_DARKGOLDENROD3            0xCD950C00
#define Y_DARK_GOLDENROD_3          0xCD950C00
#define Y_DARKGOLDENROD4            0x8B650800
#define Y_DARK_GOLDENROD_4          0x8B650800
#define Y_ROSYBROWN1                0xFFC1C100
#define Y_ROSY_BROWN_1              0xFFC1C100
#define Y_ROSYBROWN2                0xEEB4B400
#define Y_ROSY_BROWN_2              0xEEB4B400
#define Y_ROSYBROWN3                0xCD9B9B00
#define Y_ROSY_BROWN_3              0xCD9B9B00
#define Y_ROSYBROWN4                0x8B696900
#define Y_ROSY_BROWN_4              0x8B696900
#define Y_INDIANRED1                0xFF6A6A00
#define Y_INDIAN_RED_1              0xFF6A6A00
#define Y_INDIANRED2                0xEE636300
#define Y_INDIAN_RED_2              0xEE636300
#define Y_INDIANRED3                0xCD555500
#define Y_INDIAN_RED_3              0xCD555500
#define Y_INDIANRED4                0x8B3A3A00
#define Y_INDIAN_RED_4              0x8B3A3A00
#define Y_SIENNA1                   0xFF824700
#define Y_SIENNA_1                  0xFF824700
#define Y_SIENNA2                   0xEE794200
#define Y_SIENNA_2                  0xEE794200
#define Y_SIENNA3                   0xCD683900
#define Y_SIENNA_3                  0xCD683900
#define Y_SIENNA4                   0x8B472600
#define Y_SIENNA_4                  0x8B472600
#define Y_BURLYWOOD1                0xFFD39B00
#define Y_BURLYWOOD_1               0xFFD39B00
#define Y_BURLYWOOD2                0xEEC59100
#define Y_BURLYWOOD_2               0xEEC59100
#define Y_BURLYWOOD3                0xCDAA7D00
#define Y_BURLYWOOD_3               0xCDAA7D00
#define Y_BURLYWOOD4                0x8B735500
#define Y_BURLYWOOD_4               0x8B735500
#define Y_WHEAT1                    0xFFE7BA00
#define Y_WHEAT_1                   0xFFE7BA00
#define Y_WHEAT2                    0xEED8AE00
#define Y_WHEAT_2                   0xEED8AE00
#define Y_WHEAT3                    0xCDBA9600
#define Y_WHEAT_3                   0xCDBA9600
#define Y_WHEAT4                    0x8B7E6600
#define Y_WHEAT_4                   0x8B7E6600
#define Y_TAN1                      0xFFA54F00
#define Y_TAN_1                     0xFFA54F00
#define Y_TAN2                      0xEE9A4900
#define Y_TAN_2                     0xEE9A4900
#define Y_TAN3                      0xCD853F00
#define Y_TAN_3                     0xCD853F00
#define Y_TAN4                      0x8B5A2B00
#define Y_TAN_4                     0x8B5A2B00
#define Y_CHOCOLATE1                0xFF7F2400
#define Y_CHOCOLATE_1               0xFF7F2400
#define Y_CHOCOLATE2                0xEE762100
#define Y_CHOCOLATE_2               0xEE762100
#define Y_CHOCOLATE3                0xCD661D00
#define Y_CHOCOLATE_3               0xCD661D00
#define Y_CHOCOLATE4                0x8B451300
#define Y_CHOCOLATE_4               0x8B451300
#define Y_FIREBRICK1                0xFF303000
#define Y_FIREBRICK_1               0xFF303000
#define Y_FIREBRICK2                0xEE2C2C00
#define Y_FIREBRICK_2               0xEE2C2C00
#define Y_FIREBRICK3                0xCD262600
#define Y_FIREBRICK_3               0xCD262600
#define Y_FIREBRICK4                0x8B1A1A00
#define Y_FIREBRICK_4               0x8B1A1A00
#define Y_BROWN1                    0xFF404000
#define Y_BROWN_1                   0xFF404000
#define Y_BROWN2                    0xEE3B3B00
#define Y_BROWN_2                   0xEE3B3B00
#define Y_BROWN3                    0xCD333300
#define Y_BROWN_3                   0xCD333300
#define Y_BROWN4                    0x8B232300
#define Y_BROWN_4                   0x8B232300
#define Y_SALMON1                   0xFF8C6900
#define Y_SALMON_1                  0xFF8C6900
#define Y_SALMON2                   0xEE826200
#define Y_SALMON_2                  0xEE826200
#define Y_SALMON3                   0xCD705400
#define Y_SALMON_3                  0xCD705400
#define Y_SALMON4                   0x8B4C3900
#define Y_SALMON_4                  0x8B4C3900
#define Y_LIGHTSALMON1              0xFFA07A00
#define Y_LIGHT_SALMON_1            0xFFA07A00
#define Y_LIGHTSALMON2              0xEE957200
#define Y_LIGHT_SALMON_2            0xEE957200
#define Y_LIGHTSALMON3              0xCD816200
#define Y_LIGHT_SALMON_3            0xCD816200
#define Y_LIGHTSALMON4              0x8B574200
#define Y_LIGHT_SALMON_4            0x8B574200
#define Y_ORANGE1                   0xFFA50000
#define Y_ORANGE_1                  0xFFA50000
#define Y_ORANGE2                   0xEE9A0000
#define Y_ORANGE_2                  0xEE9A0000
#define Y_ORANGE3                   0xCD850000
#define Y_ORANGE_3                  0xCD850000
#define Y_ORANGE4                   0x8B5A0000
#define Y_ORANGE_4                  0x8B5A0000
#define Y_DARKORANGE1               0xFF7F0000
#define Y_DARK_ORANGE_1             0xFF7F0000
#define Y_DARKORANGE2               0xEE760000
#define Y_DARK_ORANGE_2             0xEE760000
#define Y_DARKORANGE3               0xCD660000
#define Y_DARK_ORANGE_3             0xCD660000
#define Y_DARKORANGE4               0x8B450000
#define Y_DARK_ORANGE_4             0x8B450000
#define Y_CORAL1                    0xFF725600
#define Y_CORAL_1                   0xFF725600
#define Y_CORAL2                    0xEE6A5000
#define Y_CORAL_2                   0xEE6A5000
#define Y_CORAL3                    0xCD5B4500
#define Y_CORAL_3                   0xCD5B4500
#define Y_CORAL4                    0x8B3E2F00
#define Y_CORAL_4                   0x8B3E2F00
#define Y_TOMATO1                   0xFF634700
#define Y_TOMATO_1                  0xFF634700
#define Y_TOMATO2                   0xEE5C4200
#define Y_TOMATO_2                  0xEE5C4200
#define Y_TOMATO3                   0xCD4F3900
#define Y_TOMATO_3                  0xCD4F3900
#define Y_TOMATO4                   0x8B362600
#define Y_TOMATO_4                  0x8B362600
#define Y_ORANGERED1                0xFF450000
#define Y_ORANGE_RED_1              0xFF450000
#define Y_ORANGERED2                0xEE400000
#define Y_ORANGE_RED_2              0xEE400000
#define Y_ORANGERED3                0xCD370000
#define Y_ORANGE_RED_3              0xCD370000
#define Y_ORANGERED4                0x8B250000
#define Y_ORANGE_RED_4              0x8B250000
#define Y_RED1                      0xFF000000
#define Y_RED_1                     0xFF000000
#define Y_RED2                      0xEE000000
#define Y_RED_2                     0xEE000000
#define Y_RED3                      0xCD000000
#define Y_RED_3                     0xCD000000
#define Y_RED4                      0x8B000000
#define Y_RED_4                     0x8B000000
#define Y_DEEPPINK1                 0xFF149300
#define Y_DEEP_PINK_1               0xFF149300
#define Y_DEEPPINK2                 0xEE128900
#define Y_DEEP_PINK_2               0xEE128900
#define Y_DEEPPINK3                 0xCD107600
#define Y_DEEP_PINK_3               0xCD107600
#define Y_DEEPPINK4                 0x8B0A5000
#define Y_DEEP_PINK_4               0x8B0A5000
#define Y_HOTPINK1                  0xFF6EB400
#define Y_HOT_PINK_1                0xFF6EB400
#define Y_HOTPINK2                  0xEE6AA700
#define Y_HOT_PINK_2                0xEE6AA700
#define Y_HOTPINK3                  0xCD609000
#define Y_HOT_PINK_3                0xCD609000
#define Y_HOTPINK4                  0x8B3A6200
#define Y_HOT_PINK_4                0x8B3A6200
#define Y_PINK1                     0xFFB5C500
#define Y_PINK_1                    0xFFB5C500
#define Y_PINK2                     0xEEA9B800
#define Y_PINK_2                    0xEEA9B800
#define Y_PINK3                     0xCD919E00
#define Y_PINK_3                    0xCD919E00
#define Y_PINK4                     0x8B636C00
#define Y_PINK_4                    0x8B636C00
#define Y_LIGHTPINK1                0xFFAEB900
#define Y_LIGHT_PINK_1              0xFFAEB900
#define Y_LIGHTPINK2                0xEEA2AD00
#define Y_LIGHT_PINK_2              0xEEA2AD00
#define Y_LIGHTPINK3                0xCD8C9500
#define Y_LIGHT_PINK_3              0xCD8C9500
#define Y_LIGHTPINK4                0x8B5F6500
#define Y_LIGHT_PINK_4              0x8B5F6500
#define Y_PALEVIOLETRED1            0xFF82AB00
#define Y_PALE_VIOLET_RED_1         0xFF82AB00
#define Y_PALEVIOLETRED2            0xEE799F00
#define Y_PALE_VIOLET_RED_2         0xEE799F00
#define Y_PALEVIOLETRED3            0xCD688900
#define Y_PALE_VIOLET_RED_3         0xCD688900
#define Y_PALEVIOLETRED4            0x8B475D00
#define Y_PALE_VIOLET_RED_4         0x8B475D00
#define Y_MAROON1                   0xFF34B300
#define Y_MAROON_1                  0xFF34B300
#define Y_MAROON2                   0xEE30A700
#define Y_MAROON_2                  0xEE30A700
#define Y_MAROON3                   0xCD299000
#define Y_MAROON_3                  0xCD299000
#define Y_MAROON4                   0x8B1C6200
#define Y_MAROON_4                  0x8B1C6200
#define Y_VIOLETRED1                0xFF3E9600
#define Y_VIOLET_RED_1              0xFF3E9600
#define Y_VIOLETRED2                0xEE3A8C00
#define Y_VIOLET_RED_2              0xEE3A8C00
#define Y_VIOLETRED3                0xCD327800
#define Y_VIOLET_RED_3              0xCD327800
#define Y_VIOLETRED4                0x8B225200
#define Y_VIOLET_RED_4              0x8B225200
#define Y_MAGENTA1                  0xFF00FF00
#define Y_MAGENTA_1                 0xFF00FF00
#define Y_MAGENTA2                  0xEE00EE00
#define Y_MAGENTA_2                 0xEE00EE00
#define Y_MAGENTA3                  0xCD00CD00
#define Y_MAGENTA_3                 0xCD00CD00
#define Y_MAGENTA4                  0x8B008B00
#define Y_MAGENTA_4                 0x8B008B00
#define Y_ORCHID1                   0xFF83FA00
#define Y_ORCHID_1                  0xFF83FA00
#define Y_ORCHID2                   0xEE7AE900
#define Y_ORCHID_2                  0xEE7AE900
#define Y_ORCHID3                   0xCD69C900
#define Y_ORCHID_3                  0xCD69C900
#define Y_ORCHID4                   0x8B478900
#define Y_ORCHID_4                  0x8B478900
#define Y_PLUM1                     0xFFBBFF00
#define Y_PLUM_1                    0xFFBBFF00
#define Y_PLUM2                     0xEEAEEE00
#define Y_PLUM_2                    0xEEAEEE00
#define Y_PLUM3                     0xCD96CD00
#define Y_PLUM_3                    0xCD96CD00
#define Y_PLUM4                     0x8B668B00
#define Y_PLUM_4                    0x8B668B00
#define Y_MEDIUMORCHID1             0xE066FF00
#define Y_MEDIUM_ORCHID_1           0xE066FF00
#define Y_MEDIUMORCHID2             0xD15FEE00
#define Y_MEDIUM_ORCHID_2           0xD15FEE00
#define Y_MEDIUMORCHID3             0xB452CD00
#define Y_MEDIUM_ORCHID_3           0xB452CD00
#define Y_MEDIUMORCHID4             0x7A378B00
#define Y_MEDIUM_ORCHID_4           0x7A378B00
#define Y_DARKORCHID1               0xBF3EFF00
#define Y_DARK_ORCHID_1             0xBF3EFF00
#define Y_DARKORCHID2               0xB23AEE00
#define Y_DARK_ORCHID_2             0xB23AEE00
#define Y_DARKORCHID3               0x9A32CD00
#define Y_DARK_ORCHID_3             0x9A32CD00
#define Y_DARKORCHID4               0x68228B00
#define Y_DARK_ORCHID_4             0x68228B00
#define Y_PURPLE1                   0x9B30FF00
#define Y_PURPLE_1                  0x9B30FF00
#define Y_PURPLE2                   0x912CEE00
#define Y_PURPLE_2                  0x912CEE00
#define Y_PURPLE3                   0x7D26CD00
#define Y_PURPLE_3                  0x7D26CD00
#define Y_PURPLE4                   0x551A8B00
#define Y_PURPLE_4                  0x551A8B00
#define Y_MEDIUMPURPLE1             0xAB82FF00
#define Y_MEDIUM_PURPLE_1           0xAB82FF00
#define Y_MEDIUMPURPLE2             0x9F79EE00
#define Y_MEDIUM_PURPLE_2           0x9F79EE00
#define Y_MEDIUMPURPLE3             0x8968CD00
#define Y_MEDIUM_PURPLE_3           0x8968CD00
#define Y_MEDIUMPURPLE4             0x5D478B00
#define Y_MEDIUM_PURPLE_4           0x5D478B00
#define Y_THISTLE1                  0xFFE1FF00
#define Y_THISTLE_1                 0xFFE1FF00
#define Y_THISTLE2                  0xEED2EE00
#define Y_THISTLE_2                 0xEED2EE00
#define Y_THISTLE3                  0xCDB5CD00
#define Y_THISTLE_3                 0xCDB5CD00
#define Y_THISTLE4                  0x8B7B8B00
#define Y_THISTLE_4                 0x8B7B8B00
#define Y_GRAY0                     0x00000000
#define Y_GRAY_0                    0x00000000
#define Y_GREY0                     0x00000000
#define Y_GREY_0                    0x00000000
#define Y_GRAY1                     0x03030300
#define Y_GRAY_1                    0x03030300
#define Y_GREY1                     0x03030300
#define Y_GREY_1                    0x03030300
#define Y_GRAY2                     0x05050500
#define Y_GRAY_2                    0x05050500
#define Y_GREY2                     0x05050500
#define Y_GREY_2                    0x05050500
#define Y_GRAY3                     0x08080800
#define Y_GRAY_3                    0x08080800
#define Y_GREY3                     0x08080800
#define Y_GREY_3                    0x08080800
#define Y_GRAY4                     0x0A0A0A00
#define Y_GRAY_4                    0x0A0A0A00
#define Y_GREY4                     0x0A0A0A00
#define Y_GREY_4                    0x0A0A0A00
#define Y_GRAY5                     0x0D0D0D00
#define Y_GRAY_5                    0x0D0D0D00
#define Y_GREY5                     0x0D0D0D00
#define Y_GREY_5                    0x0D0D0D00
#define Y_GRAY6                     0x0F0F0F00
#define Y_GRAY_6                    0x0F0F0F00
#define Y_GREY6                     0x0F0F0F00
#define Y_GREY_6                    0x0F0F0F00
#define Y_GRAY7                     0x12121200
#define Y_GRAY_7                    0x12121200
#define Y_GREY7                     0x12121200
#define Y_GREY_7                    0x12121200
#define Y_GRAY8                     0x14141400
#define Y_GRAY_8                    0x14141400
#define Y_GREY8                     0x14141400
#define Y_GREY_8                    0x14141400
#define Y_GRAY9                     0x17171700
#define Y_GRAY_9                    0x17171700
#define Y_GREY9                     0x17171700
#define Y_GREY_9                    0x17171700
#define Y_GRAY10                    0x1A1A1A00
#define Y_GRAY_10                   0x1A1A1A00
#define Y_GREY10                    0x1A1A1A00
#define Y_GREY_10                   0x1A1A1A00
#define Y_GRAY11                    0x1C1C1C00
#define Y_GRAY_11                   0x1C1C1C00
#define Y_GREY11                    0x1C1C1C00
#define Y_GREY_11                   0x1C1C1C00
#define Y_GRAY12                    0x1F1F1F00
#define Y_GRAY_12                   0x1F1F1F00
#define Y_GREY12                    0x1F1F1F00
#define Y_GREY_12                   0x1F1F1F00
#define Y_GRAY13                    0x21212100
#define Y_GRAY_13                   0x21212100
#define Y_GREY13                    0x21212100
#define Y_GREY_13                   0x21212100
#define Y_GRAY14                    0x24242400
#define Y_GRAY_14                   0x24242400
#define Y_GREY14                    0x24242400
#define Y_GREY_14                   0x24242400
#define Y_GRAY15                    0x26262600
#define Y_GRAY_15                   0x26262600
#define Y_GREY15                    0x26262600
#define Y_GREY_15                   0x26262600
#define Y_GRAY16                    0x29292900
#define Y_GRAY_16                   0x29292900
#define Y_GREY16                    0x29292900
#define Y_GREY_16                   0x29292900
#define Y_GRAY17                    0x2B2B2B00
#define Y_GRAY_17                   0x2B2B2B00
#define Y_GREY17                    0x2B2B2B00
#define Y_GREY_17                   0x2B2B2B00
#define Y_GRAY18                    0x2E2E2E00
#define Y_GRAY_18                   0x2E2E2E00
#define Y_GREY18                    0x2E2E2E00
#define Y_GREY_18                   0x2E2E2E00
#define Y_GRAY19                    0x30303000
#define Y_GRAY_19                   0x30303000
#define Y_GREY19                    0x30303000
#define Y_GREY_19                   0x30303000
#define Y_GRAY20                    0x33333300
#define Y_GRAY_20                   0x33333300
#define Y_GREY20                    0x33333300
#define Y_GREY_20                   0x33333300
#define Y_GRAY21                    0x36363600
#define Y_GRAY_21                   0x36363600
#define Y_GREY21                    0x36363600
#define Y_GREY_21                   0x36363600
#define Y_GRAY22                    0x38383800
#define Y_GRAY_22                   0x38383800
#define Y_GREY22                    0x38383800
#define Y_GREY_22                   0x38383800
#define Y_GRAY23                    0x3B3B3B00
#define Y_GRAY_23                   0x3B3B3B00
#define Y_GREY23                    0x3B3B3B00
#define Y_GREY_23                   0x3B3B3B00
#define Y_GRAY24                    0x3D3D3D00
#define Y_GRAY_24                   0x3D3D3D00
#define Y_GREY24                    0x3D3D3D00
#define Y_GREY_24                   0x3D3D3D00
#define Y_GRAY25                    0x40404000
#define Y_GRAY_25                   0x40404000
#define Y_GREY25                    0x40404000
#define Y_GREY_25                   0x40404000
#define Y_GRAY26                    0x42424200
#define Y_GRAY_26                   0x42424200
#define Y_GREY26                    0x42424200
#define Y_GREY_26                   0x42424200
#define Y_GRAY27                    0x45454500
#define Y_GRAY_27                   0x45454500
#define Y_GREY27                    0x45454500
#define Y_GREY_27                   0x45454500
#define Y_GRAY28                    0x47474700
#define Y_GRAY_28                   0x47474700
#define Y_GREY28                    0x47474700
#define Y_GREY_28                   0x47474700
#define Y_GRAY29                    0x4A4A4A00
#define Y_GRAY_29                   0x4A4A4A00
#define Y_GREY29                    0x4A4A4A00
#define Y_GREY_29                   0x4A4A4A00
#define Y_GRAY30                    0x4D4D4D00
#define Y_GRAY_30                   0x4D4D4D00
#define Y_GREY30                    0x4D4D4D00
#define Y_GREY_30                   0x4D4D4D00
#define Y_GRAY31                    0x4F4F4F00
#define Y_GRAY_31                   0x4F4F4F00
#define Y_GREY31                    0x4F4F4F00
#define Y_GREY_31                   0x4F4F4F00
#define Y_GRAY32                    0x52525200
#define Y_GRAY_32                   0x52525200
#define Y_GREY32                    0x52525200
#define Y_GREY_32                   0x52525200
#define Y_GRAY33                    0x54545400
#define Y_GRAY_33                   0x54545400
#define Y_GREY33                    0x54545400
#define Y_GREY_33                   0x54545400
#define Y_GRAY34                    0x57575700
#define Y_GRAY_34                   0x57575700
#define Y_GREY34                    0x57575700
#define Y_GREY_34                   0x57575700
#define Y_GRAY35                    0x59595900
#define Y_GRAY_35                   0x59595900
#define Y_GREY35                    0x59595900
#define Y_GREY_35                   0x59595900
#define Y_GRAY36                    0x5C5C5C00
#define Y_GRAY_36                   0x5C5C5C00
#define Y_GREY36                    0x5C5C5C00
#define Y_GREY_36                   0x5C5C5C00
#define Y_GRAY37                    0x5E5E5E00
#define Y_GRAY_37                   0x5E5E5E00
#define Y_GREY37                    0x5E5E5E00
#define Y_GREY_37                   0x5E5E5E00
#define Y_GRAY38                    0x61616100
#define Y_GRAY_38                   0x61616100
#define Y_GREY38                    0x61616100
#define Y_GREY_38                   0x61616100
#define Y_GRAY39                    0x63636300
#define Y_GRAY_39                   0x63636300
#define Y_GREY39                    0x63636300
#define Y_GREY_39                   0x63636300
#define Y_GRAY40                    0x66666600
#define Y_GRAY_40                   0x66666600
#define Y_GREY40                    0x66666600
#define Y_GREY_40                   0x66666600
#define Y_GRAY41                    0x69696900
#define Y_GRAY_41                   0x69696900
#define Y_GREY41                    0x69696900
#define Y_GREY_41                   0x69696900
#define Y_GRAY42                    0x6B6B6B00
#define Y_GRAY_42                   0x6B6B6B00
#define Y_GREY42                    0x6B6B6B00
#define Y_GREY_42                   0x6B6B6B00
#define Y_GRAY43                    0x6E6E6E00
#define Y_GRAY_43                   0x6E6E6E00
#define Y_GREY43                    0x6E6E6E00
#define Y_GREY_43                   0x6E6E6E00
#define Y_GRAY44                    0x70707000
#define Y_GRAY_44                   0x70707000
#define Y_GREY44                    0x70707000
#define Y_GREY_44                   0x70707000
#define Y_GRAY45                    0x73737300
#define Y_GRAY_45                   0x73737300
#define Y_GREY45                    0x73737300
#define Y_GREY_45                   0x73737300
#define Y_GRAY46                    0x75757500
#define Y_GRAY_46                   0x75757500
#define Y_GREY46                    0x75757500
#define Y_GREY_46                   0x75757500
#define Y_GRAY47                    0x78787800
#define Y_GRAY_47                   0x78787800
#define Y_GREY47                    0x78787800
#define Y_GREY_47                   0x78787800
#define Y_GRAY48                    0x7A7A7A00
#define Y_GRAY_48                   0x7A7A7A00
#define Y_GREY48                    0x7A7A7A00
#define Y_GREY_48                   0x7A7A7A00
#define Y_GRAY49                    0x7D7D7D00
#define Y_GRAY_49                   0x7D7D7D00
#define Y_GREY49                    0x7D7D7D00
#define Y_GREY_49                   0x7D7D7D00
#define Y_GRAY50                    0x7F7F7F00
#define Y_GRAY_50                   0x7F7F7F00
#define Y_GREY50                    0x7F7F7F00
#define Y_GREY_50                   0x7F7F7F00
#define Y_GRAY51                    0x82828200
#define Y_GRAY_51                   0x82828200
#define Y_GREY51                    0x82828200
#define Y_GREY_51                   0x82828200
#define Y_GRAY52                    0x85858500
#define Y_GRAY_52                   0x85858500
#define Y_GREY52                    0x85858500
#define Y_GREY_52                   0x85858500
#define Y_GRAY53                    0x87878700
#define Y_GRAY_53                   0x87878700
#define Y_GREY53                    0x87878700
#define Y_GREY_53                   0x87878700
#define Y_GRAY54                    0x8A8A8A00
#define Y_GRAY_54                   0x8A8A8A00
#define Y_GREY54                    0x8A8A8A00
#define Y_GREY_54                   0x8A8A8A00
#define Y_GRAY55                    0x8C8C8C00
#define Y_GRAY_55                   0x8C8C8C00
#define Y_GREY55                    0x8C8C8C00
#define Y_GREY_55                   0x8C8C8C00
#define Y_GRAY56                    0x8F8F8F00
#define Y_GRAY_56                   0x8F8F8F00
#define Y_GREY56                    0x8F8F8F00
#define Y_GREY_56                   0x8F8F8F00
#define Y_GRAY57                    0x91919100
#define Y_GRAY_57                   0x91919100
#define Y_GREY57                    0x91919100
#define Y_GREY_57                   0x91919100
#define Y_GRAY58                    0x94949400
#define Y_GRAY_58                   0x94949400
#define Y_GREY58                    0x94949400
#define Y_GREY_58                   0x94949400
#define Y_GRAY59                    0x96969600
#define Y_GRAY_59                   0x96969600
#define Y_GREY59                    0x96969600
#define Y_GREY_59                   0x96969600
#define Y_GRAY60                    0x99999900
#define Y_GRAY_60                   0x99999900
#define Y_GREY60                    0x99999900
#define Y_GREY_60                   0x99999900
#define Y_GRAY61                    0x9C9C9C00
#define Y_GRAY_61                   0x9C9C9C00
#define Y_GREY61                    0x9C9C9C00
#define Y_GREY_61                   0x9C9C9C00
#define Y_GRAY62                    0x9E9E9E00
#define Y_GRAY_62                   0x9E9E9E00
#define Y_GREY62                    0x9E9E9E00
#define Y_GREY_62                   0x9E9E9E00
#define Y_GRAY63                    0xA1A1A100
#define Y_GRAY_63                   0xA1A1A100
#define Y_GREY63                    0xA1A1A100
#define Y_GREY_63                   0xA1A1A100
#define Y_GRAY64                    0xA3A3A300
#define Y_GRAY_64                   0xA3A3A300
#define Y_GREY64                    0xA3A3A300
#define Y_GREY_64                   0xA3A3A300
#define Y_GRAY65                    0xA6A6A600
#define Y_GRAY_65                   0xA6A6A600
#define Y_GREY65                    0xA6A6A600
#define Y_GREY_65                   0xA6A6A600
#define Y_GRAY66                    0xA8A8A800
#define Y_GRAY_66                   0xA8A8A800
#define Y_GREY66                    0xA8A8A800
#define Y_GREY_66                   0xA8A8A800
#define Y_GRAY67                    0xABABAB00
#define Y_GRAY_67                   0xABABAB00
#define Y_GREY67                    0xABABAB00
#define Y_GREY_67                   0xABABAB00
#define Y_GRAY68                    0xADADAD00
#define Y_GRAY_68                   0xADADAD00
#define Y_GREY68                    0xADADAD00
#define Y_GREY_68                   0xADADAD00
#define Y_GRAY69                    0xB0B0B000
#define Y_GRAY_69                   0xB0B0B000
#define Y_GREY69                    0xB0B0B000
#define Y_GREY_69                   0xB0B0B000
#define Y_GRAY70                    0xB3B3B300
#define Y_GRAY_70                   0xB3B3B300
#define Y_GREY70                    0xB3B3B300
#define Y_GREY_70                   0xB3B3B300
#define Y_GRAY71                    0xB5B5B500
#define Y_GRAY_71                   0xB5B5B500
#define Y_GREY71                    0xB5B5B500
#define Y_GREY_71                   0xB5B5B500
#define Y_GRAY72                    0xB8B8B800
#define Y_GRAY_72                   0xB8B8B800
#define Y_GREY72                    0xB8B8B800
#define Y_GREY_72                   0xB8B8B800
#define Y_GRAY73                    0xBABABA00
#define Y_GRAY_73                   0xBABABA00
#define Y_GREY73                    0xBABABA00
#define Y_GREY_73                   0xBABABA00
#define Y_GRAY74                    0xBDBDBD00
#define Y_GRAY_74                   0xBDBDBD00
#define Y_GREY74                    0xBDBDBD00
#define Y_GREY_74                   0xBDBDBD00
#define Y_GRAY75                    0xBFBFBF00
#define Y_GRAY_75                   0xBFBFBF00
#define Y_GREY75                    0xBFBFBF00
#define Y_GREY_75                   0xBFBFBF00
#define Y_GRAY76                    0xC2C2C200
#define Y_GRAY_76                   0xC2C2C200
#define Y_GREY76                    0xC2C2C200
#define Y_GREY_76                   0xC2C2C200
#define Y_GRAY77                    0xC4C4C400
#define Y_GRAY_77                   0xC4C4C400
#define Y_GREY77                    0xC4C4C400
#define Y_GREY_77                   0xC4C4C400
#define Y_GRAY78                    0xC7C7C700
#define Y_GRAY_78                   0xC7C7C700
#define Y_GREY78                    0xC7C7C700
#define Y_GREY_78                   0xC7C7C700
#define Y_GRAY79                    0xC9C9C900
#define Y_GRAY_79                   0xC9C9C900
#define Y_GREY79                    0xC9C9C900
#define Y_GREY_79                   0xC9C9C900
#define Y_GRAY80                    0xCCCCCC00
#define Y_GRAY_80                   0xCCCCCC00
#define Y_GREY80                    0xCCCCCC00
#define Y_GREY_80                   0xCCCCCC00
#define Y_GRAY81                    0xCFCFCF00
#define Y_GRAY_81                   0xCFCFCF00
#define Y_GREY81                    0xCFCFCF00
#define Y_GREY_81                   0xCFCFCF00
#define Y_GRAY82                    0xD1D1D100
#define Y_GRAY_82                   0xD1D1D100
#define Y_GREY82                    0xD1D1D100
#define Y_GREY_82                   0xD1D1D100
#define Y_GRAY83                    0xD4D4D400
#define Y_GRAY_83                   0xD4D4D400
#define Y_GREY83                    0xD4D4D400
#define Y_GREY_83                   0xD4D4D400
#define Y_GRAY84                    0xD6D6D600
#define Y_GRAY_84                   0xD6D6D600
#define Y_GREY84                    0xD6D6D600
#define Y_GREY_84                   0xD6D6D600
#define Y_GRAY85                    0xD9D9D900
#define Y_GRAY_85                   0xD9D9D900
#define Y_GREY85                    0xD9D9D900
#define Y_GREY_85                   0xD9D9D900
#define Y_GRAY86                    0xDBDBDB00
#define Y_GRAY_86                   0xDBDBDB00
#define Y_GREY86                    0xDBDBDB00
#define Y_GREY_86                   0xDBDBDB00
#define Y_GRAY87                    0xDEDEDE00
#define Y_GRAY_87                   0xDEDEDE00
#define Y_GREY87                    0xDEDEDE00
#define Y_GREY_87                   0xDEDEDE00
#define Y_GRAY88                    0xE0E0E000
#define Y_GRAY_88                   0xE0E0E000
#define Y_GREY88                    0xE0E0E000
#define Y_GREY_88                   0xE0E0E000
#define Y_GRAY89                    0xE3E3E300
#define Y_GRAY_89                   0xE3E3E300
#define Y_GREY89                    0xE3E3E300
#define Y_GREY_89                   0xE3E3E300
#define Y_GRAY90                    0xE5E5E500
#define Y_GRAY_90                   0xE5E5E500
#define Y_GREY90                    0xE5E5E500
#define Y_GREY_90                   0xE5E5E500
#define Y_GRAY91                    0xE8E8E800
#define Y_GRAY_91                   0xE8E8E800
#define Y_GREY91                    0xE8E8E800
#define Y_GREY_91                   0xE8E8E800
#define Y_GRAY92                    0xEBEBEB00
#define Y_GRAY_92                   0xEBEBEB00
#define Y_GREY92                    0xEBEBEB00
#define Y_GREY_92                   0xEBEBEB00
#define Y_GRAY93                    0xEDEDED00
#define Y_GRAY_93                   0xEDEDED00
#define Y_GREY93                    0xEDEDED00
#define Y_GREY_93                   0xEDEDED00
#define Y_GRAY94                    0xF0F0F000
#define Y_GRAY_94                   0xF0F0F000
#define Y_GREY94                    0xF0F0F000
#define Y_GREY_94                   0xF0F0F000
#define Y_GRAY95                    0xF2F2F200
#define Y_GRAY_95                   0xF2F2F200
#define Y_GREY95                    0xF2F2F200
#define Y_GREY_95                   0xF2F2F200
#define Y_GRAY96                    0xF5F5F500
#define Y_GRAY_96                   0xF5F5F500
#define Y_GREY96                    0xF5F5F500
#define Y_GREY_96                   0xF5F5F500
#define Y_GRAY97                    0xF7F7F700
#define Y_GRAY_97                   0xF7F7F700
#define Y_GREY97                    0xF7F7F700
#define Y_GREY_97                   0xF7F7F700
#define Y_GRAY98                    0xFAFAFA00
#define Y_GRAY_98                   0xFAFAFA00
#define Y_GREY98                    0xFAFAFA00
#define Y_GREY_98                   0xFAFAFA00
#define Y_GRAY99                    0xFCFCFC00
#define Y_GRAY_99                   0xFCFCFC00
#define Y_GREY99                    0xFCFCFC00
#define Y_GREY_99                   0xFCFCFC00
#define Y_GRAY100                   0xFFFFFF00
#define Y_GRAY_100                  0xFFFFFF00
#define Y_GREY100                   0xFFFFFF00
#define Y_GREY_100                  0xFFFFFF00
#define Y_DARKGREY                  0xA9A9A900
#define Y_DARK_GREY                 0xA9A9A900
#define Y_DARKGRAY                  0xA9A9A900
#define Y_DARK_GRAY                 0xA9A9A900
#define Y_DARKBLUE                  0x00008B00
#define Y_DARK_BLUE                 0x00008B00
#define Y_DARKCYAN                  0x008B8B00
#define Y_DARK_CYAN                 0x008B8B00
#define Y_DARKMAGENTA               0x8B008B00
#define Y_DARK_MAGENTA              0x8B008B00
#define Y_DARKRED                   0x8B000000
#define Y_DARK_RED                  0x8B000000
#define Y_LIGHTGREEN                0x90EE9000
#define Y_LIGHT_GREEN               0x90EE9000

